/* Generated by Together */

package ap.data;

import ap.data.CAccountList;

/**
 * Title:        Ametas StockService
 * Description:  Datenhaltungsklasse fr eine im Service registrierten Benutzeraccount
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CAccount implements java.io.Serializable {

	/** Einige real-world-Membervariablen der Klasse */
	private String m_sUsername;
	private double m_dStartedBalance;
	private double m_dCurrentBalance;
	private CDepot m_objDepot;



	/**
	 * Der Konstruktor der Klasse erzeugt einen neuen CAccount-Datensatz mit
	 * den bergebenen Parametern wir Benutzername, initalier Kontostand und
	 * legt ein neues Depot fr den Account an.
	 *
	 * @param sUsername Der Benutzername des Accounts
	 * @param dStartedBalance Der initiale Kontostand des Accounts
	 */
	public CAccount (String sUsername, double dStartedBalance) {

		/* bernehmen der bergebenen Parameter in die Memebervariablen */
		m_sUsername = sUsername;
		m_dStartedBalance = dStartedBalance;
		m_dCurrentBalance = dStartedBalance;

		/* Neues Objekt vom typ CDepot anlegen */
		m_objDepot = new CDepot ();

	}

	/**
	 * Liefert den Wert der Membervariable m_sUsername zurck, die den Benutezrnamen
	 * des Accounts enthlt.
	 *
	 * @return Der Benutzername des Accounts
	 */
	public String getUsername () {
		return m_sUsername;
	}

	/**
	 * Setzt den Wert der Membervariable m_sUsername, die den Benutzernamen des
	 * Accounts aufnimmt.
	 *
	 * @param sUsername Der neue Benutzername des Accounts
	 */
	public void setName (String sUsername) {
		m_sUsername = sUsername;
	}

	/**
	 * Liefert den Wert der Membervariable m_dCurrentBalance zurck, die
	 * den aktuellen Kontostand des Accounts enthlt.
	 *
	 * @return Der aktuelle Kontostand des Accounts
	 */
	public double getCurrentBalance () {
		return m_dCurrentBalance;
	}

	/**
	 * Setzt den Wert der Membervariable m_dCurrentBalance, die den aktuellen
	 * Kontostand des Accounts aufnimmt.
	 *
	 * @param dCurrentBalance Der neue, aktuelle Kontostand des Accounts
	 */
	public void setCurrentBalance (double dCurrentBalance) {
		m_dCurrentBalance = dCurrentBalance;
	}

	/**
	 * Gibt das Depot des Accounts zurck (gespeichert in der der Membervariable
	 * m_objDepot zurck).
	 *
	 * @return Das Depot des Accounts
	 */
	public CDepot getDepot () {
		return m_objDepot;
	}

	/**
	 * Neuen Depoteintrag zu Accountdepot hinzufgen und Kaufpreis abbuchen
	 *
	 * @param objDepotEntry der zu buchende Depoteintrag
	 */
        public void bookDepotEntry (CDepotEntry objDepotEntry) {

		/* Depoteintrag in Depot einfgen */
		m_objDepot.addElement (objDepotEntry);

		/* Kaufpreis von m_dCurrentBalance abbuchen */
		m_dCurrentBalance -= (double)(objDepotEntry.getPriceBought() * objDepotEntry.getAmount());

	}

	/**
	 * Neuen Depoteintrag zu Accountdepot hinzufgen und Kaufpreis abbuchen
	 *
	 * @param objStock der Datensatz der zu verkaufenden Aktie
	 * @param iAmount die Anzahl der zu verkaufenden Aktien, des in objStock angegebenen Typs
	 * @param iActuallySold die Anzahl der tatschlich verkauften Aktien
	 * @return der tatschlich erzielte Verkaufsgesamtpreis, der dem Konto gutgeschrieben wurde
	 */
        public double sellStocks (CStock objStock, int iAmount, int iActuallySold) {

		/* Die Variable speichert den Gesamtpreis der verkauften Aktien */
		double dTotalValue = 0.0;

		/* Der Name der zu verkaufenden Aktie */
		String sStockname = objStock.getName();

		/* Anzahl der zu verkaufenden Aktien in Laufvariable halten */
		int iStillToSellAmount = iAmount;

		/* Flag gibt an, ob noch Datenstze der gewnschten Aktie vorhanden sind */
		boolean bStocksAvailable = true;

		/* So lange Aktien verkaufen und bei Bedarf Eintrge aus dem Depot entfernen,
		bis die gewnschte Anzahl an Aktien verkauft wurde. */
		while ((iStillToSellAmount > 0) && (bStocksAvailable)) {

			/* 1. Depoteintrag mit niedrigstem Kaufpreis ermitteln */
	        	CDepotEntry objLowestDepEntry = null;
		        bStocksAvailable = false;
		        for (int i = 0; i < m_objDepot.size(); i++) {

		                /* aktuellen Depoteintrag in Zwischenspeicher legen */
			        CDepotEntry objCurDepEntry = m_objDepot.getDepotEntry(i);

	        	        /* Alle DepotEintrge mit sStockname als Aktiennamen prfen */
		                if (objCurDepEntry.getName().equals(sStockname)) {

				        /* Den DepotEintrag mit dem niedrigsten Kaufpreis bei Bedarf aktualisieren */
			                if (objLowestDepEntry == null) {
	        		        	objLowestDepEntry = objCurDepEntry;
			        	} else if (objCurDepEntry.getPriceBought() < objLowestDepEntry.getPriceBought()) {
	        			        objLowestDepEntry = objCurDepEntry;
		        		}

				        /* Es sind noch Datenstze mit Aktie sStockname vorhanden */
					bStocksAvailable = true;

			        }

	        	}

			/* 2. Versuchen, die Anzahl der noch zu verkaufenden Aktien aus
	        	diesem Eintrag zu verkaufen (iStillToStellAmount) */
			if (objLowestDepEntry.getAmount() > iStillToSellAmount) {

	        	        /* Aktien verkaufen: Amount-Variable in DepotEntry veringern */
		                objLowestDepEntry.setAmount(objLowestDepEntry.getAmount() - iStillToSellAmount);

			        /* Verkaufserls zu Zwischenspeicher hinzuaddieren */
	        		dTotalValue += (double)(iStillToSellAmount * objLowestDepEntry.getPriceCurrent());

	        	        /* Es mu nichts mehr verkauft werden, da dieser Depoteintrag
		                den Verkaufswunsch befriedigen konnte */
		                iStillToSellAmount = 0;

			} else if (objLowestDepEntry.getAmount() <= iStillToSellAmount) {

		                /* Aktien verkaufen: Amount-Variable in DepotEntry veringern */
	        	        int iCurDepAmount = objLowestDepEntry.getAmount();
		                objLowestDepEntry.setAmount(iCurDepAmount);

			        /* Verkaufserls zu Zwischenspeicher hinzuaddieren */
	        		dTotalValue += (double)(iCurDepAmount * objLowestDepEntry.getPriceCurrent());

		                /* Die Anzahl der zu verkaufenden Aktien um die aus diesem Depoteintrag
		                verkaufte reduzieren */
			        iStillToSellAmount -= iCurDepAmount;

	        	        /* Den Depoteintrag auflsen: aus dem Depot entfernen */
		                m_objDepot.removeElement(objLowestDepEntry);

			}

		}

		/* Wenn die Schleife beendet wurde und noch Aktien zu verkaufen wren
		(iStillToSellAmount > 0), dann wurden keine passenden Depoteintrge mehr
		gefunden, sprich: der Auftraggeber wollte mehr verkaufen, als er besa.
		Die Anzahl der tatschlich verkauften Aktien wird daher in folgender
		Variable berechnet und zurckgegeben. Wurden alle Aktien verkauft, dann
		wird die folgende Variable == iAmount sein. */
		iActuallySold = iAmount -  iStillToSellAmount;

		/* dTotalValue auf Konto gutschreiben */
		m_dCurrentBalance += dTotalValue;

		return dTotalValue;
	}

}
