/* Generated by Together */

package ap.data;

import java.util.*;
import java.io.Serializable;

/**
 * Title:        CDepot
 * Description:  Collection von CDepotEntrys, die ein zu einem Benutzeraccount gehriges
 *               Aktiendepot reprsentieren.
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CDepot extends Vector implements java.io.Serializable, AMETAS.data.AMETASSendable {

	/** Einige Membervariablen der Instanz, die das Depot beschreiben */
	Date m_dateLastUpdated = null;          // das Datum der letzten Datenaktualisierung
	Double m_dAccountBalance = null;        // aktueller Kontostand
	Double m_dAccountBalanceInitial = null; // der Kontostand bei Aussetzung des Agenten
	Double m_dTotalPriceBought = null;      // summierter Kaufpreis aller Aktien im Depot
	Double m_dTotalPriceCurrent = null;     // summierter aktueller Wert aller Aktien im Depot
	Integer m_iTotalAmount = null;          // Gesamtanzahl der Aktien im Depot
	int m_iTendency = 1;                    // 0 = not set, 1 = rising, 2 = falling, 3 = stagnating


	/**
	 * Konstruktor
	 */
	public CDepot () {

		/* Elternklassenkonstruktor Vector() aufrufen */
		super ();

		/* DEBUG: Zufallswerte fr die Membervariablen generieren */
		/*
		m_dateLastUpdated = new Date();
		long x = m_dateLastUpdated.getTime();

		Random objRandom = new Random( x );

		m_dAccountBalance = new Double (objRandom.nextDouble()*1000);
		m_dTotalPriceBought = new Double (objRandom.nextDouble()*1000);
	        m_dTotalPriceCurrent = new Double (objRandom.nextDouble()*1000);
		m_dGainCurrent = new Double (objRandom.nextDouble()*1000);
	        m_dGainOverall = new Double (objRandom.nextDouble()*1000);
		m_iTotalAmount = new Integer (objRandom.nextInt(500));
		m_iTendency = objRandom.nextInt(3)+1;
		*/

		/* Objekte fr entsprechende Membervariablen anlegen */
		m_dAccountBalance = new Double (0.0);
		m_dAccountBalanceInitial = new Double (0.0);
		m_dTotalPriceBought = new Double (0.0);
	        m_dTotalPriceCurrent = new Double (0.0);
		m_iTotalAmount = new Integer (0);
		m_iTendency = 0;
		m_dateLastUpdated = new Date();

	}

	/**
	 * Die Methode liefert das i-te im Depot gespeicherte CDepotEntry-Objekt zurck.
	 *
	 * @param i Das i-te CDepotEntry-Objekt in der Liste
	 */
	public CDepotEntry getDepotEntry (int i) {

		/* Liefert das getypcastete i-te Objekt des Vectors zurck */
		return (CDepotEntry)get(i);

	}

	/**
	 * Die Methode sucht das CDepotEntry-Objekt in der Liste, dass sName als
	 * Aktiennamen hat und liefert es zurck.
	 *
	 * @param sName Der Aktienmame des in der Liste zu suchenden CDepotEntry-Objekts
	 */
	public CDepotEntry getElementByName (String sName) {

		/* Den Vector durchgehen und sName mit dem sName-Attribut jedes
		CDepotEntry-Datensatzes vergleichen */
		for (int i = 0; i < size(); i++) {
		        CDepotEntry objCurEntry = getDepotEntry(i);

		        if (objCurEntry.getName().equals(sName)) {
			        /* Liefert das getypcastete i-te Objekt des Vectors zurck */
		        	return objCurEntry;
		        }

		}

		/* Wenn kein matchendes Objekt gefunden wurde, dann null zurckliefern */
		return null;
	}

	/**
	 * Liefert das Datum der letzten Aktualisierung dieses Depots zurck (zu beachten
	 * ist, dass dieser Wert nicht fr jede Operation auf dem Objekt aktualisiert wird!)
	 */
	public Date getDateLastUpdated () {
		return m_dateLastUpdated;
	}

	/**
	 * Liefert den aktuellen Kontostand des Depots zurck
	 */
	public Double getAccountBalance () {
		return m_dAccountBalance;
	}

	/**
	 * Setz den aktuellen Kontostand des Depots
	 */
	public void setAccountBalance (Double dAccountBalance) {
		m_dAccountBalance = dAccountBalance;
	}

	/**
	 * Liefert den initialen Kontostand des Depots zurck
	 */
	public Double getAccountBalanceInitial () {
		return m_dAccountBalanceInitial;
	}

	/**
	 * Setz den initialen Kontostand des Depots (sollte man ja eigentlich nur einmal knnen :)
	 */
	public void setAccountBalanceInitial (Double dAccountBalanceInitial) {
		m_dAccountBalanceInitial = dAccountBalanceInitial;
	}

	/**
	 * Liefert den Gesamtkaufpreis aller im Depot verwalteten Aktien zurck
	 */
	public Double getTotalPriceBought () {
		return m_dTotalPriceBought;
	}

	/**
	 * Liefert den aktuellen Gesamtwert der im Depot verwalteten Aktien zurck
	 * -> Summe ber alle Aktientitel (Kurs des Aktientitels * Aktienanzahl)
	 */
	public Double getTotalPriceCurrent () {
		return m_dTotalPriceCurrent;
	}

	/**
	 * Liefert den mit den im Depot verwalteten Aktien erzielten Gewinn/Verlust zurck
	 */
	public Double getGainCurrent () {
                return new Double( m_dTotalPriceCurrent.doubleValue() - m_dTotalPriceBought.doubleValue() );
	}

	/**
	 * Liefert den Gesamtgewinn/-verlust zurck (m_dAccountBalance + m_dTotalPriceBought)
	 */
	public Double getGainOverall () {
		return new Double( m_dAccountBalance.doubleValue() + m_dTotalPriceBought.doubleValue() );
	}

	/**
	 * Liefert die Gesamtzahl der im Depot verwalteten Aktien (aktiven!) zurck.
	 */
	public Integer getTotalAmount () {
		return m_iTotalAmount;
	}

	/**
	 * Gibt die Tendenz des Depots zurck (steigend, fallend, stagnierend :)
	 */
	public int getTendency () {
		return m_iTendency;
	}

        /**
         * Aktualisiert alle "berechneten" Angaben des Depots, wie z.B.
         * den Gesamtwert, Gesamtpreis, etc. Diese Funktion sollte nach
         * nderungen am Depot aufgerufen werden.
         */
        public void updateDepotInformation() {

                double dDepotPrice = 0.0;
                double dDepotValue = 0.0;
                int iStocksCount = 0;

                /* Fabian Wleklinski: Durch alle Aktien des Depots iterieren: */
                for (int i=0; i < size(); i++) {

                        /* Fabian Wleklinski: Einen Depoteintrag laden */
                        CDepotEntry deCurEntry = getDepotEntry(i);

                        /* Fabian Wleklinski: Preis und Wert des Depots
                        sowie Anzahl der Depoteintrge aktualisieren: */
                        dDepotPrice = dDepotPrice + (deCurEntry.getPriceBought() * deCurEntry.getAmount());
                        dDepotValue = dDepotValue + (deCurEntry.getPriceCurrent() * deCurEntry.getAmount());
                        iStocksCount = iStocksCount + deCurEntry.getAmount();

                }

		/* alten "aktuell realisierten Gewinn" zwischenspeichern, um Tendenzflag zu bestimmen */
		double dGainCurrentOld = getGainCurrent().doubleValue();

		/* die kumulierten Werte in die Membervariablen schreiben */
                m_dTotalPriceBought = new Double (dDepotPrice);
                m_dTotalPriceCurrent = new Double (dDepotValue);
                m_iTotalAmount = new Integer (iStocksCount);

		/* Wenn der neue "aktuell realisierte Gewinn" nur um +/- 5% vom alten abweicht,
		wird die Tendenz "stagnating" (3) gesetzt, bei < - 5% "falling" (2) und bei
		> 5% "rising" (1). */
		double dGainCurrentNew = getGainCurrent().doubleValue();

		if (dGainCurrentNew > dGainCurrentOld * 1.05) {
		        m_iTendency = 1; // rising
		} else if (dGainCurrentNew < dGainCurrentOld * 0.95) {
		        m_iTendency = 2; // falling
		} else {
			m_iTendency = 3; // stagnating
		}

		/* Jetzt setzen wir auch die Membervariable fr lastUpdated! */
		m_dateLastUpdated = new Date();

        }

	/**
	 * Fgt alle Depoteintrge des bergebenen Depots in die Instanz ein.
	 *
	 * @param depotSource Das Depot, dessen Depoteintrge bernommen werden sollen
	 */
	public void addFromDepot( CDepot depotSource ) {
		for (int i=0; i < depotSource.size(); i++) {
			addElement (depotSource.getDepotEntry(i));
		}
	}

        /**
         * Die Funktion &quot;getData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members in generische
         * Objekte zu verwandeln.
         */
        public Object[] getData() {

                /* Fabian Wleklinski: Es muss ein Feld von generischen Objekten zurckgegeben
                werden, dies fordert das "AMETASSendable"-Interface. Dieses Feld wird nun
                erzeugt: */
                Object[] aobjData = new Object[ 2 + this.size() ];

                aobjData[0] = m_dateLastUpdated;
                aobjData[1] = m_dAccountBalance;

                /* Fabian Wleklinski: Nun alle Daten/Variablen/Members in dieses Feld
                schreiben: */
                for (int i=0; i < size(); i++) {
                        CDepotEntry deCurEntry = getDepotEntry(i);
                        Object[] aobjCurData = deCurEntry.getData();
                        aobjData[ i+2 ] = aobjCurData;
                }

                /* Fabian Wleklinski: Nun dieses Feld zurckgeben: */
                return aobjData;

        }

        /**
         * Die Funktion &quot;setData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members aus generischen
         * Objekten zu lesen.
         */
        public void setData( Object[] aobjData ) {

                /* Fabian Wleklinski: Nun alle Daten/Variablen/Members aus dem
                bergebenen Feld lesen, und in die entsprechenden Variablen eintragen: */
		m_dateLastUpdated = (Date) aobjData[0];
                m_dAccountBalance = (Double) aobjData[1];

                for (int i=2; i < aobjData.length; i++) {
                        Object[] aobjCurData = (Object[]) aobjData[i];
                        CDepotEntry deCurEntry = new CDepotEntry(aobjCurData);
                        addElement (deCurEntry);
                }

        }

}


