package ap.data;

/**
 * Title:        Klasse CStockAgentMsgMove
 * Description:  Nachricht an einen StockAgent, der ihn zum Umbewegen auf
 *		 eine andere Stelle veranlasst
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CStockAgentMsgMove extends CStockAgentMsg implements AMETAS.data.AMETASSendable {

        /**
         * der Name der Stelle, zu der der StockAgent migrieren soll
         */
        private String m_sPlaceName;

        public CStockAgentMsgMove() {
        }

        /**
         * Konstruktor der Klasse
         * @param sPlaceName der Name der Stelle, zu der der StockAgent migrieren soll
         */
        public CStockAgentMsgMove( String sPlaceName ) {

                /* Fabian Wleklinski: Die bergebenen Parameter merken */
                this.m_sPlaceName = sPlaceName;
        }

        /**
         * Liefert den Namen der Stelle zurck, zu der sich der StockAgent bewegen soll
         */
        public String getPlaceName() {
                return m_sPlaceName;
        }

        /**
         * Setzt den Namen der Stelle, zu der sich der StockAgent bewegen soll
         * @param sPlaceName der Name der Stelle, zu der sich der StockAgent bewegen soll
         */
        public void setPlaceName( String sPlaceName ) {
                this.m_sPlaceName = sPlaceName;
        }

        /**
         * Die Funktion &quot;getData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members in generische
         * Objekte zu verwandeln.
         */
        public Object[] getData() {

                /* Fabian Wleklinski: Es muss ein Feld von generischen Objekten zurckgegeben
                werden, dies fordert das "AMETASSendable"-Interface. Dieses Feld wird nun
                erzeugt: */
                Object[] aobjData = new Object[ 1 ];

                /* Fabian Wleklinski: Nun alle Daten/Variablen/Members in dieses Feld
                schreiben: */
                aobjData[ 0 ] = m_sPlaceName;

                /* Fabian Wleklinski: Nun dieses Feld zurckgeben: */
                return aobjData;
        }

        /**
         * Die Funktion &quot;setData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members aus generischen
         * Objekten zu lesen.
         */
        public void setData( Object[] aobjData ) {

                /* Fabian Wleklinski: Nun alle Daten/Variablen/Members aus dem
                bergebenen Feld lesen, und in die entsprechenden Variablen eintragen: */
                m_sPlaceName = (String) aobjData[ 0 ];
        }

}
