package ap.data;

import java.util.*;

/**
 * Title:        Klasse CStockAgentMsgs
 * Description:  Collection von CStockAgentMsg, die jeweils eine Nachricht an einen
 *               StockAgent, oder von einem StockAgent darstellt.
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CStockAgentMsgs extends Vector implements java.io.Serializable, AMETAS.data.AMETASSendable {

        /**
         * Der Konstruktor der CStockAgentMsgs-Klasse
         */
        public CStockAgentMsgs(  ) {
        }

        /**
         * Der Konstruktor der CStockAgentMsgs-Klasse
         * @param objSerializedData Serialisierte Daten dieser Klasse, wie sie
         * von der Methode "getData()" erzeugt werden
         */
        public CStockAgentMsgs( Object[] objSerializedData ) {
                this.setData( objSerializedData );
        }

        /**
         * Die Methode liefert das i-te in der Liste gespeicherte CStockAgentMsg-Objekt zurck
         *
         * @param i Das i-te Objekt in der Liste
         */
        public CStockAgentMsg getMsg( int i ) {

                /* Liefert das getypcastete i-te CStockAgentMsg-Objekt des Vectors zurck */
                return (CStockAgentMsg) get( i );

        }

        /**
         * Die Funktion &quot;getData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members in generische
         * Objekte zu verwandeln.
         */
        public Object[] getData() {

                /* Fabian Wleklinski: Es muss ein Feld von generischen Objekten zurckgegeben
                werden, dies fordert das "AMETASSendable"-Interface. Dieses Feld wird nun
                erzeugt: */
                Object[] aobjData = new Object[ this.size()*3 ];

                /* Fabian Wleklinski: Nun durch alle enthaltenen Nachrichten iterieren und
                diese wiederum serialisieren */

                for (int i=0; i < this.size(); i++) {

                        /* Fabian Wleklinski: diese Nachricht serialisieren: */
                        String curClassName = this.getMsg( i ).getClass().getName();
                        Date curDate = this.getMsg( i ).getTimeCreated();
                        Object[] curMsg = this.getMsg( i ).getData();

                        /* Fabian Wleklinski: das von der aktuellen Nachricht erzeugte
                        Feld mit den bereits erzeugten Feldern zusammenfhren: */
                        aobjData[ i*3 ]		= curClassName;
                        aobjData[ i*3+1 ]	= curDate;
                        aobjData[ i*3+2 ]	= curMsg;

                }

                /* Fabian Wleklinski: Nun dieses Feld zurckgeben: */
                return aobjData;
        }

        /**
         * Die Funktion &quot;setData()&quot; wird vom Interface &quot;AMETASSendable&quot;
         * gefordert. Sie dient dazu, im Falle einer Serialisierung der Klasse (bei einer
         * Migration des Agenten) alle enthaltenen Daten/Variablen/Members aus generischen
         * Objekten zu lesen.
         */
        public void setData( Object[] aobjData ) {

                /* Fabian Wleklinski: Wir haben ein Feld mit generischen Objekten bergeben
                bekommen, von denen jedes einer Nachricht entspricht. Durch dieses Feld
                iterieren wir nun: */

                for (int i=0; i <= aobjData.length-3; i=i+3) {

                        CStockAgentMsg curMsg = null;

                        /* Fabian Wleklinski: Den Namen der Klasse lesen, deren
                        Daten nun folgen */
                        String curClassName = (String) aobjData[ i ];

                        /* Fabian Wleklinski: Je nach gelesenem Klassennamen eine
                        andere Instanz erzeugen */
                        if (curClassName.equals( "ap.data.CStockAgentMsgMove" )) {
                                curMsg = new CStockAgentMsgMove();
                        } else if (curClassName.equals( "ap.data.CStockAgentMsgExit" )) {
                                curMsg = new CStockAgentMsgExit();
                        } else if (curClassName.equals( "ap.data.CStockAgentMsgDepot" )) {
                                curMsg = new CStockAgentMsgDepot();
                        } else if (curClassName.equals( "ap.data.CStockAgentMsgStrategy" )) {
                                curMsg = new CStockAgentMsgStrategy();
                        } else {
                                System.out.println( "Unknown class: \"" + curClassName + "\"!" );
                        }

                        /* Fabian Wleklinski: Den TimeStamp der erzeugten
                        Nachricht restaurieren */
                        if (curMsg != null) {
                                curMsg.setTimeCreated((Date) aobjData[ i+1 ]);
                        }

                        /* Fabian Wleklinski: Nun die gelesenen Daten in die
                        erzeugte Instanz deserialisieren */
                        try {
                                if (curMsg != null) {
                                        curMsg.setData( (Object[]) aobjData[ i+2 ] );
                                }
                        } catch ( Exception curException ) {
                                System.out.println( "CStockAgentMsgs: Couldn't deserialize object data of class \"" + curClassName + "\"!\n" + curException.toString() );
                        }

                        if (curMsg != null) {
                                this.add( curMsg );
                        }
                }

        }

}
