/* Generated by Together */

package ap.data;

import AMETAS.servicedev.*;

import java.util.*;

/**
 * Title:        Ametas StockService
 * Description:  Collection von CStocks, die die gehandelten Aktien des Service reprsentieren. Diese
 *               Klasse ist als Runnable implementiert und besitzt mit der Run()-Methode einen
 *               Thread-gesteuerten Mechanismus zum Update der Atkienkurse.
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CStockList extends Vector implements java.io.Serializable, java.lang.Runnable {

	/** Die Anzahl der zu handelnden Wertpapiere (= m_aobjStocks.size()) */
	private int m_iCompanyAmount = 10;

	/** Die Anzahl von Sekunden zwischen zwei Kursupdates */
	private int m_iCalcIntervall = 5;

	/** Flag gibt an, ob die Kurse in run() berechnet werden sollen. Bei false
	wird der Updateloop in run() und damit der Thread beendet. */
	private boolean m_bUpdate = true;

	/** Der minimale und maximale Preis einer neu ausgegebenen Aktie */
	private double m_dMinIpoPrice = 0.0, m_dMaxIpoPrice = 50.0;

	/** Damit wir einigermaen zufllige Zahlen bekommen (von der Systemzeit
	zum Zeitpunkt der Objekterzeugung abhngig, diese Membervariable der
	Random-Klasse */
	private Random m_objRandom = null ;

	/** Der Treiber des instanzierenden Agenten oder Services */
	AMETAS.servicedev.AMETASServiceManagerDriverIf m_objDriver = null;



	/**
	 * Mit dem Konstruktor der CStockList-Klasse kann festgelegt werden, wie viele
	 * Aktien "gehandelt" werden sollen (iCompanyAmount) und in welchen Intervallen
	 * in Sekunden die Kurswerte geupdatet werden sollen (iCalcIntervall).
	 *
	 * @param iCompanyAmount Die Anzahl der zu handelnden Wertpapiere
	 * @param iCalcIntervall Die Anzahl der Sekunden zwischen zwei Kursberechnungen
	 */
	public CStockList (int iCompanyAmount, int iCalcIntervall) {

		/* Parameter in Membervariablen eintragen */
		m_iCompanyAmount = iCompanyAmount;
		m_iCalcIntervall = iCalcIntervall;

		/* Instanzweiten Zufallszahlengenerator anlegen */
		m_objRandom = new Random();

	}

	/**
	 * Mit diesem Konstruktor werden die Standardwerte fr die Anzahl der zu
	 * handelnden Aktien "gehandelt" (iCompanyAmount) und die Zeitintervalle
	 * fr Kursupdates (iCalcIntervall) bernommen.
	 */
	public CStockList () {

		/* Instanzweiten Zufallszahlengenerator anlegen */
		m_objRandom = new Random();

	}

	/**
	 * Die Methode liefert das i-te in der StockList gespeicherte CStock-Objekt zurck.
	 *
	 * @param i Das i-te CStock-Objekt in der Liste
	 */
	public CStock getStock (int i) {

		/* Liefert das getypcastete i-te Objekt des Vectors zurck */
		return (CStock)get(i);

	}

	/**
	 * Die Methode sucht das CStock-Objekt in der Liste, dass sName als Namen
	 * hat und liefert es zurck.
	 *
	 * @param sName Der Name des in der Liste zu suchenden CStock-Objekts
	 */
	public CStock getElementByName (String sName) {

		/* Den Vector durchgehen und sName mit dem sName-Attribut jedes
		CStock-Objekts vergleichen */
		for (int i = 0; i < size(); i++) {
		        CStock objCurrentStock = getStock(i);

		        if (objCurrentStock.getName().equals(sName)) {
			        /* Liefert das getypcastete i-te Objekt des Vectors zurck */
		        	return objCurrentStock;
		        }

		}

		/* Wenn kein matchendes Objekt gefunden wurde, dann null zurckliefern */
		return null;
	}

	/**
	 * Liefert ein Aktienkrzel fr ein neues CStock-Objekt. Dabei werden die
	 * in der Liste verwalteten CStock-Objekte und ihre Namen bercksichtigt,
	 * um einen eindeutigen neuen Namen zu erhalten
	 */
	private String createStockName () {

		/* Einen neuen String fr den Namen (3 Grobuchstaben) anlegen */
		String sNewStockName = "";

		/* Mit diesem Flag wird geregelt, ob nach weiteren, unbenutzten
		Namen gesucht werden mu (=> bUnique == false), damit keine
		neuen Aktien mit dem gleichen Krzel wie "bankrotten" Firmen
		emissioniert werden. */
		boolean bUnique = false;

		/* Die Anzahl der Namensfindungsversuche wird auf 26^3 beschrnkt. Eine
		solche Anzahl von Versuchen werden wir wohl nie erreichen, aber
		sicher ist sicher... wer wei, ob die Soft nicht mal nen Tag luft oder so */
		int iTrialCounter = 0;

		while ((bUnique == false) && (iTrialCounter < 17576)) {

			/* Zufallszahlengenerator anlegen fr 1-26 Zufallszahlen und
	        	3stelligen Namen (Grobuchstaben) generieren. */
	        	sNewStockName = "" + (char)(m_objRandom.nextInt(26)+65)
			                   + (char)(m_objRandom.nextInt(26)+65)
			                   + (char)(m_objRandom.nextInt(26)+65);

			/* Alle Objekte durchlaufen und auf Namensvertrglichkeit prfen */
		        bUnique = true;
	                for (int i = 0; i < size(); i++) {
				if (sNewStockName.equals( getStock(i).getName() )) {
				        bUnique = false;
				}
	        	}

		        /* Anzahl der Versuche wird um eins erhht. */
			iTrialCounter++;
		}

		return sNewStockName;
	}

	/**
	 * Martin Klossek: Methode gibt den als Parameter bergebenen String outSt mit der Methode
	 * des Treibers getDriver().output() aus. Damit werden Meldungen ins Log der Stelle
	 * an zentraler Stelle ausgegeben. Modifizierungen wie das Einfgen der PlaceUserID
	 * sind dadurch leicht mglich.
	 */
	private void enhancedOutput (String sOutSt) {

		/* Ausgabe der in sOutSt bergebenen Mitteilung, wenn m_objDriver
		verfgbar und damit ungleich null ist. Alternativ schreiben wir
		auf die Standardausgabe. */
		if (m_objDriver != null) {
		        m_objDriver.output ( sOutSt );
		} else {
		        System.out.println ( sOutSt );
		}

	}

	/**
	 * Setzt die Membervariable m_objDriver, um Zugriff auf das Driver-Objekt
	 * des Services und damit auf die output()-Methode zu haben. Dieses Objekt
	 * wird nur bentigt, wenn die CStockList als Thread ablaufen soll, damit
	 * Ausgaben ins Logfile erfolgen knnen.
	 *
	 * @param objDriver Der Treiber des aufrufenden Services
	 */
	public void setDriver (AMETASServiceManagerDriverIf objDriver) {
		m_objDriver = objDriver;
	}

	/**
	 * Die fr eine mit Runnable implementierte Klasse ntige Methode, in
	 * der der Thread abluft. Hier werden neue Aktien ausgegeben und Kurswerte
	 * in periodischen Intervallen aktualisiert.
	 */
	public void run() {

		/* Statusmeldung zum "Start des Aktienhandels" ausgeben */
		enhancedOutput ("Simulierter Aktienhandel von " + m_iCompanyAmount + " Aktien hat begonnen.");

		while (m_bUpdate) {

		        /* Kurse aktualisieren, dabei die Anzahl der nicht bankrotten
		        Firmen (Kurswert > 0 Agentendollar) ermitteln. */
	        	int iCurrentStocks = size();
		        int iTradedStocks = 0;
		        for (int i = 0; i < iCurrentStocks; i++) {

				CStock objCurrentStock = getStock(i);
				if (objCurrentStock != null) {

				        if (objCurrentStock.isTraded()) {

						/* Kurs dieses Wertpapiers updaten */
						objCurrentStock.updatePrice(m_objRandom);

						/* Anzahl der gehandelten Wertpapiere erhhen */
				                iTradedStocks++;

				        }

				}

		        }

		        /* Wenn die Anzahl der gehandelten Wertpapiere iTradedStocks unterhalb der
		        in dieser CStockList zu handelnden Wertpapiere (m_iCompanyAmount)
		        liegt, dann werden m_iCompanyAmount - iTradedStocks neue CStocks angelegt. */
		        if (m_iCompanyAmount > iTradedStocks) {

				/* Anzahl der zu erzeugenden CStock-Objekte ("Firmen") */
				int iCreateAmount = m_iCompanyAmount - iTradedStocks;

				/* String fr die Ausgabe der neuen Aktien */
				String sNewStocks = "";

				/* CStock-Objekte erzeugen */
				for (int i = 0; i < iCreateAmount; i++) {

				        /* Neuen Namen fr Aktie ermitteln (sollte nicht in bestehender Liste
				        enthalten sein, da "tote" Aktien gespeichert bleiben) */
				        String sNewStockName = createStockName();
				        double dNewPriceIPO = m_objRandom.nextDouble() * (m_dMaxIpoPrice - m_dMinIpoPrice) + m_dMinIpoPrice;

				        /* Neues CStock-Objekt anlegen */
				        CStock objNewStock = new CStock (sNewStockName, dNewPriceIPO, dNewPriceIPO);

				        /* Frs Protokoll: die neu erzeugten Aktien ausgeben */
				        if (sNewStocks.equals("")) { sNewStocks += sNewStockName; }
				        else  { sNewStocks += ","+sNewStockName; }

				        /* Und in repository einfgen */
				        addElement (objNewStock);

				}

				/* Ausgabe der Namen der neu erzeugten Aktien */
				enhancedOutput ("Neue Aktien: " +sNewStocks);

		        }

		        /* die in m_iCalcIntervall in Sekunden angegebene Zeit warten (Kurse sollen
			nur in diesen Intervallen neu berechnet werden, das ist einigermaen realistisch). */
		        try {
				Thread.currentThread().sleep(m_iCalcIntervall * 1000);
		        } catch (java.lang.InterruptedException iex) {
				/* Hier passiert nichts, es wird einfach neu berechnet. */
		        }

		}

	}

}
