package ap.stockagent;

import AMETAS.data.*;
import AMETAS.event.*;
import ap.data.*;

/**
 * <p>Agent f&uuml;r &quot;Ametas&quot; (<a href="http://www.ametas.de" target="_blank">www.Ametas.de</a>),
 * der mit anderen Agenten &uuml;ber Nachrichten kommuniziert.</p>
 * <p>Der &quot;NotifyAgent&quot; dient als abstrakte Basisklasse fr Agenten, die von einem Stellennutzer
 * beauftragt werden, Nachrichten zu einem anderen Stellennutzer zu befrdern. Dazu bekommt der
 * NotifyAgent bei seinem Start vom Stellennutzer eine initiale Nachricht. Dieser entnimmt der Agent Informationen
 * darber, auf welche Stelle er migrieren soll, welchen Stellennutzer er dort suchen soll, und welche Daten er
 * dort abliefern soll.</p>
 *
 * <p>Diese Klasse wurde im Rahmen des Praktikums &quot;Mobile Agenten&quot; im Wintersemester 2000/01
 * an der Professur &quot;Verteilte Systeme und Betriebssysteme&quot; des Fachbereiches Informatik
 * und Biologie an der Johann Wolfgang Goethe-Universit&auml;t in Frankfurt erstellt:</p>
 * <p>L&ouml;sung zu Aufgabenblatt 7</p> <p>Autoren (Gruppe &quot;Orange&quot;):</p> <ul>
 * <li>Martin Klossek (<a href="mailto:klossek@informatik.uni-frankfurt.de">klossek@informatik.uni-frankfurt.de</a>)</li>
 * <li>Fabian Wleklinski (<a href="mailto:wleklins@informatik.uni-frankfurt.de">wleklins@informatik.uni-frankfurt.de</a>)</li>
 * </ul>
 * @see <a href="http://www.ametas.de" target="_blank">www.Ametas.de</a>
 */
public abstract class NotifyAgent extends AMETAS.event.AMETASNotifiableAgent {

        /** Die zu wartende Anzahl von Sekunden, bis der Agent ohne Erhalt einer Nachricht beendet. */
        final static int m_nSecondsToWait = 120;

        /** Diverse Konstanten fr die Ausgabe von Texten in die Protokolldatei */
        final static java.lang.String c_sLogMigrate_1 = "Ich migriere zu Stelle \"";
        final static java.lang.String c_sLogMigrate_2 = "\". ";
        final static java.lang.String c_sLogExiting = "Ich beende die Ausfhrung. ";
        final static java.lang.String c_sLogCouldntMigrate_1 = "Fehler: Ich konnte nicht migrieren! Fehler: \"";
        final static java.lang.String c_sLogCouldntMigrate_2 = "\".";
        final static java.lang.String c_sLogCouldntMigrateAtAll = "Fehler: Ich konnte auch nach mehreren Versuchen nicht migrieren! Ich beende mich. ";
        final static java.lang.String c_sLogException_1 = "Fehler: Der Fehler \"";
        final static java.lang.String c_sLogException_2 = "\" ist aufgetreten.";

        /** Hlt die jeweils zuletzt eingetroffene Nachricht */
        private AMETAS.data.AMETASMessage m_oMessage;

        /**
         * Der aktuelle Modus/Zustand des Agenten. Es werden die folgenden Modi unterschieden: <ul>
         * <li>0: Der Agent wurde gerade gestartet, und wartet auf externe Anweisungen</li>
         * <li>1: Der Agent migriert zum StockAgent und liefert dort seine Daten ab</li>
         * <li>2: Der Agent befindet sich auf der Ersatzstelle und wartet</li>
         * <li>3: ist mit allen Aufgaben fertig und wird nun beendet</li> </ul>
         */
        private int m_iState = 0;

        /* Der Name der aktuellen Stelle */
        private String m_sCurPlace;

        /* Der Name der nchsten, anzuspringenden Stelle */
        private String m_sNextPlace;

        /** Die PlaceUserID des Stellennutzers, den wir suchen sollen */
        private AMETASPlaceUserID m_puidReceiver;

        /** Die Stelle des Stellennutzers, den wir suchen sollen */
        private String m_sPlaceOfReceiver;

        /** Die PlaceUserID des Stellennutzers, der uns ausgesandt hat */
        private AMETASPlaceUserID m_puidSender;

        /** Die Nachrichten, die wir berbringen sollen */
        private CStockAgentMsgs m_samMsgs;

        /**
         * Wird auf TRUE gesetzt, wenn die Stelle im Begriff ist, beendet
         * zu werden
         */
        private boolean m_bPlaceShutdown = false;

	/**
	 * TRUE wenn die temporre Stelle wieder aktiv ist
	 */
	private boolean m_bPlaceBackAgain;

        /**
         * Der Konstruktor der Klasse: Hier wird der Name des Agenten gesetzt.
         * Der Name charakterisiert den Typ des Agenten, wodurch eine Identifikation anderer Agenten des gleichen Typs mglich wird.
         * @param sAgentClassName der Name der Klasse des Agenten
         */
        public NotifyAgent( String sAgentClassName ) {
                super( sAgentClassName, "orange" );
        }

        /**
         * Die "invoke"-Methode des Agenten, die vom AMETAS-Laufzeitsystem bei jedem Eintreffen
         * oder Starten eines Agenten auf dieser Stelle aufgerufen wird.
         */
        public void invoke() {

                /* Fabian Wleklinski: Wenn dies der erste Start des Agenten ist,
                den Namen der Heimatstele als den Namen der aktuellen Stelle
                merken. Falls nicht, dann muss eine Migration vorausgegangen
                sein, in diesem Fall verwendet wir den Wert aus m_sNextPlace
                als Namen der aktuellen Stelle */

                /*
                if (this.m_bFirstStart) {
                        this.m_sCurPlace = this.m_sHomePlace;
                } else {
                        this.m_sCurPlace = this.m_sNextPlace;
                }
                */

                m_sCurPlace = m_Driver.getPlaceName();

                /* Fabian Wleklinski: Die Funktion getPlaceName() liefert
                manchmal (immer?) Stellennamen mit abschlieendem Punkt (".")
                zurck. Diesen schneiden wir nun raus! */
                if (m_sCurPlace.endsWith( "." )) {
                        m_sCurPlace = m_sCurPlace.substring( 0, m_sCurPlace.length()-1 );
                }

                /* Fabian Wleklinski: den Agenten als Ereignishandler
                installieren, es sei denn, wir sind gerade gestartet worden.
                In diesem Fall mssen wir spter die initiale Nachricht
                "per Hand" abholen */
                if (! this.m_bFirstStart) {
                        if (registerAsListener()) {
                                return;
                        }
                }

                /* Fabian Wleklinski: Die gesamte Logik des Agenten ist hnlich einem
                "endlichen Automaten" aufgebaut. Es gibt eine endliche Menge diskreter
                Zustnde, zwischen denen gewechselt wird. Jeweils beim Wechsel von einem
                Zustand zu einem anderen wird berprft, ob der Agent migrieren muss, und
                wenn ja, wird die Migration durchgefhrt - anderenfalls wird iteriert,
                und zwar solange, bis der Agent irgendwann doch migriert, oder ein
                Endzustand erreicht ist. */

                /* Fabian Wleklinski: Wir stellen sicher, dass "nextPlace" auf den Namen der
                aktuellen Stelle zeigt. Falls das nicht so ist, wird spter entschieden
                werden, dass derAgent migrieren mu... */
                m_sNextPlace = m_sCurPlace;

                /* Fabian Wleklinski: wir iterieren hier so lange, bis ein Endzustand erreicht
                ist. (Bei einer Migration des Agenten und dem damit verbundenen Neustart der
                "invoke"-Methode wird natrlich kein Endzustand erreicht.) */

                while (m_iState < 3) {

                        /* Fabian Wleklinski: Ganz wie bei einem endlichen Automaten werten wir nun den
                        aktuellen "Modus" des Agenten aus: */

                        switch (m_iState) {
                                case 0:
                                        /* Fabian Wleklinski: Der Agent wurde gerade gestartet, und wartet auf externe Anweisungen */

                                        /* Fabian Wleklinski: Die Startnachricht empfangen */
                                        AMETASMessage[] ames = m_Driver.getMessages( true );
                                        if (ames == null) {
                                                enhancedOutput( "Fehler: Ich erwarte eine Nachricht \"GO_NOTIFY\" bei meinem Start. Keine passende Nachricht gefunden. " );
                                                return;
                                        }

                                        // Durch die Nachrichten iterieren
                                        m_oMessage = null;
                                        for (int i=0; i<ames.length; i++) {
                                                if (ames[ i ].getSpecification().equalsIgnoreCase( "GO_NOTIFY" )) {
                                                        m_oMessage = ames[ i ];
                                                }
                                        }

                                        if (m_oMessage == null) {
                                                enhancedOutput( "Fehler: Ich erwarte eine Nachricht \"GO_NOTIFY\" bei meinem Start. Keine passende Nachricht gefunden. " );
                                                return;
                                        }

                                        /* Fabian Wleklinski: Die Startnachricht auswerten */
                                        if (m_oMessage.getBodySize() != 4) {
                                                enhancedOutput( "Fehler: Die vorgefundene Nachricht \"GO_NOTIFY\" verfuegt nicht ber genau vier Felder. " );
                                                return;
                                        }
                                        Object[] oBody = m_oMessage.getBody();
                                        if ( (! (oBody[ 1 ] instanceof AMETASPlaceUserID)) || (! (oBody[ 2 ] instanceof String)) || (! (oBody[ 3 ] instanceof Object[])) ) {
                                                enhancedOutput( "Fehler: Die vorgefundene Nachricht \"GO_NOTIFY\" entspricht nicht dem erwarteten Format (\"GO_NOTIFY\", AMETASPlaceUserID, String, CStockAgentMsgs). " );
                                                return;
                                        }

                                        /* Fabian Wleklinski: Wir merken wir uns die ID des
                                        Absenders fr spter ... */
                                        m_puidSender = m_oMessage.getSenderID();

                                        /* Fabian Wleklinski: Der Agent migriert zu einer anderen Stelle */
                                        m_puidReceiver = (AMETASPlaceUserID) oBody[ 1 ];

                                        m_sPlaceOfReceiver = (String) oBody[ 2 ];
                                        if (m_sPlaceOfReceiver.endsWith( "." )) {
                                                m_sPlaceOfReceiver = m_sPlaceOfReceiver.substring( 0, m_sPlaceOfReceiver.length()-1 );
                                        }


                                        m_sNextPlace = m_sPlaceOfReceiver;
                                        m_samMsgs = new CStockAgentMsgs( (Object[]) oBody[ 3 ] );

                                        oBody = null;

                                        m_iState++;

                                        break;

                                case 1:
                                        /* Fabian Wleklinski: Der Agent migriert zum Empfnger und liefert dort seine Daten ab */

                                        /* Fabian Wleklinski: Detektieren, ob wir
                                        auf einer Ersatzstelle gelandet sind, indem
                                        wir den Namen der Stelle, auf die wir
                                        wollten, vergleichen mit den Namen der
                                        Stelle, auf der wir gelandet sind... */
                                        if (! m_sCurPlace.equalsIgnoreCase( m_sPlaceOfReceiver )) {

                                                enhancedOutput( "bin auf der Stelle \"" + m_sCurPlace + "\", wollte aber nach \"" + m_sPlaceOfReceiver + "\"" );
                                                enhancedOutput( "  => bin auf der Ersatzstelle! Warten auf Aktivierung der Stelle \"" + m_sPlaceOfReceiver + "\"." );

                                                /* Fabian Wleklinski: Aua, wir
                                                sind auf der Ersatzstelle =>
                                                => Modus wechseln */

                                                m_iState = 2;
                                        } else {
                                                Object[] aMessageBody = new Object[ 3 ];
                                                aMessageBody[ 0 ] = "NOTIFICATION";
                                                aMessageBody[ 1 ] = m_puidSender;
                                                aMessageBody[ 2 ] = m_samMsgs.getData();
                                                boolean result = this.sendMessageAndWait( m_puidReceiver, aMessageBody, false );

                                                /* Fabian Wleklinski: Wenn ein Fehler unterlaufen ist, zum Beispiel, weil die
                                                Stelle nicht mehr online ist, oder weil die aktuelle Stelle gerade runtergefahren
                                                wird, dann beenden wir: */
                                                if (result) {
                                                        return;
                                                }

                                                m_iState = 3;
                                        }

                                        break;

                                case 2:
                                        /* Fabian Wleklinski: Der Agent befindet sich auf der Ersatzstelle und wartet */

					while ((! m_bPlaceBackAgain) && (! m_bPlaceShutdown)) {

						/* Fabian Wleklinski: Damit sich das JAVA Laufzeitsystem nicht toteiert, streuen
						wir hier ein paar Delays ein */
						m_Driver.idle( 3000 );

					}

					if (m_bPlaceBackAgain) {
					
                                        	enhancedOutput( "temporaere Stelle steht wieder zur Verfuegung, versuche, zu migrieren..." );
                                        	m_iState = 1;
						m_sNextPlace = m_sPlaceOfReceiver;

					} else {
                                        	m_iState = 3;
					}

                                        break;

                        }

                        /* ************************************************************
                        ACHTUNG: An dieser Stelle hat sich der Modus gendert!
                        ************************************************************ */

                        /* Fabian Wleklinski: Falls wir "auf der Stelle treten", sparen
                        wir uns die Migration und iterieren "lokal". Insbesondere wird in
                        dem Fall, dass ein Endzustand erreicht ist, "m_sNextPlace" auf den
                        Wert von "m_sCurPlace" gesetzt. */
                        if (! m_sNextPlace.equals( m_sCurPlace )) {

                                /* Fabian Wleklinski: Fr den Fall, dass die
                                folgende Migration ohne Fehler stattfindet, wird
                                der Agent von dem AMETAS-System automatisch
                                entladen. Fr den Fall, dass die Migration wg.
                                eines Fehlers nicht stattfindet, werden wir
                                die Migration bis zu x Mal wiederholen. */

                                for (int i=1; i<=10; i++) {

                                        enhancedOutput( c_sLogMigrate_1 + m_sNextPlace + c_sLogMigrate_2 );

                                        try {

                                                /* Fabian Wleklinski: Nun versuchen wir, zu migrieren: */
                                                prepareMigration();
                                                m_Driver.go( m_sNextPlace, 600000 );

                                        } catch( Exception curException ) {

                                                exceptionOutput(
                                                        "m_Driver.go( m_sNextPlace, 600000 );",
                                                        curException
                                                );

                                                /* Fabian Wleklinski: Whrend der Migration ist ein Fehler aufgetreten.
                                                Wir werden dies protokollieren, ohne anschlieend gesondert zu verfahren. */

                                                enhancedOutput( c_sLogCouldntMigrate_1 + curException.toString() + c_sLogCouldntMigrate_2 );
                                        }
                                }

                                /* Fabian Wleklinski: Wenn wir hier angekommen
                                sind, konnte offenbar nicht migriert werden.
                                Diesen Umstand protokollieren wir, und
                                beenden */

                                enhancedOutput( c_sLogCouldntMigrateAtAll );
                                return;
                        }

                } // while (m_nState < ??)

                /* Fabian Wleklinski: Protokollieren, dass nun beendet werden wird */

                enhancedOutput( c_sLogExiting );
        }

        /**
         * Sendet die bergebene AMETAS-Nachricht an den bergebenen Stellennutzer
         * @param puidTarget Die Stellennutzer-ID des anzusprechenden Stellennutzers
         * @param aMessageBody Der Inhalt der zu sendenden Nachricht
         * @param doWait TRUE wenn auf die Antwort des Stellennutzers gewartet werden soll
         */
        private boolean sendMessageAndWait( AMETASPlaceUserID puidTarget, Object[] aMessageBody, boolean doWait ) {

                /*
                AMETASMediationResult[] aoRunningAgents = m_Driver.request( null );
                AMETASPlaceUserIDMask oPlaceUserMask = new AMETASPlaceUserIDMask( null, "CDShopUA", null, 0 );
                m_Driver.depositMessage( new AMETASMessage( oPlaceUserMask, null, aMessageBody ) );
                */

                if (aMessageBody != null) {
                        enhancedOutput( "Sende eine Nachricht an " + puidTarget );
                        m_Driver.depositMessage( new AMETASMessage( puidTarget, aMessageBody ) );
                        enhancedOutput( "Nachricht gesendet" );
                }

                if (doWait) {
                        /* Fabian Wleklinski: eine gewisse Weile auf Nachrichten warten */
                        boolean wakedUp = m_Driver.idle( m_nSecondsToWait * 1000 );

                        if (! wakedUp) {
                                enhancedOutput( "Fehler: Habe nach " + m_nSecondsToWait + " Sekunden keine Antwort vom Stellennutzer erhalten" );
                                return true;
                        }

                        if (m_oMessage == null) {
                                /* Fabian Wleklinski: Die Stelle wird offensichtlich heruntergefahren */
                                enhancedOutput( "Achtung: Die Stelle wird heruntergefahren. " );
                                return true;
                        }
                }

                return false;
        }

        /**
         * Bereitet den Agenten auf die Migration vor. Falls der Agent in seinen
         * Member-Variablen Instanzen von Objekten enthlt, die nicht im
         * AMETAS-Sinne "serialisierbar" sind, wird die Migration nicht
         * funktionieren, bevor diese Instanzen gelscht worden sind. Daher
         * diese Methode.
         */
        private void prepareMigration() {

                // prepares for migration
                m_oMessage = null;
        }

        /**
         * Registriert den aktuellen Agenten auf der aktuellen Stelle als &quot;Listener&quot;
         * fr die ben&ouml;tigten Ereignisse.
         */
        private boolean registerAsListener() {
                /* Fabian Wleklinski: Den Agenten als Event-Listener der aktuellen
                Stelle registrieren. Wichtig ist hierbei, dass wenn keine explizit
                anderslautende Option angegeben wird, die Nachricht, die ein Event
                auslst, mit diesem Event mitgeschickt wird UND aus dem Postamt
                gelscht wird */

                AMETASMessageMask mask = new AMETASMessageMask(null,
                    new AMETASPlaceUserIDMask(this.getID()), AMETASMessage.UNSPECIFIED);
                Object aMask[] =  { mask };
                addEventHandler( new MyEventHandler() );
                try {
                        m_Driver.registerEventListener(AMETASMessageEvent.MESSAGE_RECEIVED_EVENT, 0, aMask);
                } catch (Exception curException) {
                        exceptionOutput("m_Driver.registerEventListener( AMETASMessageEvent.MESSAGE_RECEIVED_EVENT, 0, aMask );", curException);
                        return true;
                }
                try {
                        m_Driver.registerEventListener(AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT, null);
                } catch (Exception curException) {
                        exceptionOutput("m_Driver.registerEventListener( AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT, 0, aMask );", curException);
                        return true;
                }
                try {
                        //m_Driver.registerEventListener(AMETASPlaceEvent.RELAYED_PLACE_ACTIVE_EVENT, null);
                } catch (Exception curException) {
                        //exceptionOutput("m_Driver.registerEventListener(AMETASPlaceEvent.RELAYED_PLACE_ACTIVE_EVENT, null);", curException);
                        //return true;
                }
                return false;
        }

        /**
         * Martin Klossek: Methode gibt den als Parameter bergebenen String outSt mit der Methode
         * des Treibers m_Driver.output() aus und fgt die letzten 5 Zeichen der hexadezimalen
         * placeUserId vor jeden Ausgabestring an. Damit wird erreicht, dass jede log-Ausgabe
         * der jeweiligen Agenteninstanz zugordnet werden kann.
         */
        private void enhancedOutput(String outSt) {
                String ts = getID().toHexString();
                if (ts.length() > 5) {
                        m_Driver.output(ts.substring(ts.length() - 5) + ": " + outSt);
                } else {
                        m_Driver.output(ts + ": " + outSt);
                }
        }

        /** Methode gibt den bergebenen String mittels "enhancedOutput()" aus, fgt aber noch einen Fehlertext hinzu. */
        private void exceptionOutput(String outSt, Exception curException) {
                enhancedOutput("Bei der Ausfuehrung von \"" + outSt + "\" ist folgender Fehler aufgetreten: \"" +
                curException.toString() + "\"");
        }

        public class MyEventHandler extends AMETASEventHandler {
                public boolean handleMessageMessage(String sSpec, Object[] aBody, AMETASMessage mes) {
                        enhancedOutput( "Nachricht via handleMessageMessage() empfangen." );
                        if (mes.getCategory() == AMETASMessage.APPLICATION) {
                                m_oMessage = mes;
                                getDriver().wakeup();
                                return true;
                        }
                        return false;
                }

                public boolean handleApplicationMessage(String sSpec, Object[] aBody, AMETASMessage mes) {
                        enhancedOutput( "Nachricht via handleApplicationMessage() empfangen.");

                        if ((mes.getCategory() == AMETASMessage.APPLICATION)) {
                                m_oMessage = mes;
                                getDriver().wakeup();
                                return true;
                        }
                        return false;
                }

                /** Wird von AMETASNotifiableAgent.notifyListener beim Eintreffen eines Stellenereignisses aufgerufen. */
                public boolean handlePlaceEvent(AMETASPlaceEvent evt) {
                        enhancedOutput( "Nachricht via handlePlaceEvent() empfangen.");
                        if (evt.getID() == AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT) {
                                enhancedOutput( "...Stelle wird in " + evt.getGraceTime() + " Millisekunden geschlossen. Beende Verarbeitung. ");

                                /* Fabian Wleklinski: Die Stelle wird in Krze beendet,
                                eventuelle Aufrumaktionen mssen jetzt durchgefhr werden */

                                m_bPlaceShutdown = true;

                                m_oMessage = null;
                                getDriver().wakeup();
                                return true;
                        } else if (evt.getID() == AMETASPlaceEvent.RELAYED_PLACE_ACTIVE_EVENT) {

				/* Fabian Wleklinski: Eine temporre Stelle ist wieder aktiv.
				Dorthin muss dieser Agent nun migrieren! */

				m_bPlaceBackAgain = true;

				getDriver().wakeup();
				return true;
			}
                        return false;
                }
        }
}
