package ap.stockagent;

import java.util.*;
import java.io.*;
import AMETAS.data.*;
import AMETAS.data.type.*;
import AMETAS.servicedev.*;
import AMETAS.event.*;
import ap.stockserviceproxy.*;
import ap.data.*;

/**
 * <p>Agent f&uuml;r &quot;Ametas&quot; (<a href="http://www.ametas.de" target="_blank">www.Ametas.de</a>),
 * der mit anderen Agenten &uuml;ber Nachrichten kommuniziert.</p>
 * <p>Ein &quot;StockAgent&quot; wird auf einer Stelle von dem Benutzeradapter &quot;StockUA&quot;
 * gestartet, und bekommt von diesem eine initiale Nachricht. Dieser entnimmt der Agent Informationen
 * darber, auf welche Stelle er migrieren soll, und welche Strategie er beim Kaufen und Verkaufen von
 * Aktien auf dieser Stelle anwenden soll, sowie einige Parameter bezglich dieser Strategie. Weiterhin
 * erhlt der Agent durch diese initiale Nachricht eine Menge an &quot;Geld&quot;, den Namen des
 * angemeldeten Benutzers, sowie seinen eigenen Namen zugeteilt.</p>
 * <p>Sobald der StockAgent auf seiner Zielstelle angekommen ist, erffnet er dort ein Konto unter
 * dem Namen <Benutzername>#<Agentname> und zahlt sein vom Benutzeradapter erhaltenes Geld ein.
 * Sollte auf dieser Stelle bereits ein Konto f&uuml;r den Agenten bestehen, so wird dieses
 * weiterverwendet, und kein neues Konto er&ouml;ffnet.</p>
 * <p>Fr den Aktienhandel verwendet der StockAgent den Dienst &quot;StockService&quot; und den
 * ServiceProxy &quot;StockServiceProxy&quot;.</p>
 * <p>Der StockAgent versucht nun, unter Anwendung der definierten Strategie durch den Kauf und Verkauf
 * von Aktien einen mglichst hohen Gewinn zu erzielen - vollkommen autonom.</p>
 * <p>Der StockAgent &quot;wartet&quot; auf andere Agenten vom Typ &quot;PrefAgent&quot;, die der
 * Benutzeradapter auf die Reise schickt, um den StockAgent umzukonfigurieren. Mit diesen Kommuniziert
 * der StockAgent.</p>
 * <p>Der StockAgent versendet von Zeit zu Zeit Agenten vom Typ StockInfoAgent, die zum Benutzeradapter
 * zurckreisen, und ihm Informationen ber den Stand des Depots liefern. </p>
 *
 * <p>Diese Klasse wurde im Rahmen des Praktikums &quot;Mobile Agenten&quot; im Wintersemester 2000/01
 * an der Professur &quot;Verteilte Systeme und Betriebssysteme&quot; des Fachbereiches Informatik
 * und Biologie an der Johann Wolfgang Goethe-Universit&auml;t in Frankfurt erstellt:</p>
 * <p>L&ouml;sung zu Aufgabenblatt 7</p> <p>Autoren (Gruppe &quot;Orange&quot;):</p> <ul>
 * <li>Martin Klossek (<a href="mailto:klossek@informatik.uni-frankfurt.de">klossek@informatik.uni-frankfurt.de</a>)</li>
 * <li>Fabian Wleklinski (<a href="mailto:wleklins@informatik.uni-frankfurt.de">wleklins@informatik.uni-frankfurt.de</a>)</li>
 * </ul>
 * @see <a href="http://www.ametas.de" target="_blank">www.Ametas.de</a>
 */
public class StockAgent extends AMETAS.event.AMETASNotifiableAgent {

        /** Der Name der Agenten(klasse) */
        final static String m_sAgentClassName = "ap.stockagent.StockAgent";

        /** Die zu wartende Anzahl von Sekunden, bis der Agent ohne Erhalt einer Nachricht beendet. */
        final static int m_nSecondsToWait = 120;

        /** Diverse Konstanten fr die Ausgabe von Texten in die Protokolldatei */
        final static java.lang.String c_sLogPlaceDone_1 = "Ich habe die Arbeit auf der Stelle \"";
        final static java.lang.String c_sLogPlaceDone_2 = "\" abgeschlossen. ";

        final static java.lang.String c_sLogGoingHome_1 = "Ich habe alle zu besuchenden Stellen besucht, und kehre nun zu meiner Heimatstelle \"";
        final static java.lang.String c_sLogGoingHome_2 = "\" zurck. ";
        final static java.lang.String c_sLogMigrate_1 = "Ich migriere zu Stelle \"";
        final static java.lang.String c_sLogMigrate_2 = "\". ";
        final static java.lang.String c_sLogAlreadyHome_1 = "Ich befinde mich bereits auf meiner Heimatstelle \"";
        final static java.lang.String c_sLogAlreadyHome_2 = "\", und habe meine Reise daher abgeschlossen. ";
        final static java.lang.String c_sLogExiting = "Ich beende die Ausfhrung. ";
        final static java.lang.String c_sLogHomeReached = "Ich bin wieder auf meiner Heimatstelle angekommen. ";
        final static java.lang.String c_sLogCouldntMigrate_1 = "Fehler: Ich konnte nicht migrieren! Fehler: \"";
        final static java.lang.String c_sLogCouldntMigrate_2 = "\".";
        final static java.lang.String c_sLogCouldntMigrateAtAll = "Fehler: Ich konnte auch nach mehreren Versuchen nicht migrieren! Ich beende mich. ";
        final static java.lang.String c_sLogException_1 = "Fehler: Der Fehler \"";
        final static java.lang.String c_sLogException_2 = "\" ist aufgetreten.";

        /** Hlt die jeweils zuletzt eingetroffene Nachricht */
        private AMETAS.data.AMETASMessage m_oMessage;

        /**
         * Der aktuelle Modus/Zustand des Agenten. Es werden die folgenden Modi unterschieden: <ul>
         * <li>0: Der Agent wurde gerade gestartet, und wartet auf externe Anweisungen</li>
         * <li>1: Der Agent migriert zu einer anderen Stelle</li>
         * <li>2: Der Agent erffnet ein Konto und handelt mit Aktien</li>
         * <li>3: Der Agent hat eine Beendigungsaufforderung erhalten und beendet nun</li>
         * <li>4: Der Agent sendet sein Guthaben an den Benutzeradapter zurck</li>
         * <li>5: Der Agent ist mit allen Aufgaben fertig und wird nun beendet.</li> </ul>
         */
        private int m_iState = 0;

        /* Der Name der aktuellen Stelle */
        private String m_sCurPlace;

        /* Der Name der nchsten, anzuspringenden Stelle */
        private String m_sNextPlace;

        /** Der Name des Benutzers, der den Agenten auf die Reise geschickt hat */
        private String m_sUserName;

        /** Der Name des Agenten, der ihm vom Benutzeradapter zugeteilt worden ist */
        private String m_sAgentName;

        /** Das "Geld" des Agenten */
        private double m_dCredits;

        /**
         * Die Stellennutzer-ID des Benutzeradapters
         */
        private AMETASPlaceUserID m_puidUA = null;

        /**
         * Der numerische Bezeichner fr die Strategie, der Vorgabewert
         * ist 0, und bedeutet, dass bislang noch keine Strategie definiert
         * worden ist. Ein Wert von 0 wird spter zu einem Fehler fhren,
         * falls keine Strategie definiert worden ist, und der Aktienhandel
         * beginnen soll.
         */
        private int m_iStrategy = 0;

        /**
         * Die Werte fr die Parameter (Die Bedeutung ist von der
         * gewhlten Strategie abhngig)
         */
        private double[] m_dStrategyParams;

        /**
         * Wird auf TRUE gesetzt, wenn die Stelle im Begriff ist, beendet
         * zu werden
         */
        private boolean m_bPlaceShutdown = false;

        /**
         * Wird auf TRUE gesetzt, wenn der Agent beendet werden soll
         */
        private boolean m_bAgentShutdown = false;

        /**
         * Der Konstruktor der StockAgent-Klasse: Hier wird der Name des Agenten gesetzt.
         * Der Name charakterisiert den Typ des Agenten, wodurch eine Identifikation anderer Agenten des gleichen Typs mglich wird.
         */
        public StockAgent() {
                super( m_sAgentClassName, "orange" );
        }

        /**
         * Die "invoke"-Methode des Agenten, die vom AMETAS-Laufzeitsystem bei jedem Eintreffen
         * oder Starten eines Agenten auf dieser Stelle aufgerufen wird.
         */
        public void invoke() {

                /* Fabian Wleklinski: Wenn dies der erste Start des Agenten ist,
                den Namen der Heimatstele als den Namen der aktuellen Stelle
                merken. Falls nicht, dann muss eine Migration vorausgegangen
                sein, in diesem Fall verwendet wir den Wert aus m_sNextPlace
                als Namen der aktuellen Stelle */

                /*
                if (this.m_bFirstStart) {
                        this.m_sCurPlace = this.m_sHomePlace;
                } else {
                        this.m_sCurPlace = this.m_sNextPlace;
                }
                */

                m_sCurPlace = m_Driver.getPlaceName();

                /* Fabian Wleklinski: den Agenten als Ereignishandler
                installieren, es sei denn, wir sind gerade gestartet worden.
                In diesem Fall mssen wir spter die initiale Nachricht
                "per Hand" abholen */
                if (! this.m_bFirstStart) {
                        if (registerAsListener()) {
                                return;
                        }
                }

                /* Fabian Wleklinski: Die gesamte Logik des Agenten ist hnlich einem
                "endlichen Automaten" aufgebaut. Es gibt eine endliche Menge diskreter
                Zustnde, zwischen denen gewechselt wird. Jeweils beim Wechsel von einem
                Zustand zu einem anderen wird berprft, ob der Agent migrieren muss, und
                wenn ja, wird die Migration durchgefhrt - anderenfalls wird iteriert,
                und zwar solange, bis der Agent irgendwann doch migriert, oder ein
                Endzustand erreicht ist. */

                /* Fabian Wleklinski: Wir stellen sicher, dass "nextPlace" auf den Namen der
                aktuellen Stelle zeigt. Falls das nicht so ist, wird spter entschieden
                werden, dass derAgent migrieren mu... */
                m_sNextPlace = m_sCurPlace;

                /* Fabian Wleklinski: wir iterieren hier so lange, bis ein Endzustand erreicht
                ist. (Bei einer Migration des Agenten und dem damit verbundenen Neustart der
                "invoke"-Methode wird natrlich kein Endzustand erreicht.) */

                while (m_iState <= 4) {

                        /* Fabian Wleklinski: Ganz wie bei einem endlichen Automaten werten wir nun den
                        aktuellen "Modus" des Agenten aus: */

                        switch (m_iState) {
                                case 0:
                                        /* Fabian Wleklinski: Der Agent wurde gerade gestartet, und wartet auf externe Anweisungen */

                                        for (int i=0; i<10; i++) {

                                                /* Fabian Wleklinski: Die Startnachricht empfangen */
                                                AMETASMessage[] ames = m_Driver.getMessages( true );

                                                if (ames != null) {

                                                        // Durch die Nachrichten iterieren
                                                        m_oMessage = null;
                                                        for (int j=0; j<ames.length; j++) {
                                                                if (ames[ j ].getSpecification().equalsIgnoreCase( "GO_TRADE" )) {
                                                                        m_oMessage = ames[ j ];
                                                                        break;
                                                                }
                                                        }
                                                }

                                                if (m_oMessage != null) {
                                                        break;
                                                }
                                                this.m_Driver.idle( 500 );
                                        }

                                        if (m_oMessage == null) {
                                                enhancedOutput( "Fehler: Ich erwarte eine Nachricht (\"GO_TRADE\") bei meinem Start. Es wurde keine Nachricht gefunden. " );
                                                return;
                                        }

                                        /* Fabian Wleklinski: Die Startnachricht auswerten */
                                        if (m_oMessage.getBodySize() != 5) {
                                                enhancedOutput( "Fehler: Die gefundene Nachricht \"GO_TRADE\" verfuegt nicht ber genau fuenf Felder. " );
                                                return;
                                        }
                                        Object[] oBody = m_oMessage.getBody();
                                        if ( (! (oBody[ 1 ] instanceof String)) || (! (oBody[ 2 ] instanceof String)) || (! (oBody[ 3 ] instanceof Object[])) || (! (oBody[ 4 ] instanceof Double)) ) {
                                                enhancedOutput( "Fehler: Die gefundene Nachricht \"GO_TRADE\" entspricht nicht dem erwarteten Format (\"GO_TRADE\", String, String, CStockAgentMsgs, Double). " );
                                                return;
                                        }


                                        m_sUserName = (String) oBody[ 1 ];
                                        m_sAgentName = (String) oBody[ 2 ];
                                        CStockAgentMsgs samMsgs = new CStockAgentMsgs( (Object[]) oBody[ 3 ] );
                                        m_dCredits = ((Double) oBody[ 4 ]).doubleValue();
                                        oBody = null;

                                        /* Fabian Wleklinski: Wir merken wir uns die ID des
                                        Absenders fr spter ... */
                                        m_puidUA = m_oMessage.getSenderID();

                                        m_iState = this.processMessages( samMsgs, m_iState );

                                        break;
                                case 1:
                                        /* Fabian Wleklinski: Der Agent migriert zu einer anderen Stelle */


                                        m_iState++;
                                        break;
                                case 2:
                                        /* Fabian Wleklinski: Der Agent erffnet ein Konto und handelt mit Aktien */

                                        /* Fabian Wleklinski: Falls bislang keine
                                        Strategie definiert worden ist -> Fehler */
                                        if (this.m_iStrategy == 0) {
                                                enhancedOutput( "Es ist bislang keine Strategie definiert worden, " +
                                                        "der Agent ist nicht einsatzbereit. " );
                                                return;
                                        }

                                        /* Fabian Wleklinski: Eine Instanz des "StockServiceProxy"
                                        erstellen, mit dem wir auf den "StockService" zugreifen */
                                        StockServiceProxy SSP = getServiceProxy();
                                        if (SSP == null) {
                                                return;
                                        }

                                        /* Fabian Wleklinski: Den Kontostand des
                                        Benutzers auf dieser Stelle abfragen.
                                        Falls der Benutzer hier noch ber kein
                                        Konto verfgt, erhalten wir einen negativen
                                        Kontostand zurck; auf diesen Fall wird
                                        weiter unten eingegangen. */
                                        double dBalance = 0;
                                        try {
                                                dBalance = SSP.getBalance( m_sUserName + "#" + m_sAgentName );
                                        } catch (Exception curException) {
                                                exceptionOutput("dBalance = SSP.getBalance( m_sUserName + \"#\" + m_sAgentName );", curException);
                                        }

                                        /* Fabian Wleklinski: Fr den Fall, dass noch
                                        kein entsprechendes Benutzerkonto auf dieser
                                        Stelle existiert, legen wir jetzt ein neues
                                        Benuterkonto an, und zahlen das gesamte Geld,
                                        ber das wir verfgen, ein: */

                                        if (dBalance == -1) {
                                                try {
                                                        dBalance = SSP.newAccount( m_sUserName + "#" + m_sAgentName, (int) java.lang.Math.floor( m_dCredits ) );
                                                } catch (Exception curException) {
                                                        exceptionOutput(
                                                                "dBalance = SSP.newAccount( m_sUserName + \"#\" + m_sAgentName, m_iInitialAMount);",
                                                                curException
                                                        );
                                                }
                                        }

                                        /* Fabian Wleklinski: In "dBalance" steht nun
                                        die Menge an Agentendollar, die wir zur
                                        Verfgung haben */

                                        // enhancedOutput( "Aktueller Kontostand: " + dBalance );

                                        /* Fabian Wleklinski: Hier merken wir
                                        uns Datum und Uhrzeit des (erfolgreichen)
                                        letzten Versandes eines StockInfoAgent */
                                        java.util.Date dateStockInfoAgent = null;

                                        /* Fabian Wleklinski: Es folgt nun die
                                        "Hauptschleife" des Aktienhandels, in
                                        der der Agent Aktien kauft und verkauft.
                                        Das geschieht so lange, bis entweder
                                        der UA eine Nachricht zum Beenden
                                        sendet, oder die Stelle geschlossen
                                        wird. */

                                        while ((! m_bAgentShutdown) && (! m_bPlaceShutdown)) {

                                                /* Fabian Wleklinski: Das Depot ermitteln, dass
                                                wir auf dieser Stelle besitzen */
                                                CDepot dpDepot = new CDepot();
                                                try {
                                                        dpDepot = SSP.getDepot( m_sUserName + "#" + m_sAgentName );
                                                } catch (Exception curException) {
                                                        exceptionOutput("dpDepot = SSP.getDepot( m_sUserName + \"#\" + m_sAgentName );", curException);
                                                }

                                                /* Fabian Wleklinski: Die Menge aller Aktien
                                                ermitteln, die auf der aktuellen Stelle gehandelt
                                                werden */

                                                CStockList slStocks = new CStockList();
                                                try {
                                                        slStocks = SSP.getStockList( true );
                                                } catch (Exception curException) {
                                                        exceptionOutput(
                                                                "slStocks = SSP.getStockList( true );",
                                                                curException
                                                        );
                                                }

                                                /* Fabian Wleklinski: Der prozentuale
                                                Anteil vom aktuellen Geld, den
                                                wir in neue Aktien investieren: */
                                                double dRelativeBudget = 1.0;

                                                /* Fabian Wleklinski: Je nach
                                                verwendeter Strategie ermitteln
                                                wir nun Aktienwerte, die gekauft
                                                bzw. verkauft werden sollen */

                                                CStockList slStocksToBuy = new CStockList();
                                                CDepot dpStocksToSell = new CDepot();

                                                if (this.m_iStrategy == 1) {

                                                        dRelativeBudget = this.m_dStrategyParams[ 3 ];

                                                        for (int i = 0; i < slStocks.size(); i++) {
                                                                CStock csCurStock = (CStock)slStocks.get(i);
                                                                if (csCurStock.getPriceCurrent() - csCurStock.getPriceIPO() <= this.m_dStrategyParams[ 2 ]) {
                                                                        if (csCurStock.getPriceCurrent() - csCurStock.getPriceIPO() > this.m_dStrategyParams[0]) {
                                                                                slStocksToBuy.add(csCurStock);
                                                                        }
                                                                }
                                                        }

                                                        for (int i = 0; i < dpDepot.size(); i++) {
                                                                CDepotEntry dpCurDepot = (CDepotEntry)dpDepot.get(i);

                                                                boolean doSell = false;

                                                                if (dpCurDepot.getPriceCurrent() - dpCurDepot.getPriceBought() >= this.m_dStrategyParams[1]) {
                                                                        doSell = true;
                                                                }
                                                                if (dpCurDepot.getPriceCurrent() - dpCurDepot.getPriceIPO() <= this.m_dStrategyParams[0]) {
                                                                        doSell = true;
                                                                }

                                                                if (doSell) {
                                                                        dpStocksToSell.add(dpCurDepot);
                                                                }
                                                        }

                                                } else {
                                                        /* Fabian Wleklinski:
                                                        eine unbekannte Strategie
                                                        ist definiert worden */
                                                        enhancedOutput( "Die Strategie \"" + this.m_iStrategy + "\" ist unbekannt." );
                                                        return;
                                                }


                                                /* Fabian Wleklinski: Aktien, die sowohl gekauft als
                                                auch verlauft werden sollen, werden nur gekauft */

                                                for (int i = 0; i < dpStocksToSell.size(); i++) {
                                                        CDepotEntry curDepotEntry = (CDepotEntry)dpStocksToSell.get(i);
                                                        for (int j = 0; j < slStocksToBuy.size(); j++) {
                                                                CStock curStock = (CStock)slStocksToBuy.get(j);
                                                                if (curDepotEntry.getName().equalsIgnoreCase(curStock.getName())) {
                                                                        dpStocksToSell.remove(i);
                                                                        i--;
                                                                }
                                                        }
                                                }

                                                /* Fabian Wleklinski: Wir verkaufen nun alle Aktien,
                                                die wir uns zum Verkauf notiert haben */

                                                for (int i = 0; i < dpStocksToSell.size(); i++) {
                                                        CDepotEntry dpCurDepot = (CDepotEntry) dpStocksToSell.get(i);
                                                        // enhancedOutput("Verkaufe " + dpCurDepot.getAmount() + " * " + dpCurDepot.getName() + " zu " + dpCurDepot.getPriceCurrent() );
                                                        try {
                                                                SSP.sellStock( m_sUserName + "#" + m_sAgentName, dpCurDepot.getName(), dpCurDepot.getAmount());

                                                        } catch (Exception curException) {
                                                                exceptionOutput("SSP.sellStock( m_sUserName + \"#\" + m_sAgentName, dpCurDepot.getName(), dpCurDepot.getAmount() );",
                                                                curException);
                                                        }

                                                        /* Fabian Wleklinski: Den Kontostand des
                                                        Benutzers auf dieser Stelle abfragen.*/
                                                        dBalance = 0;
                                                        try {
                                                                dBalance = SSP.getBalance( m_sUserName + "#" + m_sAgentName );
                                                        } catch (Exception curException) {
                                                                exceptionOutput("dBalance = SSP.getBalance( m_sUserName + \"#\" + m_sAgentName );", curException);
                                                        }

                                                        // enhancedOutput( "Aktueller Kontostand: " + dBalance );
                                                }

                                                double dBalanceRest = dBalance - (dBalance * dRelativeBudget);

                                                /* Fabian Wleklinski: Wir kaufen nun alle Aktien,
                                                die wir uns zum Kauf notiert haben */

                                                while (slStocksToBuy.size() > 0) {

                                                        CStock csCurStock = (CStock)slStocksToBuy.get( 0 );

                                                        /* Fabian Wleklinski: Den Kontostand des
                                                        Benutzers auf dieser Stelle abfragen.*/
                                                        dBalance = 0;
                                                        try {
                                                                dBalance = SSP.getBalance( m_sUserName + "#" + m_sAgentName );
                                                        } catch (Exception curException) {
                                                                exceptionOutput("dBalance = SSP.getBalance( m_sUserName + \"#\" + m_sAgentName );", curException);
                                                        }

                                                        // enhancedOutput( "Aktueller Kontostand: " + dBalance );

                                                        double dBudgetPerStock = (dBalance - dBalanceRest) / slStocksToBuy.size();
                                                        int iCountPerStock = (int) java.lang.Math.floor( dBudgetPerStock / csCurStock.getPriceCurrent() );

                                                        if (iCountPerStock > 0) {

                                                                CDepotEntry deStockBuyed = new CDepotEntry();
                                                                // enhancedOutput("Kaufe " + iCountPerStock + " * " + csCurStock.getName() + " zu " + csCurStock.getPriceCurrent() + " (total: " + csCurStock.getPriceCurrent() * iCountPerStock + ")");
                                                                try {
                                                                        deStockBuyed = SSP.buyStock( m_sUserName + "#" + m_sAgentName, csCurStock.getName(), iCountPerStock);
                                                                } catch (Exception curException) {
                                                                        exceptionOutput( "deStockBuyed = SSP.buyStock( m_sUserName + \"#\" + m_sAgentName, csCurStock.getName(), iCountPerStock);", curException);
                                                                }
                                                        }

                                                        slStocksToBuy.removeElementAt( 0 );
                                                }

                                                /* Fabian Wleklinski: Das Depot ermitteln, dass
                                                wir auf dieser Stelle besitzen */
                                                dpDepot = new CDepot();
                                                try {
                                                        dpDepot = SSP.getDepot( m_sUserName + "#" + m_sAgentName );
                                                } catch (Exception curException) {
                                                        exceptionOutput("dpDepot = SSP.getDepot( m_sUserName + \"#\" + m_sAgentName );", curException);
                                                }

                                                /* Fabian Wleklinski: Den Kontostand des
                                                Benutzers auf dieser Stelle abfragen.*/
                                                dBalance = 0;
                                                try {
                                                        dBalance = SSP.getBalance( m_sUserName + "#" + m_sAgentName );
                                                } catch (Exception curException) {
                                                        exceptionOutput("dBalance = SSP.getBalance( m_sUserName + \"#\" + m_sAgentName );", curException);
                                                }

                                                dpDepot.setAccountBalance( new Double( dBalance ) );

                                                dateStockInfoAgent = this.sendStockInfoAgent( dpDepot, dateStockInfoAgent, false );

                                                m_Driver.idle( 3000 );
                                        }

                                        /* Fabian Wleklinski: Das Depot ermitteln, dass
                                        wir auf dieser Stelle besitzen */
                                        CDepot dpDepot = new CDepot();
                                        try {
                                                dpDepot = SSP.getDepot( m_sUserName + "#" + m_sAgentName );
                                        } catch (Exception curException) {
                                                exceptionOutput("dpDepot = SSP.getDepot( m_sUserName + \"#\" + m_sAgentName );", curException);
                                        }

                                        /* Fabian Wleklinski: Nun schnell _alle_
                                        Aktien aus dem Depot zu Geld machen */
                                        for (int i = 0; i < dpDepot.size(); i++) {
                                                CDepotEntry dpCurDepot = (CDepotEntry) dpDepot.get(i);

                                                // enhancedOutput("Verkaufe " + dpCurDepot.getAmount() + " * " + dpCurDepot.getName() + " zu " + dpCurDepot.getPriceCurrent() );
                                                try {
                                                        SSP.sellStock( m_sUserName + "#" + m_sAgentName, dpCurDepot.getName(), dpCurDepot.getAmount());

                                                } catch (Exception curException) {
                                                        exceptionOutput( "SSP.sellStock( m_sUserName + \"#\" + m_sAgentName, dpCurDepot.getName(), dpCurDepot.getAmount() );",
                                                        curException);
                                                }
                                        }

                                        /* Fabian Wleklinski: Den Kontostand des
                                        Benutzers auf dieser Stelle abfragen.*/
                                        dBalance = 0;
                                        try {
                                                dBalance = SSP.getBalance( m_sUserName + "#" + m_sAgentName );
                                        } catch (Exception curException) {
                                                exceptionOutput("dBalance = SSP.getBalance( m_sUserName + \"#\" + m_sAgentName );", curException);
                                        }

                                        // enhancedOutput( "Aktueller Kontostand: " + dBalance );

                                        /* Fabian Wleklinski: Nun das Konto schlieen */
                                        try {
                                                m_dCredits = m_dCredits + SSP.closeAccount( m_sUserName + "#" + m_sAgentName );
                                        } catch (Exception curException) {
                                                exceptionOutput( "SSP.closeAccount( m_sUserName + \"#\" + m_sAgentName );", curException);
                                        }

                                        // enhancedOutput( "Verfge nun ber folgenden Geldbetrag: " + dBalance );

                                        m_iState++;
                                        break;
                                case 3:
                                        /* Fabian Wleklinski: Der Agent hat eine Beendigungsaufforderung erhalten und beendet nun */
                                        m_iState++;
                                        break;
                                case 4:
                                        /* Fabian Wleklinski: Der Agent sendet sein Guthaben an den Benutzeradapter zurck */

                                        /* Fabian Wleklinski: Wir sind fertig mit der Verarbeitung, und werden den Agenten nun beenden */
                                        m_iState++;
                                        break;

                        }

                        /* ************************************************************
                        ACHTUNG: An dieser Stelle hat sich der Modus gendert!
                        ************************************************************ */

                        /* Fabian Wleklinski: Falls wir "auf der Stelle treten", sparen
                        wir uns die Migration und iterieren "lokal". Insbesondere wird in
                        dem Fall, dass ein Endzustand erreicht ist, "m_sNextPlace" auf den
                        Wert von "m_sCurPlace" gesetzt. */
                        if (! m_sNextPlace.equals( m_sCurPlace )) {

                                /* Fabian Wleklinski: Fr den Fall, dass die
                                folgende Migration ohne Fehler stattfindet, wird
                                der Agent von dem AMETAS-System automatisch
                                entladen. Fr den Fall, dass die Migration wg.
                                eines Fehlers nicht stattfindet, werden wir
                                die Migration bis zu x Mal wiederholen. */

                                for (int i=1; i<=10; i++) {

                                        enhancedOutput( c_sLogMigrate_1 + m_sNextPlace + c_sLogMigrate_2 );

                                        AMETASSecurityFeature sfSecurityFeature = new AMETASSecurityFeature( AMETASSecurityFeature.ENCRYPT, AMETASSecurityFeature.ENCRYPT, true );
                                        m_Driver.setSecurityFeature( sfSecurityFeature );

                                        try {

                                                /* Fabian Wleklinski: Nun versuchen wir, zu migrieren: */
                                                prepareMigration();
                                                m_Driver.go( m_sNextPlace, DONT_RELAY );

                                        } catch ( Exception curException ) {

                                                exceptionOutput(
                                                        "m_Driver.go( m_sNextPlace, DONT_RELAY );",
                                                        curException
                                                );

                                                /* Fabian Wleklinski: Whrend der Migration ist ein Fehler aufgetreten.
                                                Wir werden dies protokollieren, ohne anschlieend gesondert zu verfahren. */

                                                enhancedOutput( c_sLogCouldntMigrate_1 + curException.toString() + c_sLogCouldntMigrate_2 );
                                        }
                                }

                                /* Fabian Wleklinski: Wenn wir hier angekommen
                                sind, konnte offenbar nicht migriert werden.
                                Diesen Umstand protokollieren wir, und
                                beenden */

                                enhancedOutput( c_sLogCouldntMigrateAtAll );
                                return;
                        }

                } // while (m_nState < ??)

                /* Fabian Wleklinski: Protokollieren, dass nun beendet werden wird */

                enhancedOutput( c_sLogExiting );
        }

        /**
         * Sendet einen Agenten der Klasse StockInfoAgent an den Benutzeradapter
         * zurck und bermittelt in ihm mittels Instanzen der Klassen
         * CStockAgentMsgDepot und CStockAgentMsgs die bergebene Instanz
         * von CDepot.
         * @param depotCurDepot die Instanz von CDepot, die versendet werden soll
         * @param dateLastStockInfoAgent Datum und Uhrzeit der letzten Versendung eines StockInfoAgent
         * @param bEnforceTransfer TRUE wenn der Transport auf jeden Fall und
         *      unabhngig vom TimeStamp erfolgen soll
         */
        private java.util.Date sendStockInfoAgent( CDepot depotCurDepot, java.util.Date dateLastStockInfoAgent, boolean bEnforceTransfer ) {
                java.util.Date dateCurDate = new java.util.Date();
                java.util.Date dateResult = dateLastStockInfoAgent;

                /* Fabian Wleklinski: Erst einmal entscheiden, ob berhaupt ein
                StockInfoAgent gesendet werden soll */
                boolean bDoSend = false;
                if (dateLastStockInfoAgent != null) {
                        if (dateCurDate.getTime() - dateLastStockInfoAgent.getTime() > 10000) {
                                bDoSend = true;
                        }
                } else {
                        bDoSend = true;
                }
                bDoSend = bDoSend || bEnforceTransfer;

                if (bDoSend) {

                        /* Fabian Wleklinski: Die Nachricht zusammenbauen, die wir dem UA
                        zurcksenden */
                        CStockAgentMsgDepot samdUAMessage = new CStockAgentMsgDepot( depotCurDepot );
                        CStockAgentMsgs samMsgsUA = new CStockAgentMsgs();
                        samMsgsUA.add( samdUAMessage );
                        Object[] asBody = { "GO_NOTIFY",
                                m_puidUA,
                                m_sHomePlace,
                                samMsgsUA.getData()
                        };
                        AMETASMessage mesUA = new AMETASMessage( null, asBody );

                        /* Starte den StockAgent */
                        AMETAS.data.AMETASPlaceUserID m_pidAgent = null;
                        try {
                                m_pidAgent = m_Driver.spawnAgent( "ap.stockagent.StockInfoAgent", mesUA );

                                /* Fabian Wleklinski: Wenn die Ausfhrung bis hierher
                                fortgeschritten ist, konnte der Agent versandt werden,
                                und wir merken uns den TimeStamp */
                                dateResult = dateCurDate;
                        } catch (Exception curException) {
                                exceptionOutput( "m_pidAgent = m_Driver.spawnAgent( \"ap.stockagent.StockInfoAgent\", mesUA );", curException);
                        }
                }

                return dateResult;
        }

        /**
         * Bereitet den Agenten auf die Migration vor. Falls der Agent in seinen
         * Member-Variablen Instanzen von Objekten enthlt, die nicht im
         * AMETAS-Sinne "serialisierbar" sind, wird die Migration nicht
         * funktionieren, bevor diese Instanzen gelscht worden sind. Daher
         * diese Methode.
         */
        private void prepareMigration() {

                // prepares for migration
                m_oMessage = null;
        }

        /**
         * Registriert den aktuellen Agenten auf der aktuellen Stelle als &quot;Listener&quot;
         * f&uuml;r die ben&ouml;tigten Ereignisse.
         */
        private boolean registerAsListener() {
                /* Fabian Wleklinski: Den Agenten als Event-Listener der aktuellen
                Stelle registrieren. Wichtig ist hierbei, dass wenn keine explizit
                anderslautende Option angegeben wird, die Nachricht, die ein Event
                auslst, mit diesem Event mitgeschickt wird UND aus dem Postamt
                gelscht wird */

                AMETASMessageMask mask = new AMETASMessageMask(null,
                    new AMETASPlaceUserIDMask(this.getID()), AMETASMessage.UNSPECIFIED);
                Object aMask[] =  { mask };
                addEventHandler( new MyEventHandler() );
                try {
                        m_Driver.registerEventListener(AMETASMessageEvent.MESSAGE_RECEIVED_EVENT, 0, aMask);
                } catch (Exception curException) {
                        exceptionOutput("m_Driver.registerEventListener( AMETASMessageEvent.MESSAGE_RECEIVED_EVENT, 0, aMask );", curException);
                        return true;
                }
                try {
                        // m_Driver.registerEventListener(AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT, 0, aMask);
                        m_Driver.registerEventListener(AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT, null);
                } catch (Exception curException) {
                        exceptionOutput("m_Driver.registerEventListener( AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT, 0, aMask );", curException);
                        return true;
                }
                return false;
        }

        /* Erstellt eine Instant des "StockServiceProxy", mit der auf den
        "StockService" zugegriffen wird */

        private StockServiceProxy getServiceProxy() {
                StockServiceProxy SSP = new StockServiceProxy(this, m_Driver);
                int nErrorStatus = SSP.init();
                // Fange Fehler bei der Initialisierung ab.
                if (nErrorStatus == AMETASServiceProxy.SERVICE_NOT_FOUND) {
                        enhancedOutput("Proxy hat den Service \"ap.stockservice.StockService\" nicht gefunden. Ich beende die Ausfuehrung. ");
                        return null;
                } else if (nErrorStatus == AMETASServiceProxy.NO_SERVICE_NAME) {
                        enhancedOutput("Interner Fehler. Es wurde kein Dienstname angegeben. Ich beende die Ausfuehrung. ");
                        return null;
                }
                return SSP;
        }

        /**
         * Verarbeitet eine gegebene Menge von Nachrichten, die von einem
         * StockInfoAgent, einem PrefsAgent oder direkt von einem
         * Benutzeradapter empfangen worden sind.
         * @param samMsgs die zu verarbeitenden Nachrichten
         * @param iCurrentState der aktuelle Modus/Zustand des Agenten
         * @return der neue Modus/Zustand des Agenten
         */
        private int processMessages( CStockAgentMsgs samMsgs, int iCurrentState ) {
                /* Fabian Wleklinski: Die vorhandenen Nachrichten mssen
                in ihrer zeitlichen Reihenfolge abgearbeitet werden, sonst
                knnte Chaos entstehen. Daher iterieren wir nun durch die
                Menge der Nachrichten, suchen die lteste Nachricht,
                verarbeiten diese, und lschen diese danach. Dieser Vorgang
                wird so lange wieerholt, bis die Menge der Nachrichten
                leer ist. */

                int iReturnValue = iCurrentState;

                while (samMsgs.size() > 0) {

                        /* Fabian Wleklinski: Hier merken wir uns die jeweils "bisher"
                        lteste Nachricht: */
                        CStockAgentMsg samOldestMsg = samMsgs.getMsg(0);

                        for (int iMsg=1; iMsg<samMsgs.size(); iMsg++) {
                                /* Fabian Wleklinski: Ist diese Nachricht lter
                                als die bisher gefundenen? */
                                if (samMsgs.getMsg(iMsg).getTimeCreated().before(samOldestMsg.getTimeCreated())) {
                                        samOldestMsg = samMsgs.getMsg( iMsg );
                                }
                        }

                        /* Fabian Wleklinski: In "samOldestMsg" ist nun die
                        lteste Nachricht. Dieese lschen wir nun: */

                        samMsgs.remove( samOldestMsg );

                        if (samOldestMsg instanceof CStockAgentMsgMove) {
                                /* Fabian Wleklinski: Der Agent soll sich auf eine
                                andere Stelle bewegen und dort fortfahren */

                                /* Fabian Wleklinski: Eigentlich mssten wir
                                dafr sorgen, dass die bereits gekauften Aktien
                                verkauft werden, und das Depot geschlossen wird.
                                Gem der aktuellen Version der Aufgabenstellung
                                knnen wir aber davon ausgehen, dass wenn diese
                                Nachricht kommt, der Agent noch berhaupt kein
                                Konto erffnet hat, und auch noch keine Aktien
                                gekauft hat. */

                                this.m_sNextPlace = ((CStockAgentMsgMove) samOldestMsg).getPlaceName();

                                iReturnValue = 1;

                        } else if (samOldestMsg instanceof CStockAgentMsgExit) {

                                m_bAgentShutdown = true;

                        } else if (samOldestMsg instanceof CStockAgentMsgStrategy) {

                                /* Fabian Wleklinski: Eine neue Strategie soll
                                verwendet werden. Diese in einigen
                                Klassenvariablen merken */

                                m_iStrategy = ((CStockAgentMsgStrategy) samOldestMsg).getStrategy();
                                int iParamsCount = ((CStockAgentMsgStrategy) samOldestMsg).getParamsCount();
                                m_dStrategyParams = new double[ iParamsCount ];

                                String sTmp = "Strategie \"" + m_iStrategy + "\" gesetzt, Parameter: ";

                                for (int i=0; i<iParamsCount; i++) {
                                        m_dStrategyParams[ i ] = ((CStockAgentMsgStrategy) samOldestMsg).getParam( i );
                                        sTmp = sTmp + m_dStrategyParams[ i ] + ",";
                                }
                                enhancedOutput( sTmp );


                        } else {
                                /* Fabian Wleklinski: Diese Nachricht ist unbekannt.
                                Wir melden einen Fehler und bergehen sie. */
                                this.enhancedOutput( "Nachricht der Klasse \"" + samOldestMsg.getClass().getName() + "\" nicht bekannt. Nachricht wird ignoriert. " );
                        }
                }

                return iReturnValue;
        }

        /**
         * Martin Klossek: Methode gibt den als Parameter bergebenen String outSt mit der Methode
         * des Treibers m_Driver.output() aus und fgt die letzten 5 Zeichen der hexadezimalen
         * placeUserId vor jeden Ausgabestring an. Damit wird erreicht, dass jede log-Ausgabe
         * der jeweiligen Agenteninstanz zugordnet werden kann.
         */
        private void enhancedOutput(String outSt) {
                String ts = getID().toHexString();
                if (ts.length() > 5) {
                        m_Driver.output(ts.substring(ts.length() - 5) + ": " + outSt);
                } else {
                        m_Driver.output(ts + ": " + outSt);
                }
        }

        /** Methode gibt den bergebenen String mittels "enhancedOutput()" aus, fgt aber noch einen Fehlertext hinzu. */
        private void exceptionOutput(String outSt, Exception curException) {
                enhancedOutput("Bei der Ausfuehrung von \"" + outSt + "\" ist folgender Fehler aufgetreten: \"" +
                curException.toString() + "\"");
        }

        public class MyEventHandler extends AMETASEventHandler {
                public boolean handleMessageMessage(String sSpec, Object[] aBody, AMETASMessage mes) {
                        enhancedOutput( "Nachricht via handleMessageMessage() empfangen." );
                        if (mes.getCategory() == AMETASMessage.APPLICATION) {
                                m_oMessage = mes;
                                getDriver().wakeup();
                                return true;
                        }
                        return false;
                }

                public boolean handleApplicationMessage(String sSpec, Object[] aBody, AMETASMessage mes) {
                        enhancedOutput( "Nachricht via handleApplicationMessage() empfangen.");

                        if (sSpec.equalsIgnoreCase( "NOTIFICATION" ) ) {

				enhancedOutput( "...empfangene Nachricht ist NOTIFICATION" );

                                CStockAgentMsgs samsMsgs = new CStockAgentMsgs( (Object[]) aBody[ 2 ] );

                                m_iState = processMessages( samsMsgs, m_iState );

                                getDriver().wakeup();
                                return true;
                        }

                        if ((mes.getCategory() == AMETASMessage.APPLICATION)) {
                                m_oMessage = mes;
                                getDriver().wakeup();
                                return true;
                        }
                        return false;
                }

                /** Wird von AMETASNotifiableAgent.notifyListener beim Eintreffen eines Stellenereignisses aufgerufen. */
                public boolean handlePlaceEvent(AMETASPlaceEvent evt) {
                        enhancedOutput( "Nachricht via handlePlaceEvent() empfangen.");
                        if (evt.getID() == AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT) {
                                enhancedOutput( "...Stelle wird in " + evt.getGraceTime() + " Millisekunden geschlossen. Beende Verarbeitung. ");

                                /* Fabian Wleklinski: Die Stelle wird in Krze beendet,
                                eventuelle Aufrumaktionen mssen jetzt durchgefhr werden */

                                m_bPlaceShutdown = true;

                                m_oMessage = null;
                                getDriver().wakeup();
                                return true;
                        }
                        return false;
                }
        }
}
