package ap.stockservice;

import AMETAS.data.AMETASMessage;
import AMETAS.servicedev.*;
import AMETAS.util.threadpool.*;
import AMETAS.data.*;
import AMETAS.data.type.*;

import ap.data.*;

import java.util.*;

/**
 * Title:        Ametas StockService
 * Description:  Hauptklasse des Ametas-Service fr simulierten Mini-Aktienhandel
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class StockService extends AMETASServiceObject {

	/** Der symbolische Name des Services */
	private String m_sPUName = "StockService";

	/** Die dynamische Liste der Benutzeraccounts, die ihrerseits CDepot-Klassen enthalten */
	private CAccountList m_objAccountList;

	/** Die maximale Anzahl von Klientenanfragen, die gleichzeitig von diesem Dienst bearbeitet
	werden knnen (= die Anzahl der gleichzeitig offengehaltenen Threads im ThreadPool fr die Anfragen). */
	private int m_iMaxClients = 3;

	/** Der ThreadPool, aus dem dieser Dienst die Anfragen seiner Klienten bedient. */
	private ThreadPool m_objThreadPool;

	/** Der fr den Aktienhandel verantwortliche Thread */
	private Thread m_thdTrading;

	/** Die Liste der gehandelten Aktien (mit Threadfunktionalitt -> die Liste updatet sich selbst :) */
	private CStockList m_stlTrading;


	/**
	 * Der Konstruktor der StockService-Klasse. Hier passiert nicht viel ;-)
	 */
	public StockService() {
	}

	/**
	 * Initialisiert den Dienst. Hier knnen bei Bedarf Parameter ausgelesen werden,
	 * die dem Service beim Start vom Ametas-System bergeben werden (-> siehe dazu
	 * auch die Dateien pu.reg pu.prm).
	 */
	public void initService(Vector vctServiceParams) {

		/* Eltern-Methode aufrufen */
		super.initService(vctServiceParams);

		/* Erzeuge einen neuen ParamHandler, um die Dienstparameter,
		die beim Start des Dienstes bergeben wurden, auszuwerten */
		AMETASServiceParamHandler phnd = new AMETASServiceParamHandler (vctServiceParams);

		/* Das autonom handelnde Objekt m_stlTrading zum "Handeln der Aktienkurse" anlegen  */
		m_stlTrading = new CStockList (10, 2);  // 10 firmen, 2 sekunden verzgerung

		/* Einen Ausgabekanal fr die tradende StockList bereitstellen */
		AMETAS.servicedev.AMETASServiceManagerDriverIf x = m_ServiceMgr.getDriver();
		m_stlTrading.setDriver (x);

		/* Anlegen des Threads zum "Aktienhandel" mit der CStockList m_stlTrading */
		m_thdTrading = new Thread(m_stlTrading);
		m_thdTrading.start();

		/* Objekt fr repository der Benutzeraccounts anlegen */
		m_objAccountList = new CAccountList ();

		/* Einen Ametas ThreadPool fr das "parallele" Handling der Requests erzeugen */
		m_objThreadPool = new ThreadPool(m_iMaxClients);

	}

	/**
	 * Liefert die Beschreibung des Dienstes.
	 */
	public AMETASServiceDescription getServiceDescription(String sServiceName) {

		return new AMETASServiceDescription(sServiceName,
		  "This service implements a very simple stock exchange system :)",
		  AMETASServiceDescription.SHARED);

	}

	/**
	 * Martin Klossek: Methode gibt den als Parameter bergebenen String outSt mit der Methode
	 * des Treibers getDriver().output() aus. Damit werden Meldungen ins Log der Stelle
	 * an zentraler Stelle ausgegeben. Modifizierungen wie das Einfgen der PlaceUserID
	 * sind dadurch leicht mglich.
	 */
	private void enhancedOutput (String sOutSt) {

		/* Ausgabe der in sOutSt bergebenen Mitteilung */
		// etwas zu lang geworden: m_ServiceMgr.getDriver().output ( m_sPUName + ": " + sOutSt );
		m_ServiceMgr.getDriver().output ( sOutSt );

	}

	public void startService(AMETASMessage msgRequest) {

		/* Neue Anfrage ist eingetroffen und wird von einem StockServiceResponder-
		Objekt im ThreadPool verarbeitet */
		m_objThreadPool.startNewWorker(new StockServiceResponder(msgRequest));

	}

	/**
	 * Diese Klasse implementiert die Worker-Schnittstelle des Paketes AMETAS.util.threadpool.
	 * Bei jeder Anfrage wird ein neuer Responder erzeugt, der einem ThreadPool zur Bearbeitung
         * bergeben wird.
	 *
	 * In dieser internen Klasse werden die requests dispatcht und an die einzelnen
	 * Bearbeitungsfunktionen/objekte weitergeleitet. Durch die quasi-Parallelitt unter
	 * Verwendung eines ThreadPools knnen mehrere Anfragen gleichzeitig bearbeiten
	 * werden. Darauf ist bei der Implementierung besonders zu achten.
	 */
	class StockServiceResponder implements Worker {

		/** Die Anfragenachricht, die in diesem Worker bearbeitet wird. */
		private AMETASMessage m_msgRequest;

		/**
		 * Konstruktor der die Anfragenachricht entgegennimmt.
		 */
		public StockServiceResponder (AMETASMessage msgRequest) {
			m_msgRequest = msgRequest;
		}

		/**
		 * Die work()-Methode der Worker-Schnittstelle. Sie wird von einem Thread
		 * im ThreadPool ausgefhrt.
		 */
		public void work() {

			/* Daten aus der Anfragenachricht extrahieren */
			Object[] aobjBody = m_msgRequest.getBody();
			AMETASPlaceUserID puidClient = m_msgRequest.getSenderID();
			AMETASMessageID midRequestID = m_msgRequest.getID();
			String sCommand = (String)aobjBody[0];

		        /* Variablen fr die Antwortnachricht */
			Vector vctResponse = new Vector();
			Vector vctResponseSpec = new Vector();

		        /* Ausgabe der Daten der Reqeust-Nachricht */
		        /* String sOutStr = "";
		        for (int i = 0; i < aobjBody.length; i++) {
				if (sOutStr.equals("")) { sOutStr += aobjBody[i].toString(); }
				else { sOutStr += ";"+aobjBody[i].toString(); }
		        }
		        enhancedOutput ("---> request: [" + sOutStr + "]"); */

			/* Im folgenden werden die einzelnen Anfragetypen dispatcht. Momentan
		        erfolgt das Processing noch in dieser Funktion, vielleiht wre eine
		        Auslagerung in einzelne Methoden bersichtlicher */

		        /* Gibt den aktuellen Preis einer Aktie zurck */
		        if ( sCommand.equals("GET_CURRENT_PRICE") ) {

				/* Krzel der Aktie in Nutzlast der Antwortnachricht legen */
				vctResponse.addElement (aobjBody[1]);

				/* Objekt der angeforderten Aktie suchen */
				CStock objRequestedStock = m_stlTrading.getElementByName ((String)aobjBody[1]);

				if (objRequestedStock != null) {
				        if (objRequestedStock.isTraded()) {
						vctResponseSpec.addElement ("PRICE_IS");
					        vctResponse.addElement ( new Double(objRequestedStock.getPriceCurrent()) );
				        } else {
				                vctResponseSpec.addElement ("NOT_TRADED");
				        }
			        } else {
				        vctResponseSpec.addElement ("NOT_TRADED");
				}

		        /* Gibt den Erffnungskurs (IPO-Preis) einer Aktie zurck */
		        } else if ( sCommand.equals("GET_IPO_PRICE") ) {

				/* Krzel der Aktie in Nutzlast der Antwortnachricht legen */
				vctResponse.addElement (aobjBody[1]);

				/* Objekt der angeforderten Aktie suchen */
				CStock objRequestedStock = m_stlTrading.getElementByName ((String)aobjBody[1]);

				if (objRequestedStock != null) {
				        if (objRequestedStock.isTraded()) {
				                vctResponseSpec.addElement ("IPO_PRICE_IS");
					        vctResponse.addElement ( new Double(objRequestedStock.getPriceIPO()) );
				        } else {
				                vctResponseSpec.addElement ("NOT_TRADED");
				        }
			        } else {
				        vctResponseSpec.addElement ("NOT_TRADED");
				}

		        /* Gibt eine Liste der im StockService gehandelten Aktien zurck */
		        } else if ( sCommand.equals("GET_STOCK_LIST") ) {

				/* Spezifikation der Rckgabenachricht setzen */
				vctResponseSpec.addElement ("STOCKS_ARE");

				/* Liste der Namen der gehandelten Papiere in vctResponse
				als Strings ablegen (okay, als was sonst :-) */
				for (int i = 0; i < m_stlTrading.size(); i++) {
				        CStock objCurrentStock = m_stlTrading.getStock (i);
				        if (objCurrentStock != null) {
						if (objCurrentStock.isTraded()) {
						        vctResponse.addElement(objCurrentStock.getName());
						}
				        }
				}

		        /* Gibt die Liste der im Depot des Accounts gespeicherten Eintrge zurck */
		        } else if ( sCommand.equals("GET_DEPOT") ) {

				try {

				        /* Die ausgelagerte Methode zur Auflistung des Depots aufrufen */
					func_Get_Depot ( vctResponseSpec, vctResponse, (String)aobjBody[1] );

				} catch (Exception ex) {

				        vctResponseSpec.clear();
				        vctResponseSpec.addElement ("UNKNOWN_REQUEST");
					vctResponse.addElement (sCommand);
				        vctResponse.addElement (ex);
				        vctResponse.addElement (ex.getMessage());

				}

		        /* Erffnet einen neuen Account mit dem bergebenen Benutzernamen und Kontostand */
		        } else if ( sCommand.equals("NEW_ACCOUNT") ) {

				try {

				        /* Neues Objekt vom Typ CAccount anlegen und mit Daten fllen und
				        in Account-repository stecken */
				        double dStartedBalance = (new Double(aobjBody[2].toString())).doubleValue();
				        CAccount objNewAccount = new CAccount ((String)aobjBody[1], dStartedBalance);
					m_objAccountList.addElement (objNewAccount);

					/* Spezifikation der Rckgabenachricht setzen */
	        			vctResponseSpec.addElement ("ACCOUNT_OK");
				        vctResponse.addElement (new Double(objNewAccount.getCurrentBalance()));

				} catch (Exception ex) {

				        vctResponseSpec.clear();
	        			vctResponseSpec.addElement ("UNKNOWN_REQUEST");
				        vctResponse.addElement (sCommand);
				        vctResponse.addElement (ex);
				        vctResponse.addElement (ex.getMessage());

				}

		        /* Schliet den Account mit dem bergebenen Benutzernamen */
		        } else if ( sCommand.equals("CLOSE_ACCOUNT") ) {

				try {

				        /* Die ausgelagerte Methode zum Schlieen des Accounts aufrufen */
					func_Close_Account ( vctResponseSpec, vctResponse, (String)aobjBody[1] );

				} catch (Exception ex) {

				        vctResponseSpec.clear();
				        vctResponseSpec.addElement ("UNKNOWN_REQUEST");
				        vctResponse.addElement (sCommand);
				        vctResponse.addElement (ex);
				        vctResponse.addElement (ex.getMessage());

				}

		        /* Gibt den Kontostand des Accounts mit dem bergebenen Benutzernamen zurck */
		        } else if ( sCommand.equals("GET_BALANCE") ) {

				try {

				        /* Die ausgelagerte Methode zur Ermittlung des aktuellen Kontostands aufrufen */
					func_Get_Balance (vctResponseSpec, vctResponse, (String)aobjBody[1] );

				} catch (Exception ex) {

				        vctResponseSpec.clear();
				        vctResponseSpec.addElement ("UNKNOWN_REQUEST");
				        vctResponse.addElement (sCommand);
				        vctResponse.addElement (ex);
				        vctResponse.addElement (ex.getMessage());

				}

		        /* Kauft Aktien in der gewnschten Menge mit dem angegebenen Benutzer  */
		        } else if ( sCommand.equals("BUY_STOCK") ) {

				try {

				        /* Die ausgelagerte Methode zum Aktienkauf aufrufen */
					func_Buy_Stock ( vctResponseSpec, vctResponse, (String)aobjBody[1],
	        			        (String)aobjBody[2], ((Integer)aobjBody[3]).intValue() );

				} catch (Exception ex) {

				        vctResponseSpec.clear();
	        			vctResponseSpec.addElement ("UNKNOWN_REQUEST");
				        vctResponse.addElement (sCommand);
				        vctResponse.addElement (ex);
				        vctResponse.addElement (ex.getMessage());

				}

		        /* Verkauft Aktien in der gewnschten Menge mit dem angegebenen Benutzer  */
		        } else if ( sCommand.equals("SELL_STOCK") ) {

				try {

				        /* Die ausgelagerte Methode zum Aktienverkauf aufrufen */
					func_Sell_Stock ( vctResponseSpec, vctResponse, (String)aobjBody[1],
	        			        (String)aobjBody[2], ((Integer)aobjBody[3]).intValue() );

				} catch (Exception ex) {

				        vctResponseSpec.clear();
				        vctResponseSpec.addElement ("UNKNOWN_REQUEST");
				        vctResponse.addElement (sCommand);
				        vctResponse.addElement (ex);
				        vctResponse.addElement (ex.getMessage());

				}

			/* Keine bekannte request-Spezifikation angegeben */
		        } else {

				/* Wenn keine bekannte request-Spezifikation angegeben wurde,
				wird UNKNOWN_REQUEST mit der request-Spezifikation zurckgegeben. */
				vctResponseSpec.clear();
				vctResponseSpec.addElement ("UNKNOWN_REQUEST");
				vctResponse.addElement ( sCommand );

		        }

		        /* Antwortnachricht zusammensetzen und versenden */
		        vctResponse.add (0, vctResponseSpec.get(0));
		        Object[] aobjResponse = vctResponse.toArray ();
			depositMessage (new AMETASMessage(puidClient, midRequestID, aobjResponse));

		        /* Ausgabe der Antwortnachricht */
		        // enhancedOutput ("<--- response: " + vctResponse);

		}

		/**
		 * Methode sucht den Datensatz fr einen Account mit Benutzernamen
		 * sUsername. Wird sie bei der Suchaufgabe nicht fndig, werden die
		 * sResponseSpec- und vctResponse-Parameter besetzt und die Methode
		 * kehrt zum Aufrufer zurck.
		 *
		 * Bei Fund hingegen wird das Objekt objAccount mit dem gefundenen
		 * Datensatz zurckgegeben.
		 */
		private CAccount helper_FindUser (Vector vctResponseSpec, Vector vctResponse, String sUsername) {

		        /* Variable fr Rckgabeobjekt */
		        CAccount objAccount;

			try {
				/* Benutzeraccount in Accountliste suchen */
	        		objAccount = m_objAccountList.getElementByName (sUsername);

				/* Prfen, ob Account gefunden werden konnte, sonst zurck */
				if (objAccount == null) {
				        vctResponseSpec.clear();
	        			vctResponseSpec.addElement ("USER_UNKNOWN");
	        		        vctResponse.addElement (sUsername);
		        		return null;
				}

			} catch ( Exception ex ) {
				vctResponseSpec.clear();
        			vctResponseSpec.addElement ("USER_UNKNOWN");
			        vctResponse.addElement (sUsername);
				return null;
			}

		        /* Die gewnschten Objekte wurden gefunden -> damit zurckkehren */
		        vctResponseSpec.clear();
		        return objAccount;

		}

		/**
		 * Methode sucht die Datenstze fr eine Aktie mit sStockname als Namen.
		 * Wird sie bei einer der Suchaufgaben nicht fndig, werden die
		 * sResponseSpec- und vctResponse-Parameter besetzt und die Methode
		 * kehrt zum Aufrufer zurck.
		 *
		 * Bei Fund hingegen wird das Objekt objStock mit dem gefundenen
		 * Datensatz zurckgegeben.
		 */
		private CStock helper_FindStock (Vector vctResponseSpec, Vector vctResponse, String sStockname) {

		        /* Variable fr Rckgabeobjekt */
		        CStock objStock;

		        try {
				/* Aktie in Aktienliste suchen */
	        		objStock = m_stlTrading.getElementByName (sStockname);

				/* Prfen, ob Aktie gefunden werden konnte, sonst zurck */
				if ((objStock == null) || ((objStock != null) && (!objStock.isTraded()))) {
				        vctResponseSpec.clear();
	        			vctResponseSpec.addElement ("NOT_TRADED");
	        		        vctResponse.addElement (sStockname);
		        		return null;
				}

		        }  catch ( Exception ex ) {
				vctResponseSpec.clear();
        			vctResponseSpec.addElement ("NOT_TRADED");
			        vctResponse.addElement (sStockname);
				return null;
			}

		        /* Die gewnschten Objekte wurden gefunden -> damit zurckkehren */
		        vctResponseSpec.clear();
		        return objStock;

		}

		/**
		 * In dieser Methode wird der Kauf von Aktien bearbeitet, der durch einen
		 * BUY_STOCK-request angestoen wird.
		 */
		private void func_Buy_Stock (Vector vctResponseSpec, Vector vctResponse,
			String sUsername, String sStockname, int iAmount) {

		        /* Mit diesen Objekten haben wir Zugriff auf die Aktie und den Account */
		        CStock objStock = null;
		        CAccount objAccount = null;

		        /* Benutzeraccount suchen -> bei Fehler: zurck */
		        objAccount = helper_FindUser (vctResponseSpec, vctResponse, sUsername);
		        if (vctResponseSpec.size() > 0) {
				return;
		        }

		        /* Aktie suchen -> bei Fehler: zurck */
		        objStock = helper_FindStock (vctResponseSpec, vctResponse, sStockname);
		        if (vctResponseSpec.size() > 0) {
				return;
		        }

		        /* Jetzt haben wir die ntigen Account- und Stock-Objekte und knnen
		        einen Aktienkauf wagen. Mgliche Fehlerquelle ist noch, dass der Account
		        ber nicht genug Kontostand verfgt. */

		        /* Kaufpreis der iAmount Aktien zu objStock.getPriceCurrent() Agentendollar */
		        double dPriceCurrent = objStock.getPriceCurrent();
		        double dValue = (double)(iAmount * dPriceCurrent);
		        if (objAccount.getCurrentBalance() < dValue) {

				/* Zu wenig Agentendollar auf Account, daher zurckspringen */
				vctResponseSpec.clear();
        			vctResponseSpec.addElement ("NOT_ENOUGH_CREDIT");
			        vctResponse.addElement (new Double(objAccount.getCurrentBalance()));
				return;

		        }

		        /* Jetzt schnell die Aktie kaufen, nicht das in nem anderen Thread die Aktie
		        gekauft wird oder sich der Kurs verndert. Das verlangt eigentlich nach einer
		        Transaktion, aber soweit wollen wir hier nicht gehen. */
		        CDepotEntry objNewDepotEntry = new CDepotEntry (sStockname, dPriceCurrent, dPriceCurrent, iAmount);

		        /* Neuen Depoteintrag zu Accountdepot hinzufgen und Kaufpreis abbuchen
		        (allerdings nur, wenn > 0 Aktien (iAmount) geordert wurden) */
		        if (iAmount > 0) {
				objAccount.bookDepotEntry (objNewDepotEntry);
		        }

		        /* Statusmeldung ber Kauferfolg an Aufrufer zurck */
		        vctResponseSpec.clear();
       			vctResponseSpec.addElement ("STOCK_BOUGHT");
		        vctResponse.addElement (sStockname);
		        vctResponse.addElement (new Integer(iAmount));
		        vctResponse.addElement (new Double(dValue));
			return;

		}

		/**
		 * In dieser Methode wird der Verkauf von Aktien bearbeitet, der durch einen
		 * SELL_STOCK-request angestoen wird.
		 */
		private void func_Sell_Stock (Vector vctResponseSpec, Vector vctResponse,
			String sUsername, String sStockname, int iAmount) {

		        /* Mit diesen Objekten haben wir Zugriff auf die Aktie und den Account */
		        CStock objStock = null;
		        CAccount objAccount = null;

		        /* Benutzeraccount suchen -> bei Fehler: zurck */
		        objAccount = helper_FindUser (vctResponseSpec, vctResponse, sUsername);
		        if (vctResponseSpec.size() > 0) {
				return;
		        }

		        /* Aktie suchen -> bei Fehler: zurck */
		        objStock = helper_FindStock (vctResponseSpec, vctResponse, sStockname);
		        if (vctResponseSpec.size() > 0) {
				return;
		        }

		        /* Jetzt haben wir die ntigen Account- und Stock-Objekte und knnen
		        einen Aktienverkauf wagen. Mgliche Fehlerquelle ist noch, dass die Aktie
		        gar nicht im Depot ist. */
		        CDepot objThisDepot = objAccount.getDepot();
		        if (objThisDepot.getElementByName(sStockname) == null) {
				vctResponseSpec.clear();
       			        vctResponseSpec.addElement ("NOT_IN_DEPOT");
				vctResponse.addElement (sStockname);
				vctResponse.addElement (sUsername);
				return;
		        }

		        /* Jetzt schnell die Aktie verkaufen, nicht das in nem anderen Thread die Aktie
		        verkauft wird oder sich der Kurs verndert. Das verlangt eigentlich nach einer
		        Transaktion, aber soweit wollen wir hier nicht gehen. */
		        int iActuallySoldAmount = 0;
			double dValue = objAccount.sellStocks (objStock, iAmount, iActuallySoldAmount);

		        /* Statusmeldung ber Verkauferfolg an Aufrufer zurck */
		        vctResponseSpec.clear();
       			vctResponseSpec.addElement ("STOCK_SOLD");
		        vctResponse.addElement (sStockname);
		        vctResponse.addElement (new Double(dValue));
			return;

		}

		/**
		 * In dieser Methode wird die Liste der Depoteintrge zusammengestellt und
		 * zurckgegeben, was von einem GET_DEPOT-request angestoen wird.
		 */
		private void func_Get_Depot (Vector vctResponseSpec, Vector vctResponse, String sUsername) {

		        /* Mit diesem Objekt haben wir Zugriff auf den Account */
		        CAccount objAccount = null;

		        /* Benutzeraccount suchen -> bei Fehler: zurck */
		        objAccount = helper_FindUser (vctResponseSpec, vctResponse, sUsername);
		        if (vctResponseSpec.size() > 0) {
				return;
		        }

		        /* Das Depot durchgehen und die einzelnen Depoteintrge in den
		        Ausgabevector schreiben */
		        CDepot objDepot = objAccount.getDepot();
		        for (int i = 0; i < objDepot.size(); i++) {

				/* Eintrag aus Depot holen */
				CDepotEntry objCurDepEntry = objDepot.getDepotEntry(i);

				/* Den entsprechenden Kurswert aus m_stlTrading holen
				und in objCurDepEntry einsetzen */
				CStock objThisStock = m_stlTrading.getElementByName(objCurDepEntry.getName());
				if (objThisStock != null) {

				        /* Prfen, ob Aktie gehandelt wird */
				        if (objThisStock.isTraded()) {
						objCurDepEntry.setPriceCurrent (objThisStock.getPriceCurrent());
				        } else {
						objCurDepEntry.setPriceCurrent (0.0);
					}

				} else {

				        /* Da Aktie nicht in Handelsliste gefunden wurde, wird 0.0
				        als aktueller Kurs zurckgegeben */
					objCurDepEntry.setPriceCurrent (0.0);

				}

				/* mit durch Ametas-Interface AMETASSendable vorgegebener
				In-Objekt-Array-Serialisierungsfunktion in Rckgabevector
				schreiben */
				vctResponse.addElement (objCurDepEntry.getData());

		        }

		        /* Rckgabe mirt Spezifikation, dass Depot geliefert wird */
		        vctResponseSpec.addElement ("DEPOT_IS");
		        return;

		}

		/**
		 * Mit dieser Methode wird der aktuelle Kontostand eines Accounts
		 * zurckgegeben, was von einem GET_BALANCE-request angestoen wird.
		 */
		private void func_Get_Balance (Vector vctResponseSpec, Vector vctResponse, String sUsername) {

		        /* Mit diesem Objekt haben wir Zugriff auf den Account */
		        CAccount objAccount = null;

		        /* Benutzeraccount suchen -> bei Fehler: zurck */
		        objAccount = helper_FindUser (vctResponseSpec, vctResponse, sUsername);
		        if (vctResponseSpec.size() > 0) {
				return;
		        }

		        /* Kontostand in Ausgabevector schreiben */
			double dCurrentBalance = objAccount.getCurrentBalance();
		        vctResponse.addElement (new Double(dCurrentBalance));

		        /* Rckgabe mirt Spezifikation, dass Depot geliefert wird */
		        vctResponseSpec.addElement ("CREDIT_IS");
		        return;

		}

		/**
		 * Mit dieser Methode wird der Account mit dem angegeben Benutzernamen
		 * geschlossen, was von einem CLOSE_ACCOUNT-request angestoen wird. Dazu
		 * mu der Account natrlich vorhanden sein und gem Aufgabenspezifikation
		 * auch das Depot leer.
		 */
		private void func_Close_Account (Vector vctResponseSpec, Vector vctResponse, String sUsername) {

		        /* Mit diesem Objekt haben wir Zugriff auf den Account */
		        CAccount objAccount = null;

		        /* Benutzeraccount suchen -> bei Fehler: zurck */
		        objAccount = helper_FindUser (vctResponseSpec, vctResponse, sUsername);
		        if (vctResponseSpec.size() > 0) {
				return;
		        }

		        /* Prfen, ob Depot leer ist. Wenn nicht leer, dann kein Schlieen mglich */
		        CDepot objDepot = objAccount.getDepot();
		        if (objDepot.size() > 0) {
				vctResponseSpec.addElement ("DEPOT_NOT_EMPTY");
				vctResponse.addElement (sUsername);
		        }

		        /* Kontostand in Ausgabevector schreiben */
			double dCurrentBalance = objAccount.getCurrentBalance();
		        vctResponse.addElement (new Double(dCurrentBalance));

		        /* und Account aus Liste entfernen! */
			m_objAccountList.removeElement (objAccount);

		        /* Rckgabe mit Spezifikation, dass Depot geliefert wird */
		        vctResponseSpec.addElement ("ACCOUNT_CLOSED");
		        return;

		}

	}

}