package ap.stockserviceproxy;

import java.util.*;
import AMETAS.place.AMETASPlaceUser;
import AMETAS.place.AMETASPlaceUserDriverIf;
import AMETAS.servicedev.*;
import ap.data.*;

/**
 * <p>Synchroner Dienst-Proxy (&quot;service-proxy&quot;) f&uuml;r &quot;Ametas&quot;
 * (<a href="http://www.ametas.de" target="_blank">www.Ametas.de</a>),
 * der die Funktionalitt des Dienstes &quot;StockService&quot; kapselt.</p>
 *
 * <p>Diese Klasse wurde im Rahmen des Praktikums &quot;Mobile Agenten&quot; im Wintersemester 2000/01
 * an der Professur &quot;Verteilte Systeme und Betriebssysteme&quot; des Fachbereiches Informatik
 * und Biologie an der Johann Wolfgang Goethe-Universit&auml;t in Frankfurt erstellt:</p>
 *
 * <p>L&ouml;sung zu Aufgabenblatt 6, Aufgabe 1</p>
 *
 * <p>Autoren (Gruppe &quot;Orange&quot;):</p>
 * <ul>
 * <li>Martin Klossek (<a href="mailto:klossek@informatik.uni-frankfurt.de">klossek@informatik.uni-frankfurt.de</a>)</li>
 * <li>Fabian Wleklinski (<a href="mailto:wleklins@informatik.uni-frankfurt.de">wleklins@informatik.uni-frankfurt.de</a>)</li>
 * </ul>
 *
 * @see <a href="http://www.ametas.de" target="_blank">www.Ametas.de</a>
 */
public class StockServiceProxy extends AMETASSyncServiceProxy {

        public StockServiceProxy( AMETASPlaceUser puClient, AMETASPlaceUserDriverIf drvClientDriver ) {
                super( puClient, drvClientDriver );

                /* Fabian Wleklinski: Der Name der Service, fr den wir einen Proxy
                bereitstellen mchten */
                m_sServiceName = "ap.stockservice.StockService";
        }

        /**
         * Ermittelt den aktuellen Preis einer Aktie
         *
         * @param stockname Bezeichnung der Aktie
         * @return Der aktuelle Preis der Aktie, sofern diese bekannt ist,
         * -1 anderenfalls.
         */
        public double getCurrentPrice( String stockname ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "GET_CURRENT_PRICE", stockname };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "PRICE_IS" ) ) {
                        return ((Double) m_aobjResponseBody[2]).doubleValue();
                } else if( m_sResponseTypeSpec.equals( "NOT_TRADED" ) ) {
                        return -1;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Ermittelt den Ausgabepreis einer Aktie
         *
         * @param stockname Bezeichnung der Aktie
         * @return Der Ausgabepreis der Aktie, sofern diese bekannt ist,
         * -1 anderenfalls.
         */
        public double getIPOPrice( String stockname ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "GET_IPO_PRICE", stockname };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "IPO_PRICE_IS" ) ) {
                        return ((Double) m_aobjResponseBody[2]).doubleValue();
                } else if( m_sResponseTypeSpec.equals( "NOT_TRADED" ) ) {
                        return -1;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Ermittelt alle gehandelten Aktien
         *
         * @return Die Liste aller gehandelten Aktien
         */
        public CStockList getStockList( boolean doGetStockValues ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "GET_STOCK_LIST" };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "STOCKS_ARE" ) ) {
                        CStockList result = new CStockList();

                        /* Fabian Wleklinski: Die Rckgabe kopieren wir in ein
                        lokales Feld, sonst bekommen wir ReEntrance Probleme */
                        Object[] objStockResponse = m_aobjResponseBody;

                        for (int i=1; i < objStockResponse.length; i++ ) {
                                CStock curStock = new CStock( (String) objStockResponse[ i ], 0, 0 );

                                if (doGetStockValues) {
                                        double dCurrentPrice = getCurrentPrice( curStock.getName() );
                                        curStock.setPriceCurrent( dCurrentPrice );

                                        double dIPOPrice = getIPOPrice( curStock.getName() );
                                        curStock.setPriceIPO( dIPOPrice );

                                }

                                result.add( curStock );
                        }

                        return result;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Ermittelt das Depot des Benutzers
         *
         * @param username Der Name des Benutzers
         * @return Das Depot des Benutzers, sofern der Benutzer erkannt ist,
         * &quot;null&quot; sonst.
         */
        public CDepot getDepot( String username ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "GET_DEPOT", username };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "DEPOT_IS" ) ) {
                        CDepot result = new CDepot();

                        for (int i=1; i < m_aobjResponseBody.length; i++ ) {
                                Object[] objArray = (Object[]) m_aobjResponseBody[ i ];
                                result.addElement( new CDepotEntry (objArray) );
                        }

                        /*for (int i=1; i < m_aobjResponseBody.length; i=i+4 ) {
                                curDepotEntry = new CDepotEntry( (Strm_aobjResponseBody[ i ];
                                Vector aobjData = (Vector) m_aobjResponseBody[ i ];
                                result.addElement( new CDepotEntry( aobjData ) );
                        }*/

			result.updateDepotInformation();

                        return result;
                } else if( m_sResponseTypeSpec.equals( "USER_UNKNOWN" ) ) {
                        return null;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Legt ein neues Benutzerkonto an
         *
         * @param username Der Name des Benutzers
         * @param amount Der anf&auml;ngliche Einzahlungsbetrag
         * @return Der Kontostand des neuen Kontos
         */
        public double newAccount( String username, int amount ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "NEW_ACCOUNT", username, new Integer( amount ) };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "ACCOUNT_OK" ) ) {
                        return ((Double) m_aobjResponseBody[1]).doubleValue();
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Schlie&szlig;t ein bestehendes Benutzerkonto
         *
         * @param username Der Name des Benutzers
         * @return Im Erfolgsfall wird der Kontostand zurckgeliefert, der
         * immer gr&ouml;&szlig;er oder gleich Null ist. Falls das Depot
         * nicht leer ist, wird der Wert -1 zur&uuml;ckgegeben, falls der
         * Benutzer unbekannt ist, der Wert -2.
         */
        public double closeAccount( String username ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "CLOSE_ACCOUNT", username };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "ACCOUNT_CLOSED" ) ) {
                        return ((Double) m_aobjResponseBody[1]).doubleValue();
                } else if (m_sResponseTypeSpec.equals( "USER_UNKNOWN" ) ) {
                        return -2;
                } else if (m_sResponseTypeSpec.equals( "DEPOT_NOT_EMPTY" ) ) {
                        return -1;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Ermittelt den Kontostand eines Benutzerkontos
         *
         * @param username Der Name des Benutzers
         * @return Im Erfolgsfall wird der Kontostand zurckgeliefert, der
         * immer gr&ouml;&szlig;er oder gleich Null ist. Falls der
         * Benutzer unbekannt ist, wird der Wert -1 zur&uuml;ckgegeben.
         */
        public double getBalance( String username ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "GET_BALANCE", username };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "CREDIT_IS" ) ) {
                        return ((Double) m_aobjResponseBody[1]).doubleValue();
                } else if( m_sResponseTypeSpec.equals( "USER_UNKNOWN" ) ) {
                        return -1;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Kauft ein Aktienpacket
         *
         * @param username Der Name des Benutzers
         * @param stockname Der Name der zu kaufenden Aktie
         * @param count Die Anzahl der zu kaufenden Aktien
         * @return Der zur&uuml;ckgegebene CDepotEntry beinhaltet das
         * gekaufte Aktienpacket, inklusive der Anzahl der enthaltenen
         * Aktien, und dem gesamt-Kaufpreis. Falls ein Fehler auftritt, wird
         * hier null zur&uuml;ckgegeben.
         */
        public CDepotEntry buyStock( String username, String stockname, int count ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "BUY_STOCK", username, stockname, new Integer( count ) };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "STOCK_BOUGHT" ) ) {
                        int realCount = ((Integer) m_aobjResponseBody[2]).intValue();
                        CDepotEntry result = new CDepotEntry( (String) m_aobjResponseBody[1], ((Double) m_aobjResponseBody[3]).doubleValue()/realCount, ((Double) m_aobjResponseBody[3]).doubleValue()/realCount, realCount );
                        return result;
                } else if( m_sResponseTypeSpec.equals( "USER_UNKNOWN" ) ) {
                        return null;
                } else if( m_sResponseTypeSpec.equals( "NOT_TRATED" ) ) {
                        return null;
                } else if( m_sResponseTypeSpec.equals( "NOT_ENOUGH_CREDIT" ) ) {
                        return null;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }

        /**
         * Verkauft ein Aktienpacket
         *
         * @param username Der Name des Benutzers
         * @param stockname Der Name der zu verkaufenden Aktie
         * @param count Die Anzahl der zu verkaufenden Aktien
         * @return Der zur&uuml;ckgegebene CDepotEntry beinhaltet das
         * vergekaufte Aktienpacket, inklusive der Anzahl der enthaltenen
         * Aktien, und dem gesamt-Verkaufspreis. Falls ein Fehler auftritt, wird
         * hier null zur&uuml;ckgegeben.
         */
        public CDepotEntry sellStock( String username, String stockname, int count ) throws ProxyTimeoutException, ProxyServiceErrorException {

                /* Fabian Wleklinski: Baue Nutzlast der Anfragenachricht */
                Object[] aobjBody = { "SELL_STOCK", username, stockname, new Integer( count ) };

                /* Fabian Wleklinski: Lege Anfrage ab und warte auf Antwort oder Timeout */
                depositRequestAndWait( aobjBody );

                // Werte Antwort aus. Hierhin gelangen wir nur, wenn von
                // depositRequestAndWait keine ProxyTimeoutException geworfen
                // wurde.
                if (m_sResponseTypeSpec.equals( "STOCK_SOLD" ) ) {
                        CDepotEntry result = new CDepotEntry( (String) m_aobjResponseBody[1], ((Double) m_aobjResponseBody[2]).doubleValue()/count, ((Double) m_aobjResponseBody[2]).doubleValue()/count, count );
                        return result;
                } else if( m_sResponseTypeSpec.equals( "USER_UNKNOWN" ) ) {
                        return null;
                } else if( m_sResponseTypeSpec.equals( "NOT_TRATED" ) ) {
                        return null;
                } else if( m_sResponseTypeSpec.equals( "NOT_IN_DEPOT" ) ) {
                        return null;
                } else if( m_sResponseTypeSpec.equals( "UNKNOWN_REQUEST" ) ) {
                        throw new ProxyServiceErrorException( m_aobjResponseBody[1].toString() );
                } else {
                        throw new ProxyServiceErrorException( "Response: " + m_sResponseTypeSpec );
                }
        }
}

