package ap.stockua;

import java.util.*;
import java.text.SimpleDateFormat;

import java.awt.Color;

import javax.swing.*;
import javax.swing.JTable;
import javax.swing.table.*;

import ap.data.*;

/**
 * Title:        Ametas StockService
 * Description:  Klasser der im Benutzeradapter verwendeten Bilanztabelle
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CBalanceTable extends JTable implements java.io.Serializable {

	/** Ein Membervariable fr das DataModel der Tabellenklassen (zum Anlegen von Zeilen) */
	CBalanceTableModel m_objDataModel = null;

	/** Das Spaltenmodel fr die Anordnung und Breite der Spalten (scheinbar macht hier der GC Murks) */
	TableColumnModel m_objColModel = null;


	/**
	 * Der Konstructor der Tabelle, hier werden einige Initialisierungen vorgenommen
	 */
	public CBalanceTable() {

		/* Neues DataModel-Objekt anlegen und fr Tabelle setzen (damit werden
		die in der Tabelle angezeigten Daten verwaltet. */
		m_objDataModel = new CBalanceTableModel();
		setModel (m_objDataModel);

		/* Set the cellEditor variable */
		CBalanceTableCellEditor objEditor = new CBalanceTableCellEditor (new JTextField());
		setCellEditor(objEditor);

		/* Spalten-Model setzen und mit Werten fllen */
		m_objColModel = getColumnModel();
		m_objColModel.getColumn(0).setPreferredWidth(100);
		m_objColModel.getColumn(1).setPreferredWidth(50);
		m_objColModel.getColumn(2).setPreferredWidth(50);
		m_objColModel.getColumn(3).setPreferredWidth(50);
		m_objColModel.getColumn(4).setPreferredWidth(50);
		m_objColModel.getColumn(5).setPreferredWidth(20);
		m_objColModel.getColumn(6).setPreferredWidth(50);
		m_objColModel.getColumn(7).setPreferredWidth(30);
		m_objColModel.getColumn(8).setPreferredWidth(130);
		m_objColModel.getColumn(9).setPreferredWidth(100);
		m_objColModel.getColumn(10).setPreferredWidth(40);

	}

	/**
	 * Mit dieser Methode wird die Datenhaltung fr die Tabelle an das DataModel-Objekt
	 * bergeben, um die Prsentation der Daten zu realisieren.
	 */
	public void setAgentWrapperList (CStockAgentWrapperList vecAgentList) {

		/* Dem DataModel die Liste der AgentWrapper bergeben */
		m_objDataModel.setAgentWrapperList (vecAgentList);

	}

	/**
	 * Mit dieser Methode wird der Bilderhash an die Tabelle bergeben, die
	 * eine Anzahl von eingeladenen Bilder enthlt.
	 */
	public void setImages (Hashtable hashImages) {

		/* Dem DataModel den Bilderhash bergeben, dort werden die Bilder
		schlielich zum Wrzen der Anzeige gebraucht. */
		m_objDataModel.setImages (hashImages);

	}

        /**
         * Fgt dem Datenspeicher einen neuen Datensatz hinzu
         */
	public void addRow (CStockAgentWrapper objNewWrapper) {

		/* Dem DataModel das neue Objekt bergeben, dort wird dann die Anzeige aktualisiert. */
		m_objDataModel.addRow (objNewWrapper);

	}

        /**
         * Updated die Ausgabe bei Vernderung des bergebenen Datensatzes
         */
	public void updateRow (CStockAgentWrapper objNewWrapper) {

		/* Dem DataModel den Updatebefehl mitteilen */
		m_objDataModel.updateRow (objNewWrapper);

	}

        /**
         * Lscht eine Tabellenzeile und updatet die Ausgabe
         */
	public void deleteRow (CStockAgentWrapper objNewWrapper) {

		/* Dem DataModel den Updatebefehl mitteilen */
		m_objDataModel.deleteRow (objNewWrapper);

	}

	/**
	 * Methode berschreibt getCellRenderer-Methode der Elternklasse, um eigene
	 * Renderer fr bestimmte Zellen definieren zu knnen -> hier die Summenzeile
	 */
	public TableCellRenderer getCellRenderer(int row, int column) {

		/* Wir holen uns den Renderer der Elternklasse JTable, denn der ist gut! und setzen
		die Hintergrundfarbe auf wei (fr alle Zeilen auer Summenzeile) */
		DefaultTableCellRenderer xxx = (DefaultTableCellRenderer)super.getCellRenderer(row, column);
		xxx.setBackground (Color.white);

		/* fr letzte Zeile = Summenzeile wird spezieller Renderer verwendet */
		if (row == m_objDataModel.getRowCount() -1) {

		        /* fr die Summenzeile setzen wir eine andere Hintergrundfarbe als wei */
			xxx.setBackground (new Color(234, 234, 255));
		        return xxx;

		}

		/* und auch hier den Renderer zurckgeben */
		return xxx;
	}


	/* Inner class: Celleditor (ist vorerst nicht relevant, da keine editierbare Tabelle!) */
	static class CBalanceTableCellEditor extends DefaultCellEditor implements java.io.Serializable {

		public CBalanceTableCellEditor (JTextField textField) {

			super (textField);

		}

	}

	/* Die Datenmodell-Klasse fr die Tabelle zur Darstellung der Bilanzdaten */
	static class CBalanceTableModel extends AbstractTableModel implements java.io.Serializable {

		/**
		 * In dieser Membervariable wird das Objekt auf die Daten der Bilanztabelle
		 * gespeichert. Die Daten werden von den StockInfoAgents angeliefert, was hier
		 * aber nicht zu erkennen und irrelevant sein sollte!
		 */
		CStockAgentWrapperList m_vecAgentList = null;

		/** Die Membervariable nimmt den Bilderhash fr kleine Pictogramme auf */
		Hashtable m_hashImages = null;

		/** die Spaltenberschriften... */  // static final
		String[] m_asColumnNames = new String[] {
		        "Agentenname",
		        "Guthaben",
		        "kauf Wert",
		        "akt Wert",
		        "akt Gew",
		        "+/-",
		        "real Gew",
		        "Anzahl",
		        "Aktien",
		        "Stelle",
		        "Zeit"
		};

		/** ...und die Spaltendatentypen. */
		Class[] m_asColumnTypes = new Class[] {
		        String.class,
		        Double.class,
		        Double.class,
		        Double.class,
		        Double.class,
		        ImageIcon.class,
		        Double.class,
		        Integer.class,
		        String.class,
		        String.class,
		        String.class
		};

		/**
		 * Der Konstruktor
		 */
		public CBalanceTableModel () {

		        /* Datenhaltungsklasse initialsieren */
		        m_vecAgentList = new CStockAgentWrapperList ();

		}

		/**
		 * Mit dieser Methode wird die Verbindung zur Datenhaltung der Tabelle gesetzt.
		 */
		public void setAgentWrapperList (CStockAgentWrapperList vecAgentList) {

		        /* Die Liste in der zugehrigen Membervariable speichern */
	        	m_vecAgentList = vecAgentList;

		}

		/**
		 * Mit der Methode wird dem DataModel der Bilderhash zur Wrzung der
		 * Tabellenausgabe bergeben.
		 */
		public void setImages (Hashtable hashImages) {

		        /* den Bilderhash in die Membervariable eintragen */
	        	m_hashImages = hashImages;

		}

		/**
		 * Liefert die Anzahl der Eintrge - also der Zeilen - der Tabelle zurck.
		 */
		public int getRowCount () {

		        /* Anzahl der Eintrge in m_vecAgentList +1 zurckgeben (+1 wegen der Summenzeile) */
		        return m_vecAgentList.size() +1;

		}

		/**
		 * Liefert die Anzahl der Tabellensppalten zurck (recht konstanter Wert!)
		 */
		public int getColumnCount() {

		        return m_asColumnNames.length;

		}

		public String getColumnName (int column) {
		        return m_asColumnNames[column];
		}

		public Class getColumnClass (int column) {
		        return m_asColumnTypes[column];
		}

		/**
		 * Die Methode gibt - wie der Name schon sagt - zurck, ob die Zelle (Zeile, Spalte)
		 * bearbeitet werden kann (true) oder readonly (false).
		 */
		public boolean isCellEditable(int rowIndex, int columnIndex) {

		        /* Keine Spalte soll bearbeiten werden, wobei wir etwas am Guthaben rummschrauben knnten :) */
		        return false;

		}

		/* Gibt einen Wert zurck */
		public Object getValueAt(int row, int column) {

		        /* Bereichsberprfung vornehmen (es drfen natrlich nur vorhandene
		        Daten aus dem Datenspeicher geholt werden!) */
		        if ((row < 0) || (row >= getRowCount())) { return null; }

		        /* Bentigten Agentwrapper und zugehriges Depot zur schnelleren Abfrage zwischenspeichern */
		        CStockAgentWrapper objCurrentAgentWrapper = null;
		        CDepot objCurrentDepot = null;

		        /* in bSum vermerken ob die Summenzeile angefordert ist oder nicht */
		        boolean bSum = false;

		        /* Oben angesprochene Operationen ausfhren */
		        if (row < getRowCount() -1) {
			        objCurrentAgentWrapper = m_vecAgentList.getStockAgentWrapper(row);
				objCurrentDepot = (objCurrentAgentWrapper!=null)?objCurrentAgentWrapper.getDepot():null;
		        } else {
				objCurrentDepot = m_vecAgentList.getSumDepot();
				bSum = true;
		        }

		        /* Abhngig von der angeforderten Zeile/Spalte wird ein unterschiedlicher Wert ausgegeben */
		        switch (column) {
		        case 0: // Agentenname

				/* In der letzten Zeile wird die Summe ausgegeben, in den
				vorangehenden die Informationen aus vecAgentList */
				if (bSum) {
				        /* hier letzte Zeile, also Text "Summe:" ausgeben */
				        return "Summe:";
				} else {
				        return objCurrentAgentWrapper.getAgentName();
				}

		        case 1: // Guthaben

				return objCurrentDepot.getAccountBalance();

		        case 2: // Gesamtkaufpreis der Aktien im Depot

				return objCurrentDepot.getTotalPriceBought();

		        case 3: // aktueller Gesamtwert der Aktien im Depot

				return objCurrentDepot.getTotalPriceCurrent();

		        case 4: // Aktueller Gewinn / Verlust der Akten im Depot

				return objCurrentDepot.getGainCurrent();

		        case 5: // Tendenzbild ausgeben

				switch (objCurrentDepot.getTendency()) {
				case 1:
				        return (ImageIcon)(m_hashImages.get("arrow_up"));
				case 2:
				        return (ImageIcon)(m_hashImages.get("arrow_down"));
				case 3:
				        return (ImageIcon)(m_hashImages.get("arrow_right"));
				default:
				        return null;
				}

		        case 6: // Realisierter Gewinn im Gesamtverlauf des Handels

				return objCurrentDepot.getGainOverall();

		        case 7: // Anzahl der Aktien im Depot

				return objCurrentDepot.getTotalAmount();

		        case 8: // Aktien im Depot

				if (bSum) {
				        return "";
				} else {

				        /* Aktienkrzel und Anzahl der Aktien (Format = KRZ(2)) des
				        aktuellen Depots ausgeben*/
				        String sStockOutput = "";
				        CDepotEntry objCurrentDepotEntry = null;
				        for (int i = 0; i < objCurrentDepot.size(); i++) {

						/* Ausgabestring fr Tabellenzelle zusammensetzen */
						objCurrentDepotEntry = objCurrentDepot.getDepotEntry(i);
						if (i > 0) {
						        sStockOutput += ", " + objCurrentDepotEntry.getName() + "(" + objCurrentDepotEntry.getAmount() + ")";
						} else {
						        sStockOutput = objCurrentDepotEntry.getName() + "(" + objCurrentDepotEntry.getAmount() + ")";
						}
				        }

					return sStockOutput;
				}

		        case 9: // Stellenname

				if (bSum) {
				        return "";
				} else {
				        return objCurrentAgentWrapper.getPlaceName();
				}

		        case 10: // Datum der letzten Datenlieferung von StockInfoAgenten fr diesen StockAgent

			        /* Zeit aus Datumswert extrahieren */
				SimpleDateFormat objFormatter = new SimpleDateFormat ("hh:mm:ss");
				return objFormatter.format(objCurrentDepot.getDateLastUpdated());

		        }

		        return null;

		}

		/* Schreibt ein Objekt in die Zelle (spezifiziert durch Zeile und Spalte).
		Fr unsere Zwecke (also in unserem TableModell), wird aber nur das Setzen
		der Anzahlspalte (Spalte 0) erlaubt. */
		public void setValueAt(Object objValue, int row, int column) {

		        /* Bereichsberprfung vornehmen (es drfen natrlich nur vorhandene
		        Daten im Datenspeicher modifiziert werden!) */
		        // if ((row < 0) || (row >= m_objRecordList.size())) { return; }

		        /* newAmount setzen */
		        //Integer newAmount = (Integer)objValue;

		        /* Nur Anzahlspalte (= Spalte 0) wird bercksichtigt. */
		        //((CRecord)m_objRecordList.get(row)).setAmount(newAmount.intValue());

		        /* gui anweisen, die Daten zu aktualisieren */
		        //fireTableCellUpdated(row, column);
		}

	        /**
	         * Fgt dem Datenspeicher einen neuen Datensatz hinzu
	         */
		public void addRow (CStockAgentWrapper objNewWrapper) {

	        	/* erhaltenen CStockAgentWrapper in Wrapperliste einfgen */
		        m_vecAgentList.addElement (objNewWrapper);
		        m_vecAgentList.incAgentCount();

		        /* gui anweisen, die neuen Daten darzustellen */
		        fireTableRowsInserted(getRowCount()-1, getRowCount()-1);

		}

	        /**
	         * Updated die Ausgabe bei Vernderung des bergebenen Datensatzes
	         */
		public void updateRow (CStockAgentWrapper objNewWrapper) {

	        	/* erhaltenen CStockAgentWrapper in Wrapperliste einfgen */
		        for (int i = 0; i < m_vecAgentList.size(); i++) {

				/* bei matchenden Objekten, die i-te Zeile updaten */
			        if (m_vecAgentList.getStockAgentWrapper(i) == objNewWrapper) {

					/* gui anweisen, die Daten zu aktualisieren */
					fireTableRowsUpdated(i, i);

				}

		        }

		        /* Summenzeile sollte dann ebenfalls geupdatet werden */
			fireTableRowsUpdated(m_vecAgentList.size(), m_vecAgentList.size());

		}

	        /**
	         * Lscht einen Datensatz aus dem Datenspeicher und updatet die Ansicht
	         */
		public void deleteRow (CStockAgentWrapper objNewWrapper) {

	        	/* erhaltenen CStockAgentWrapper aus Wrapperliste lschen, es
		        leben Referenzen und Pointer! */
		        int iRow = m_vecAgentList.indexOf(objNewWrapper);
		        m_vecAgentList.removeElement (objNewWrapper);

		        /* gui anweisen, die gelschten Daten zu entfernen */
		        fireTableRowsDeleted(iRow, iRow);

		}

		/**
		 * Lscht alle Datenstze aus dem Datenspeicher
		 */
		//public void clearRecords () {

		        /* Anzahl der Eintrge holen zur Zwischenspeicherung */
		//        int iRows = getRowCount();

		        /* Nur lschen, wenn auch welche da sind, sonst Exception ;-) */
		//        if (iRows > 0) {

		                /* Daten lschen... */
		//		m_objRecordList.clear ();

	        	        /* ...und Tabelle aktualisieren */
		//                fireTableRowsDeleted(0, iRows-1);
		//        }
	        //}

	}

}