package ap.stockua;

import java.util.*;
import java.io.File;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import ap.data.*;

/**
 * Title:        Ametas StockService
 * Description:  Das Hauptfenster des Benutzeradapters (stellt die Bilanztabelle und Navigation dar)
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CMainFrame extends JFrame implements java.io.Serializable {

	/** In diesen Membervariablen werden die GUI-Elemente gehalten. */
        JPanel contentPane;
        JMenuBar jMenuBar1 = new JMenuBar();
        JMenu jMenuFile = new JMenu();
        JMenuItem jMenuFileExit = new JMenuItem();
        JMenu jMenuHelp = new JMenu();
        JMenuItem jMenuHelpAbout = new JMenuItem();
        JToolBar jToolBar = new JToolBar();
        JButton buttonNew = new JButton();
        JButton buttonEdit = new JButton();
        JLabel labelSpacingToolbar1 = new JLabel();
        JButton buttonKill = new JButton();
        JButton buttonAbout = new JButton();
        ImageIcon image1;
        ImageIcon image2;
        ImageIcon image3;
        JLabel statusBar = new JLabel();
        BorderLayout borderLayout1 = new BorderLayout();
	CBalanceTable m_objBalanceTable;
	JScrollPane m_objBalanceTableScrollPane;
        JPopupMenu popupMenuBalanceTable = new JPopupMenu();
        JMenuItem popupMenuItemEdit = new JMenuItem();
        JMenuItem popupMenuItemEnd = new JMenuItem();
        JButton debugButton = new JButton();
        JLabel labelSpacingToolbar2 = new JLabel();
	JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	JList m_objMessageList = null;
	JScrollPane m_objMessageListScrollPane = null;
	DefaultListModel m_MessagesListModel = new DefaultListModel();

	/** Diese Membervariable stellt die Collection von AgentWrappern dar.
	Hierbei handelt es sich um eine "Referenz" auf die entsprechende Variable in StockUA */
	CStockAgentWrapperList m_vecAgentList = null;

	/** Die verwendeten Bilder des Benutzeradapters werden in dieser Hashliste gehalten. */
	Hashtable m_hashImages = null;

	/** unsere Referenz auf den bergeordneten Benutzeradapter */
	StockUA m_objUA = null;

	/** Ein Standardmig eingetragener Benutzername fr "Neuen Agenten anlegen" */
	String m_sDefaultUserName = "user";

	/** Zwei Membervariablen, die den gesamten investierten und herausgeholten Geldbestand speichern */
	Double m_dAmountInvested = null;
	Double m_dAmountReturned = null;



        /**
	 * Construct the frame
	 */
        public CMainFrame(StockUA objUA) {

		/* Den parameterlosen Elternklassenkonstruktor aufrufen */
		super ();

		/* Die beiden Geld-Summen-Membervariablen instanzieren */
		m_dAmountInvested = new Double(0.0);
	        m_dAmountReturned = new Double(0.0);

		/* Referenz auf Benutzeradapter in Membervariable eintragen */
		m_objUA = objUA;
		if (objUA != null) {
			m_vecAgentList = objUA.getStockAgentWrapperList();
		}

                enableEvents(AWTEvent.WINDOW_EVENT_MASK);
                try {
                        jbInit();
                }
                catch(Exception e) {
                        e.printStackTrace();
                }
		setSize(new Double(getSize().getWidth()*2).intValue(),new Double(getSize().getHeight()).intValue());

        }

        /**
	 * Component initialization
	 */
        private void jbInit() throws Exception  {

		/* verwendete Bilder in zentralen Hash einladen */
		loadImages();

		/* Die Bilder fr die Toolbarbuttons setzen */
		try {
	                image1 = (ImageIcon)(m_hashImages.get("new"));
                        image2 = (ImageIcon)(m_hashImages.get("edit"));
                        image3 = (ImageIcon)(m_hashImages.get("about"));
	                if (m_hashImages.get("stockua") != null) {
				setIconImage((Image)(m_hashImages.get("stockua")));
		        }
		} catch (Exception e) {
		        System.out.println ("Problems loading toolbar and window images.");
		}

		/* Die Oberflchenelemente konfigurieren und verknpfen */
                contentPane = (JPanel) this.getContentPane();
                contentPane.setLayout(borderLayout1);
                this.setSize(new Dimension(400, 300));
                this.setTitle("StockUA");
                statusBar.setText(" ");
                jMenuFile.setText("Datei");
                jMenuFileExit.setText("Beenden");
                jMenuFileExit.addActionListener(new ActionListener()  {
                        public void actionPerformed(ActionEvent e) {
                                jMenuFileExit_actionPerformed(e);
                        }
                });
                jMenuHelp.setText("Hilfe");
                jMenuHelpAbout.setIcon((ImageIcon)(m_hashImages.get("about")));
                jMenuHelpAbout.setText("Info");
                jMenuHelpAbout.addActionListener(new ActionListener()  {
                        public void actionPerformed(ActionEvent e) {
                                jMenuHelpAbout_actionPerformed(e);
                        }
                });
                buttonNew.setIcon(image1);
                buttonNew.setToolTipText("Neuen Agent konfigurieren und aussenden");
                buttonNew.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                buttonNew_actionPerformed(e);
                        }
                });
                buttonEdit.setIcon(image2);
                buttonEdit.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                buttonEdit_actionPerformed(e);
                        }
                });
                buttonEdit.setToolTipText("Agent konfigurieren");
                buttonAbout.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                buttonAbout_actionPerformed(e);
                        }
                });
                buttonAbout.setToolTipText("Info");
                buttonAbout.setIcon(image3);
                popupMenuItemEdit.setActionCommand("edit_agent");
                popupMenuItemEdit.setIcon((ImageIcon)(m_hashImages.get("edit")));
                popupMenuItemEdit.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                buttonEdit_actionPerformed(e);
                        }
                });
                popupMenuItemEnd.setIcon((ImageIcon)(m_hashImages.get("stopagent")));
                popupMenuItemEnd.setText("Agent beenden");
                popupMenuItemEnd.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                buttonKill_actionPerformed(e);
                        }
                });
                debugButton.setText("debug1");
                debugButton.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                debugButton_actionPerformed(e);
                        }
                });
                labelSpacingToolbar2.setText("   ");
                labelSpacingToolbar1.setText("   ");
                buttonKill.setToolTipText("Agent beenden");
                buttonKill.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                buttonKill_actionPerformed(e);
                        }
                });
                buttonKill.setIcon((ImageIcon)(m_hashImages.get("stopagent")));
                splitPane.setDividerSize(5);
                jToolBar.add(buttonNew);
                jToolBar.add(buttonEdit);
                jToolBar.add(buttonKill, null);
                jToolBar.add(labelSpacingToolbar1, null);
                jMenuFile.add(jMenuFileExit);
                jMenuHelp.add(jMenuHelpAbout);
                jMenuBar1.add(jMenuFile);
                jMenuBar1.add(jMenuHelp);
                this.setJMenuBar(jMenuBar1);
                contentPane.add(jToolBar, BorderLayout.NORTH);
                jToolBar.add(buttonAbout, null);
                jToolBar.add(labelSpacingToolbar2, null);
                jToolBar.add(debugButton, null);
                contentPane.add(statusBar, BorderLayout.SOUTH);

		/* Das Kontextmen der Bilanztabelle */
                popupMenuItemEdit.setText("Agent konfigurieren...");
                popupMenuBalanceTable.add(popupMenuItemEdit);
                popupMenuBalanceTable.addSeparator();
                popupMenuBalanceTable.add(popupMenuItemEnd);

		/* die Bilanz-Tabelle anlegen, in ein ScrollPane stecken und initialisieren */
		m_objBalanceTable = new CBalanceTable();
		m_objBalanceTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		m_objBalanceTableScrollPane = new JScrollPane (m_objBalanceTable);
		m_objBalanceTable.setImages (m_hashImages);
		// obsolet, 2001-02-13: contentPane.add(m_objBalanceTableScrollPane, BorderLayout.CENTER);

		/* Einen Maushandler fr das Kontextmen in der Tabelle installieren */
                m_objBalanceTable.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                m_objBalanceTableScrollPane_mouseClicked(e);
                        }
                });

		/* Eine Liste fr Nachrichten einziehen */
		m_MessagesListModel = new DefaultListModel();
		m_MessagesListModel.addElement("StockUA gestartet.");
		m_objMessageList = new JList(m_MessagesListModel);
		m_objMessageListScrollPane = new JScrollPane(m_objMessageList);

	        /* Add the scroll panes (table, list) to a split pane. */
	        splitPane.setTopComponent(m_objBalanceTableScrollPane);
                splitPane.setBottomComponent(m_objMessageListScrollPane);

		m_objBalanceTableScrollPane.setMinimumSize(new Dimension(100, 120));
		m_objBalanceTableScrollPane.setPreferredSize(new Dimension(100, 120));
		m_objMessageListScrollPane.setMinimumSize(new Dimension(100, 50));
                splitPane.setLastDividerLocation(120);
		splitPane.setDividerLocation(120); // XXX: ignored in some releases of Swing. bug 4101306
	        splitPane.setPreferredSize(new Dimension(500, 300));

	        // Add the split pane to the contentlayer.
		contentPane.add(splitPane, BorderLayout.CENTER);

		/* Hier wird das Handling von Selektionen in der Tabelle konfiguriert */
		/* ListSelectionModel rowSM = m_objBalanceTable.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
		        public void valueChanged(ListSelectionEvent e) {

				if (e.getValueIsAdjusting()) return;

				ListSelectionModel lsm = (ListSelectionModel)e.getSource();
				if (lsm.isSelectionEmpty()) {
				        System.out.println("No rows are selected.");
				} else {
				        int selectedRow = lsm.getMinSelectionIndex();
				        System.out.println("Row " + selectedRow + " is now selected.");
				        System.out.println("cell="+m_objBalanceTable.getSelectedColumn());

				}
		        }
		}); */

		/* Im Applikationsmodus erzeugen wir ein paar Beispieldaten um die GUI zu testen */
		if (m_objUA == null) {

			/* testweise einen AgentWrapper (Klasse CStockAgentWrapper) anlegen und
	        	mit Daten fllen */
			CStockAgentWrapper objAgentWrapper1 = new CStockAgentWrapper (m_objUA);
	        	CDepot objDepot = objAgentWrapper1.getDepot ();

			objDepot.addElement (new CDepotEntry("ABC", 22.5, 26.2, 20));
			objDepot.addElement (new CDepotEntry("LOL", 12.5, 16.2, 12));
			objDepot.addElement (new CDepotEntry("DEA", 22.1, 19.2, 6));
		        objDepot.updateDepotInformation();

			/* neue Liste von Agentwrappern anlegen und soeben angelegten Agentwrapper hinzufgen */
			m_vecAgentList = new CStockAgentWrapperList ();
			m_vecAgentList.addElement (objAgentWrapper1);
		        m_vecAgentList.incAgentCount();

			CStockAgentWrapper objAgentWrapper2 = new CStockAgentWrapper (m_objUA);
			objDepot = objAgentWrapper2.getDepot ();
		        objDepot.updateDepotInformation();
			objAgentWrapper2.setAgentName ("XOR++");
			m_vecAgentList.addElement (objAgentWrapper2);
		        m_vecAgentList.incAgentCount();

		        objAgentWrapper1.setAgentName ("Steffi");
			m_vecAgentList.updateSum ();

		}

		/* Hier wird die zentrale Collection von Agentwrappern an die Bilanztabelle
		bergeben, um die Prsentation der Daten zu realisieren. */
		m_objBalanceTable.setAgentWrapperList (m_vecAgentList);

		/* Anzeige der Statuszeile updaten (mit eigener Methode) */
		updateStatusBar();

        }

	/**
	 * Methode ldt ein Imageobjekt ein und trgt es in den Hash ein, bei Fehler
	 * Eintrag in den Errorhash.
	 */
	private void loadImage (String sIdentifier, String sFileName, String sImageFileDir, Hashtable hashError) {

		try {
		        /* Betriebsmodus unterscheiden (Benutzeradapter oder Applikation */
		        if (m_objUA == null) {

				/* Das Bild wird als eine Resource der Applikation interpretiert */
				m_hashImages.put (sIdentifier, new ImageIcon(ap.stockua.CMainFrame.class.getResource(sImageFileDir + sFileName)));

		        } else {

			        /* Versuchen das Bild einzuladen und in Hash speichern */
	        	        m_hashImages.put (sIdentifier, new ImageIcon(sImageFileDir + sFileName));

		        }

		} catch (Exception e) {

		        /* Im Fehlerfall in das errorHash schreiben */
		        hashError.put (sIdentifier, e.getMessage()!=null?e.getMessage():"not loaded");
		}
	}

	/**
	 * In dieser Methode werden abhngig vom Operationsmodus (UA, Applikation) die
	 * vielfltig verwendeten Bilder eingeladen.
	 */
	private void loadImages () {

		/* Die verwendeten Bilder des Benutzeradapters werden in dieser Hashliste
		gespeichert und im Folgenden eingeladen. */
		m_hashImages = new Hashtable();

		/* Nimmt Fehler aus mglichen Exceptions auf, wenn Bilder nicht geladen werden knnen */
		Hashtable hashError = new Hashtable();

		/* Den Pfad zu den Bildern konstruieren (entweder gar keiner weil im Applikationsverzeichnis
		oder der $Ametas/files/StockUA-Pfad) */
		String sImageFileDir = "";
		if (m_objUA == null) {

		        /* Applikationsbetriebsmodus, Bilder liegen im Klassenverzeichnis */
		        sImageFileDir = "";

			/* Und das Applikations- und Fenstericon einladen */
			try { m_hashImages.put ("stockua", Toolkit.getDefaultToolkit().createImage(ap.stockua.CMainFrame.class.getResource(sImageFileDir + "Information16.gif"))); } catch (Exception e) { hashError.put ("stockua", e.getMessage()!=null?e.getMessage():"not loaded"); }

		} else {

			/* Systempfad fr ametas-files-Verzeichnis aufbauen */
	        	String sFileSep = System.getProperty("file.separator");
		        sImageFileDir = System.getProperty("ametas.Home") + sFileSep + "files"
		                + sFileSep + "StockUA" + sFileSep;

			/* Und das Applikations- und Fenstericon einladen */
                	if (new File(sImageFileDir + "Information16.gif").exists()) {
	                        try { m_hashImages.put ("stockua", Toolkit.getDefaultToolkit().createImage(sImageFileDir + "Information16.gif")); } catch (Exception e) { hashError.put ("stockua", e.getMessage()!=null?e.getMessage():"not loaded"); }
			}

		}

		/* Bilder fr Symbolleiste und Framefenster einladen */
		loadImage ("new", "New16.gif", sImageFileDir, hashError);
		loadImage ("edit", "Edit16.gif", sImageFileDir, hashError);
		loadImage ("stopagent", "StopAgent16.gif", sImageFileDir, hashError);
		loadImage ("about", "About16.gif", sImageFileDir, hashError);

		/* Bild fr die Aboutbox */
		loadImage ("bean1", "T2.gif", sImageFileDir, hashError);

		/* Bilder fr Tabellenanzeige einladen */
		loadImage ("arrow_down", "arrow_down.gif", sImageFileDir, hashError);
		loadImage ("arrow_up", "arrow_up.gif", sImageFileDir, hashError);
		loadImage ("arrow_right", "arrow_right.gif", sImageFileDir, hashError);

		/* Bilder fr den Agenteneinstellungen-Dialog  */
		loadImage ("zoom", "Zoom16.gif", sImageFileDir, hashError);
		loadImage ("tipoftheday", "TipOfTheDay16.gif", sImageFileDir, hashError);
		loadImage ("changeagent", "Export16.gif", sImageFileDir, hashError);
		loadImage ("killagent", "Stop16.gif", sImageFileDir, hashError);

	}



	/* Im Folgenden jede Menge Eventhandler und zugehrige Hilfsmethoden */

        /**
	 * File | Exit action performed
	 */
        public void jMenuFileExit_actionPerformed(ActionEvent e) {

		/* wenn aus Benutzeradapter gestartet, dann aufwecken, sonst
		beenden (bei Applikationsstart) */
		if (m_objUA != null) {
			m_objUA.getDriver().wakeup();
		} else {
			System.exit(0);
		}

	}

        /**
	 * Menue Help | About action performed
	 */
        public void jMenuHelpAbout_actionPerformed(ActionEvent e) {

		/* Die Infobox aufrufen als modaler Dialog */
                CAboutBox dlg = new CAboutBox(this, m_hashImages);
                Dimension dlgSize = dlg.getPreferredSize();
                Dimension frmSize = getSize();
                Point loc = getLocation();
                dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
                dlg.setModal(true);
                dlg.show();

        }

        /**
	 * Overridden so we can exit when window is closed
	 */
        protected void processWindowEvent(WindowEvent e) {
                super.processWindowEvent(e);
                if (e.getID() == WindowEvent.WINDOW_CLOSING) {
                        jMenuFileExit_actionPerformed(null);
                }
        }

	/**
	 * Info Button action performed
	 */
	void buttonAbout_actionPerformed(ActionEvent e) {

		jMenuHelpAbout_actionPerformed (e);

        }

	/**
	 * Eventhandler fr Anklicken des Toolbar-Buttons "Neu". Hier wird der
	 * Dialog zum Konfigurieren und Aussenden eines neuen StockAgent gestartet.
	 */
        void buttonNew_actionPerformed(ActionEvent e) {

		/* ein CStockAgentWrapper-Objekt und eine Hashtable erstellen, in die der Dialog seine
		Daten eintragen kann */
		CStockAgentWrapper objThisWrapper = new CStockAgentWrapper (m_objUA);
		objThisWrapper.setUserName (m_sDefaultUserName);
		Hashtable hashMessages = new Hashtable();

		/* Neuen CAgentEditBox-Dialog erzeugen und konfigurieren, so dass er in der
		Mitte ber dem Hauptfensters erscheint */
                CAgentEditBox dlgAgentEdit = new CAgentEditBox(this, "new", objThisWrapper, hashMessages, m_hashImages);
                Dimension dlgSize = dlgAgentEdit.getPreferredSize();
		// dlgAgentEdit.setSize(new Double(dlgSize.getWidth()*1.3).intValue(),new Double(dlgSize.getHeight()).intValue());
                Dimension frmSize = getSize();
                Point loc = getLocation();
                dlgAgentEdit.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
                dlgAgentEdit.setModal(true);

		/* ...und anzeigen, bei Modal wird hier die Kontrolle bergeben. */
                dlgAgentEdit.show();

		/* wenn der Benutzer einen neuen Agenten angelegt hat, wird der hier
		in die Bilanztabelle eingefgt und objThisWrapper angewiesen, einen Agenten
		auszusenden. */
		if (hashMessages.get("action") != null) {
		        if (((String)hashMessages.get("action")).equals("new")) {
				if (!objThisWrapper.getAgentName().equals("")) {

				        try {
						/* Initialen Kontostand in entsprechender Variable speichern */
						objThisWrapper.getDepot().setAccountBalanceInitial(objThisWrapper.getDepot().getAccountBalance());

					        /* Agenten zum Handel aussetzen */
					        objThisWrapper.spawnAgent ();
				                messageOutput ("Neuen Agenten " + objThisWrapper.getAgentName() + " ausgesetzt.");

						/* zu Overall-Membervariable kumulieren */
						m_dAmountInvested = addDouble (m_dAmountInvested, objThisWrapper.getDepot().getAccountBalanceInitial());

					        /* Wrapperobjekt in Tabelle bergeben zur Darstellung */
	                        		m_objBalanceTable.addRow (objThisWrapper);
		        			m_vecAgentList.updateSum ();

						/* Auch die Anzeige der Statuszeile updaten */
						updateStatusBar();

						/* Den gewhlten Benutzernamen in Membervariable speichern (ease-of-use-feature) */
						m_sDefaultUserName = objThisWrapper.getUserName ();

				        } catch (Exception ex) {

						/* Fehler beim Aussenden aufgetreten */
						messageOutput ("exception: "+ex+" ("+ex.getMessage()+")");

					}
		                }
		        }
		}
        }

	/**
	 * Eventhandler wird aufgerufen, wenn Benutzer auf StopAgent-Button/-Menueintrag
	 * klickt. Die Funktion beendet sofort, wenn keine Tabellenzeile oder die
	 * Summenzeile selektiert ist.
	 */
        void buttonKill_actionPerformed(ActionEvent e) {

		/* Wenn keine Zeile oder die Summenzeile selektiert, dann kein Beenden :) */
		if ((m_objBalanceTable.getSelectedRow() > -1) && (m_objBalanceTable.getSelectedRow() < m_objBalanceTable.getRowCount()-1)) {

			/* das CStockAgentWrapper-Objekt in lokaler Variable speichern */
	        	CStockAgentWrapper objThisWrapper = m_vecAgentList.getStockAgentWrapper(m_objBalanceTable.getSelectedRow());

			try {

		                /* und Kill-Funktion aufrufen */
			        objThisWrapper.killAgent();

				/* --> der letzte Kontostand mu bercksichtigt werden fr die Overall-
				Membervariable, sinnvollerweise mit einer Nachricht beim Beenden des StockAgent */
				m_dAmountReturned = addDouble (m_dAmountReturned, objThisWrapper.getDepot().getAccountBalance());

				/* anzeige in Statusbar updaten */
				updateStatusBar ();

		        } catch (Exception ex) {

				// Fehler beim Aussenden aufgetreten !!!
				messageOutput ("exception: "+ex+" ("+ex.getMessage()+")");

		        }

        	        /* Agenteneintrag aus Datenverwaltung lschen */
	                m_vecAgentList.removeElement (objThisWrapper);

	                /* Lschvorgang in Tabelle durch Tabellenupdate sichtbar machen */
		        m_objBalanceTable.deleteRow (objThisWrapper);
		}

        }

	/**
	 * Eventhandler fr Anklicken des Toolbar-Buttons "Bearbeiten". Hier wird der
	 * Dialog zum Konfigurieren eines in der Liste selektierten StockAgent gestartet.
	 */
        void buttonEdit_actionPerformed(ActionEvent e) {

		/* Wenn keine Zeile oder die Summenzeile selektiert, dann kein Bearbeiten! */
		if ((m_objBalanceTable.getSelectedRow() > -1) && (m_objBalanceTable.getSelectedRow() < m_objBalanceTable.getRowCount()-1)) {

			/* ein CStockAgentWrapper-Objekt und eine Hashtable erstellen, aus dem und in die der Dialog
		        seine Daten eintragen kann */
	        	CStockAgentWrapper objThisWrapper = m_vecAgentList.getStockAgentWrapper(m_objBalanceTable.getSelectedRow());
			Hashtable hashMessages = new Hashtable();

			/* Neuen CAgentEditBox-Dialog erzeugen und konfigurieren, so dass er in der
	        	Mitte ber dem Hauptfensters erscheint */
                        CAgentEditBox dlgAgentEdit = new CAgentEditBox(this, "edit", objThisWrapper, hashMessages, m_hashImages);
			Dimension dlgSize = dlgAgentEdit.getPreferredSize();
			Dimension frmSize = getSize();
			Point loc = getLocation();
			dlgAgentEdit.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
			dlgAgentEdit.setModal(true);

			/* ...und anzeigen, bei Modal wird hier die Kontrolle bergeben. */
			dlgAgentEdit.show();

			/* wenn der Benutzer das Bearbeiten beendet hat, wird ein Update der
			Bilanztabelle ausgefhrt und der Agent durch einen PrefAgent mit den
		        neuen Einstellungen beliefert. */
			if (hashMessages.get("action") != null) {
	        	        if (((String)hashMessages.get("action")).equals("change")) {
		        		if (!objThisWrapper.getAgentName().equals("")) {

						/* Extrahieren der zu verndernden Werte aus dem Parameterhash */
						String sPlaceName = (String)hashMessages.get ("placename");
						int iStrategy = ((Integer)hashMessages.get ("strategyid")).intValue();
						double[] adStrategyParams = (double[])hashMessages.get ("strategyparams");

						/* Einstellungen des StockAgent updaten durch Aussetzen eines PrefAgent */
						try {
						        objThisWrapper.updateAgent (sPlaceName, iStrategy, adStrategyParams);

						        /* Ansicht in Tabelle updaten (mglicherweise hat sich ja die Stelle verndert) */
						        m_vecAgentList.updateSum ();
						        m_objBalanceTable.updateRow (objThisWrapper);

					        } catch (Exception ex) {

	        					// Fehler beim Aussenden aufgetreten !!!
		        				messageOutput ("exception: "+ex+" ("+ex.getMessage()+")");

			        		}
			                }
	        	        }
		        }
		}
        }

	/**
	 * Eventhandler wird beim Klicken einer Maustaste in der Bilanztabelle aufgerufen
	 */
        void m_objBalanceTableScrollPane_mouseClicked(MouseEvent e) {

		/* Feststellen, welche Buttons gedrckt wurden */
		int iButton2 = e.getModifiers() & java.awt.event.InputEvent.BUTTON2_MASK;
		int iButton3 = e.getModifiers() & java.awt.event.InputEvent.BUTTON3_MASK;

		/* Wurde einer der rechten Buttons gedrckt, dann Kontextmen an Mausposition ffnen */
		if ((iButton2 == java.awt.event.InputEvent.BUTTON2_MASK)
		        || (iButton3 == java.awt.event.InputEvent.BUTTON3_MASK)) {

		        /* Tabellenzeile bestimmen, an der geklickt wurde */
		        int iSelectedRow = e.getY() / m_objBalanceTable.getRowHeight();

			/* Zeile an Mausposition aktivieren */
		        ListSelectionModel lsmThisModel = m_objBalanceTable.getSelectionModel();
		        lsmThisModel.setLeadSelectionIndex (iSelectedRow);
		        lsmThisModel.setSelectionInterval (iSelectedRow, iSelectedRow);

			/* Kontextmen an Mausposition ffnen */
		        if (iSelectedRow < m_objBalanceTable.getRowCount()-1) {
				/* Datenzeile: */
			        popupMenuItemEdit.setEnabled(true);
		                popupMenuItemEnd.setEnabled(true);
		                popupMenuBalanceTable.show (m_objBalanceTable, e.getX(), e.getY());
		        } else {
				/* Summenzeile: */
			        popupMenuItemEdit.setEnabled(false);
		                popupMenuItemEnd.setEnabled(false);
		                popupMenuBalanceTable.show (m_objBalanceTable, e.getX(), e.getY());
		        }

		}

        }

	/**
	 * Eventhandler fr den Debugbutton in der Symbolleiste
	 */
        void debugButton_actionPerformed(ActionEvent e) {

		/* Nur im UA-Modus Aktionen ausfhren */
		if (m_objUA != null) {

			/* Testnachrichten, bis der Agent funktioniert und "Real-World-Daten" liefert. */
			Object[] aMsgData = { "GO_NOTIFY", "Eye_Of_The_Beholder", "Corea,Chick", "Jazz", new Double(37.23),
						"Secret_Story", "Metheny,Pat", "Jazz", new Double(34.99) };
			m_objUA.getDriver().depositMessage (m_objUA.getID(), aMsgData);

			Object[] aMsgData2 = { "GO_NOTIFY", "Steffi", "stelle-1", aMsgData };
			m_objUA.getDriver().depositMessage (m_objUA.getID(),aMsgData2);

			Object[] aMsgData3 = { "YALLAAAA", "TEST" };
			m_objUA.getDriver().depositMessage (m_objUA.getID(),aMsgData3);

		        CStockAgentMsgs samMsgs = new CStockAgentMsgs();
		        samMsgs.addElement(new CStockAgentMsgDepot(m_vecAgentList.getStockAgentWrapper(0).getDepot()));

			Object[] aMsgData4 = { "NOTIFICATION", m_vecAgentList.getStockAgentWrapper(0).getPuidAgent(), samMsgs.getData() };
			m_objUA.getDriver().depositMessage (m_objUA.getID(),aMsgData4);

		}

        }

        /**
         * Ruft die entsprechende Methode von m_objBalanceTable auf, um das angegebene Element zu updaten
         */
	public void updateRow (CStockAgentWrapper objAgentWrapper) {

		/* Der Tabelle den Updatebefehl mitteilen */
		m_objBalanceTable.updateRow (objAgentWrapper);

	}

	/**
	 * die Methode macht genau das :)
	 */
	private Double addDouble (Double a, Double b) {
		return new Double(a.doubleValue() + b.doubleValue());
	}

	/**
	 * Methode updated die Statusbar des Hauptfenster
	 */
	public void updateStatusBar () {

		/* Variable fr Statuszeilen Ausgabe anlegen */
		String sStatusBarText = "";

		sStatusBarText += " aktuell/insgesamt handelnde Agenten: " + m_vecAgentList.size() + " / " + m_vecAgentList.getAgentCount();
		sStatusBarText += "  - ";
		sStatusBarText += " eingezahlte/ausgezahlte Agentendollar: " + m_dAmountInvested + " / " + m_dAmountReturned;

		/* und Variablenwert in Control schreiben */
		statusBar.setText(sStatusBarText);

	}

	/**
	 * Methode gibt den im Parameter bergebenen String in der Meldungsliste
	 * und - wenn als Benutzeradapter gestartet - auch in das Stellenlog aus
	 *
	 * @param sMsgText der der Messageliste hinzuzufgende Text
	 */
	public void messageOutput (String sMsgText) {
		m_MessagesListModel.addElement (sMsgText);
		if (m_objUA != null) {
		        m_objUA.getDriver().output(sMsgText);
		}
	}

}