package ap.stockua;

import AMETAS.data.*;
import AMETAS.data.type.*;

import ap.data.*;

/**
 * Title:        Ametas StockService
 * Description:  Wrapperklasse fr einen StockAgent
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CStockAgentWrapper implements java.io.Serializable {

	/** Die folgenden Membervariablen nehmen die Nutzdaten der Instanz auf
	und sollten selbsterklrend sein. Weitere Infos auf Aufgabenblatt 7. */
	String m_sUserName = "";
	String m_sAgentName = "";
	String m_sPlaceName = "";

	/** Das Flag gibt an, ob der mit der Wrapperinstanz beschriebene Agent unterwegs ist, also lebt */
	boolean m_bAgentOnTour = false;

	/** In diesem Objekt werden die Depotinformationen des gewrappten Agenten
	gespeichert. Mglicherweise ist hier auch eine Collection von CDepots interessant,
	um den Verlauf der Agententtigkeit darstellen zu knnen.*/
	CDepot m_objDepot = null;

	/** unsere Referenz auf den bergeordneten Benutzeradapter */
	StockUA m_objUA = null;

	/** die placeuserid des gewrappten Agenten */
	AMETASPlaceUserID m_puidAgent = null;

	/** Die Strategie-ID (Default 1) und die Strategie-Parameter */
	int m_iStrategy = 1;
	double[] m_adStrategyParams = { -3.0, 1.0, -1.0, 1.0 };



	/**
	 * Konstruktor
	 */
        public CStockAgentWrapper (StockUA objUA) {

		/* Referenz auf Benutzeradapter in Membervariable eintragen */
		m_objUA = objUA;

		/* Hier mssen auch die Arbeitsstelle und der Agentenname des gewrappten Agenten
		gesetzt werden. Fr den Namen kann ich mir Namen wie Claudia, Dieter, Klaus, etc.
		vorstellen */
		m_sAgentName = "";
		m_sPlaceName = "";

		/* Neues Depot fr diese Wrapperklasse anlegen, in der die von den StockInfoAgents
		gelieferten Informationen des gewrappten Agenten gespeichert werden. Mglicherweise
		ist hier auch eine Collection von CDepots interessant, um den Verlauf der
		Agententtigkeit darstellen zu knnen. */
		m_objDepot = new CDepot ();

        }

	/**
	 * Liefert den Wert der Membervariable m_sAgentName zurck, die den
	 * Namen des gewrappten Agenten aufnimmt.
	 *
	 * @return Der Name des gewrappten Agenten
	 */
	public String getAgentName () {
		return m_sAgentName;
	}

	/**
	 * Setzt den Wert der Membervariable m_sAgentName, die den Namen des
	 * gewrappten Agenten aufnimmt.
	 *
	 * @param sAgentName Der neue Name des Agenten
	 */
	public void setAgentName (String sAgentName) {
		m_sAgentName = sAgentName;
	}

	/**
	 * Liefert den Wert der Membervariable m_sPlaceName zurck, die den
	 * Namen der Stelle aufnimmt, auf der der gewrappte Agent luft.
	 *
	 * @return Der Name des Stelle, auf der der gewrappte Agent luft
	 */
	public String getPlaceName () {
		return m_sPlaceName;
	}

	/**
	 * Setzt den Wert der Membervariable m_sPlaceName, die den Namen
	 * der Agentenstelle aufnimmt.
	 *
	 * @param sPlaceName Der neue Name der Agentenstelle
	 */
	public void setPlaceName (String sPlaceName) {
		m_sPlaceName = sPlaceName;
	}

	/**
	 * Liefert den Wert der Membervariable m_sUserName zurck, die den
	 * Benutzernamen des gewrappten Agent aufnimmt.
	 *
	 * @return Der Benutzername des gewrappten Agenten (fr das Konto des jeweiligen Stellen-Service)
	 */
	public String getUserName () {
		return m_sUserName;
	}

	/**
	 * Setzt den Wert der Membervariable m_sUserName, die den Benutzernamen
	 * der Agentenstelle aufnimmt.
	 *
	 * @param sUserName Der neue Benutzername des gewrappten Agenten
	 */
	public void setUserName (String sUserName) {
		m_sUserName = sUserName;
	}

	/**
	 * Liefert den Wert der Membervariable m_puidAgent zurck, die die
	 * PlaceUserId des ausgesetzten Agenten aufnimmt.
	 *
	 * @return Die PlaceUserId des gewrappten und ausgesetzten Agenten
	 */
	public AMETASPlaceUserID getPuidAgent () {
		return m_puidAgent;
	}

	/**
	 * Liefert das Depot des Agentwrappers zurck.
	 *
	 * @return Das zum gewrappten Agenten gehrige Depot
	 */
	public CDepot getDepot () {
		return m_objDepot;
	}

	/**
	 * Methode speichert die Strategie-ID des Agenten in der entsprechenden Membervariable.
	 *
	 * @param iStrategy Die neue Strategie-ID des Agenten (Default ist 1)
	 */
	public void setStratgey (int iStrategy) {
		m_iStrategy = iStrategy;
	}

	/**
	 * Methode liefert die Strategie-ID des Agenten aus der entsprechenden Membervariable.
	 *
	 * @return Die aktuelle Strategie-ID des Agenten
	 */
	public int getStratgey () {
		return m_iStrategy;
	}

	/**
	 * Methode speichert die Strategie-Parameter des Agenten in der entsprechenden Membervariable.
	 *
	 * @param adStratgeyParams Die neuen Strategie-Parameter des Agenten als double-array
	 */
	public void setStrategyParams (double[] adStrategyParams) {
		m_adStrategyParams = adStrategyParams;
	}

	/**
	 * Methode liefert die Strategie-Parameter des Agenten aus der entsprechenden Membervariable.
	 *
	 * @return Die aktuellen Strategie-Parameter des Agenten als double-Array
	 */
	public double[] getStrategyParams () {
		return m_adStrategyParams;
	}

	/**
	 * Methode updatet die Daten des Agentwrappers (insbesondere die Depotdaten)
	 * aus dem bergebenen Parameter, der aus einer Nachricht eines StockInfoAgent kommt.
	 */
	public void setStockInfoValues (CStockAgentMsgs objStockInfoValues) {

		// ...09.02.2001: auswerten sobald das Format von Fabian klar ist
		// DEBUG: System.out.println ("Datenlieferung fr StockAgent " + m_sAgentName + " eingetroffen.");

		/* Die Nachricht von StockInfoAgent durchgehen und nach Nachrichtentyp CStockAgentMsgDepot suchen */
		CStockAgentMsg curMsg = null;
		boolean bDepotFound = false;
		for (int i = 0; i < objStockInfoValues.size(); i++) {

		        /* Wir interessieren uns hier nur fr die CStockAgentMsgDepot - Objekte */
		        curMsg = objStockInfoValues.getMsg(i);
			if (curMsg instanceof CStockAgentMsgDepot) {

				/* Daten aus curMsg herauslsen und in Membervariablen eintragen */
				CDepot objDepot = ((CStockAgentMsgDepot)curMsg).getDepot();

			/* TRANSAKTION START: */
				/* Lschen der bisher enthaltenen DepotEntrys */
				m_objDepot.clear();

				/* Aus StockInfoAgent->curMsg->objDepot gelieferte DepotEntrys ins Memberdepot einfgen */
				m_objDepot.addFromDepot(objDepot);

				/* den Kontostand bernehmen */
				m_objDepot.setAccountBalance (objDepot.getAccountBalance());

				/* und das Depot updaten, um die auto-berechneten Werte zu ermitteln */
				m_objDepot.updateDepotInformation();
			/* :TRANSAKTION ENDE */

				/* -> Jetzt mte dem Hauptfenster bzw. der Tabelle noch mitgeteilt
				werden, dass eine Vernderung stattgefunden hat -> das passiert in der
				aufrufenden Funktion und ist deutlich sauberer von OOP-Seite! */

				/* Flag setzen */
				bDepotFound = true;
		        }

		}
	}

	/**
	 * Methode erzeugt einen Agenten des angegebenen Typs mit initialiser Startnachricht
	 * aus den Parametern und liefert bei Erfolg die puid des neuen Agenten zurck, sonst
	 * eine Exception vom Typ CAgentWrapperException.
	 *
	 * @param asBody Die vollstndige (!) Nachrichtennutzlast, also mit Spezifikation als Objectarray
	 * @return Die puid des neu erzeugten Agenten im Erfolgsfall, sonst null.
	 */
	private AMETASPlaceUserID sendAgent (String sAgentClassName, Object[] asBody) throws CAgentWrapperException {

		/* Wir kennen die ID nicht, aber die Stelle. Dies ist die einzige Gelegenheit,
		eine Nachricht ohne Empfnger einzureichen! */
		AMETASMessage mesInit = new AMETASMessage(null, asBody);

		/* den in sAgentClassName angegebenen Agenten aussenden */
		AMETASPlaceUserID puidAgent = null;
		try {

		        /* der angegebene Agent mu natrlich auf der Stelle (im Verzeichnis agents) vorhanden sein */
		        puidAgent = m_objUA.getDriver().requestPUStartup(sAgentClassName, mesInit);

		}
		catch (CreationFailedException cfx) {
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException (sAgentClassName + " konnte nicht gestartet werden.");
		}
		catch (CreationDeniedException cdx) {
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException (sAgentClassName + "durfte nicht gestartet werden.");
		}
		catch (NullPointerException npx) {
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException ("Referenz auf StockUA steht nicht zur Verfgung.");
		}

		/* Wenn keine Exceptions auftraten, dann die puid des neuen Agenten zurckgeben */
		return puidAgent;

	}

	/**
	 * Methode setzt den mit der Wrapperinstanz beschriebenen Agenten aus
	 */
	public void spawnAgent() throws CAgentWrapperException {

		/* Wenn der Agent bereits unterwegs ist, wird er hier nicht mehr ausgesetzt
		und stattdessen eine Exception geworfen. */
		if (m_bAgentOnTour) {
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException ("StockAgent " + m_sAgentName + " wurde bereits ausgesandt.");
		}

		/* Format der Nachricht, die der Agent beim Aussetzen erhlt:
		"GO_TRADE", String sUserName, String sAgentName, CStockAgentMsgs samMsgs, double dCredits */

		/* Ein Objekt fr die Zielstelle erzeugen */
		CStockAgentMsgMove samPlace = new CStockAgentMsgMove (m_sPlaceName);

		/* Und ein Objekt fr die Strategiedaten (Werte aus den Membervariablen holen) */
		CStockAgentMsgStrategy samStrategy = new CStockAgentMsgStrategy (m_iStrategy, m_adStrategyParams); // strategyid, parameter

		/* Die Parameter fr die Startnachricht in die CStockAgentMsgs-Struktur eintragen */
		CStockAgentMsgs samMsgs = new CStockAgentMsgs();
		samMsgs.addElement(samPlace);
		samMsgs.addElement(samStrategy);

		/* Dem Agenten wird eine Nachricht beim Starten mitgegeben. Hier setzen
		wir die Nachrichtennutzlast zusammen */
		Object[] asBody = { "GO_TRADE",
		        m_sUserName,
		        m_sAgentName,
		        samMsgs.getData(),
		        m_objDepot.getAccountBalance()
		};

		try {
			/* StockAgent zum Handeln aussenden */
	        	m_puidAgent = sendAgent ("ap.stockagent.StockAgent", asBody);
		        if (m_puidAgent != null) { m_bAgentOnTour = true; }

		} catch (CAgentWrapperException ex) {

		        /* Im Fehlerfall die Exception "durchschleifen" */
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException (""+ex.getMessage());
		}

	}

	/**
	 * Methode updatet den mit der Wrapperinstanz beschriebenen Agenten, indem ein
	 * Prefagent mit den angegebenen Parametern ausgesendet wird
	 */
	public void updateAgent(String sPlaceName, int iStrategy, double[] adStrategyParams) throws CAgentWrapperException {

		/* Wenn der Agent bereits unterwegs ist, wird er hier nicht mehr ausgesetzt
		und stattdessen eine Exception geworfen. */
		if (!m_bAgentOnTour) {
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException ("StockAgent " + m_sAgentName + " nicht unterwegs. Es wird kein PrefAgent ausgesetzt.");
		}

		/* Format der Nachricht, die der Agent beim Aussetzen erhlt:
		"GO_NOTIFY", AMETASPlaceUserID puidTarget, String sTargetPlace, CStockAgentMsgs samMsgs */

		/* Ein Objekt fr die mglicherweise neue Zielstelle erzeugen */
		CStockAgentMsgMove samPlace = new CStockAgentMsgMove (sPlaceName);

		/* Und ein Objekt fr die Strategiedaten -> die Parameter mssen noch aus Membervariablen geholt werden */
		CStockAgentMsgStrategy samStrategy = new CStockAgentMsgStrategy (iStrategy, adStrategyParams); // strategyid, parameter

		/* Die Parameter fr die Startnachricht in die CStockAgentMsgs-Struktur eintragen */
		CStockAgentMsgs samMsgs = new CStockAgentMsgs();
		samMsgs.addElement(samPlace);
		samMsgs.addElement(samStrategy);

		/* Dem PrefAgent wird eine Nachricht beim Starten mitgegeben. Hier setzen
		wir die Nachrichtennutzlast zusammen */
		Object[] asBody = { "GO_NOTIFY",
		        m_puidAgent,
		        m_sPlaceName,
		        samMsgs.getData()
		};

		try {
			/* PrefAgent zum gewrappten StockAgent aussenden */
	        	// m_puidAgent = sendAgent ("ap.stockagent.PrefAgent", asBody);
	        	sendAgent ("ap.stockagent.PrefAgent", asBody);

		        /* Jetzt da ein PrefAgent auf die Reise geschickt ist, knnen wir die
		        Membervariablen des Agentwrappers mit den neuen Werten beschreiben. */
		        m_sPlaceName = sPlaceName;
			m_iStrategy = iStrategy;
			m_adStrategyParams = adStrategyParams;

		} catch (CAgentWrapperException ex) {

		        /* Im Fehlerfall die Exception "durchschleifen" */
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException (""+ex.getMessage());
		}

	}

	/**
	 * Methode beendet den mit der Wrapperinstanz beschriebenen Agenten indem ein Prefagent
	 * mit kill-Nachricht ausgesendet wird.
	 */
	public void killAgent() throws CAgentWrapperException {

		/* Wenn der Agent bereits unterwegs ist, wird er hier nicht mehr ausgesetzt
		und stattdessen eine Exception geworfen. */
		if (!m_bAgentOnTour) {
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException ("StockAgent " + m_sAgentName + " nicht unterwegs. Es wird kein PrefAgent ausgesetzt.");
		}

		/* Format der Nachricht, die der Agent beim Aussetzen erhlt:
		"GO_NOTIFY", AMETASPlaceUserID puidTarget, String sTargetPlace, CStockAgentMsgs samMsgs */

		/* Ein Objekt fr die Beendigung erzeugen */
		CStockAgentMsgExit samExit = new CStockAgentMsgExit ();

		/* Die Parameter fr die Startnachricht in die CStockAgentMsgs-Struktur eintragen */
		CStockAgentMsgs samMsgs = new CStockAgentMsgs();
		samMsgs.addElement(samExit);

		/* Dem PrefAgent wird eine Nachricht beim Starten mitgegeben. Hier setzen
		wir die Nachrichtennutzlast zusammen */
		Object[] asBody = { "GO_NOTIFY",
		        m_puidAgent,
		        m_sPlaceName,
		        samMsgs.getData()
		};

		try {
			/* PrefAgent zum gewrappten StockAgent aussenden */
	        	sendAgent ("ap.stockagent.PrefAgent", asBody);

		} catch (CAgentWrapperException ex) {

		        /* Im Fehlerfall die Exception "durchschleifen" */
		        CAgentWrapperException exAgentWrapperException = new CAgentWrapperException (""+ex.getMessage());
		}

	}

}