package ap.stockua;

import java.util.*;
import java.io.Serializable;

import AMETAS.data.*;
import AMETAS.data.type.*;

import ap.data.*;

/**
 * Title:        CDepot
 * Description:  Collection von CStockAgentWrapper-Instanzen, die die Verbindung zu den
 *               vom Benutzeradapter gemanagten Agenten darstellen.
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class CStockAgentWrapperList extends Vector implements java.io.Serializable {

	/** In diese Membervariable wird das Summendepot gehalten, das die
	Informationen aus den zu den Agenten gehrigen Depots kumuliert. */
	CSumDepot m_objSumDepot = null;

	/** Membervariable hlt die Anzahl der insgesamt gewrappten Agenten */
	int m_iAgentCount = 0;



	/**
	 * Der Konstruktor
	 */
	public CStockAgentWrapperList () {

		/* analogen Elternklassenkonstruktor aufrufen */
		super();

		/* Neues Depot fr die Summenberechnung anlegen */
		m_objSumDepot = new CSumDepot ();

	}

	/**
	 * Die Methode liefert das i-te in der Liste gespeicherte CStockAgentWrapper-Objekt zurck.
	 *
	 * @param i Das i-te CStockAgentWrapper-Objekt in der Liste
	 */
	public CStockAgentWrapper getStockAgentWrapper (int i) {

		/* Liefert das getypcastete i-te Objekt des Vectors zurck */
		return (CStockAgentWrapper)get(i);

	}

	/**
	 * Die Methode sucht das CStockAgentWrapper-Objekt in der Liste, dass sAgentName als
	 * Agentennamen hat und liefert es zurck.
	 *
	 * @param sAgentName Der Agentenname des in der Liste zu suchenden CStockAgentWrapper-Objekts
	 */
	public CStockAgentWrapper getElementByName (String sAgentName) {

		/* Den Vector durchgehen und sAgentName mit dem sAgentName-Attribut jedes
		CStockAgentWrapper-Datensatzes vergleichen */
		for (int i = 0; i < size(); i++) {
		        CStockAgentWrapper objCurEntry = getStockAgentWrapper(i);

		        if (objCurEntry.getAgentName().equals(sAgentName)) {
			        /* Liefert das getypcastete i-te Objekt des Vectors zurck */
		        	return objCurEntry;
		        }

		}

		/* Wenn kein matchendes Objekt gefunden wurde, dann null zurckliefern */
		return null;
	}

	/**
	 * Die Methode sucht das CStockAgentWrapper-Objekt in der Liste, dass die AMETASPlaceUserID
	 * puidAgent als puid hat und liefert es zurck.
	 *
	 * @param puidAgent Die Puid des in der Liste zu suchenden CStockAgentWrapper-Objekts
	 */
	public CStockAgentWrapper getElementByPuid (AMETASPlaceUserID puidAgent) {

		/* Den Vector durchgehen und puidAgent mit dem pidAgent-Attribut jedes
		CStockAgentWrapper-Datensatzes vergleichen */
		for (int i = 0; i < size(); i++) {
		        CStockAgentWrapper objCurEntry = getStockAgentWrapper(i);

		        if (objCurEntry.getPuidAgent().equals(puidAgent)) {
			        /* Liefert das getypcastete i-te Objekt des Vectors zurck */
		        	return objCurEntry;
		        }

		}

		/* Wenn kein matchendes Objekt gefunden wurde, dann null zurckliefern */
		return null;
	}

	/**
	 * Liefert das Summendepot der Agentwrapperliste zurck.
	 *
	 * @return Das Summendepot der Agentwrapperliste
	 */
	public CDepot getSumDepot () {
		return m_objSumDepot;
	}

	/**
	 * Liefert die Anzahl der in der Objektlebensdauer gewrappten Agenten zurck.
	 * Die aktuelle Zahl erhlt man durch Aufruf der Methode size().
	 *
	 * @return Die Anzahl der insgesamt gewrappten Agenten
	 */
	public int getAgentCount() {
		return m_iAgentCount;
	}

	/**
	 * Erhht den Counter fr die Anzahl der in der Objektlebensdauer gewrappten Agenten.
	 */
	public void incAgentCount() {
		m_iAgentCount++;
	}

	/**
	 * Methode weist die Wrapperliste an, das Summendepot neu zu berechnen. Das
	 * ist fr die Summenausgabe in der Bilanztabelle ntig
	 */
	public void updateSum () {

		/* Werte in Summenarray zurcksetzen */
		m_objSumDepot.setAccountBalance (new Double(0.0));
		m_objSumDepot.setTotalPriceBought (new Double(0.0));
		m_objSumDepot.setTotalPriceCurrent (new Double(0.0));
		// maybe obsolet: m_objSumDepot.setGainCurrent (new Double(0.0));
		// maybe obsolet: m_objSumDepot.setGainOverall (new Double(0.0));
		m_objSumDepot.setTotalAmount (new Integer(0));
		m_objSumDepot.setTendency (3);

		/* Alle verwalteten StockAgentWrapper-Klassen durchlaufen und Werte
		der zugehrigen Depots in m_objSumDepot aufaddieren. */
		for (int i = 0; i < size(); i++) {

			CStockAgentWrapper objCurrentWrapper = (CStockAgentWrapper)get(i);
		        CDepot objCurrentDepot = objCurrentWrapper.getDepot();

		        /* die einzelnen aufzukummulierenden Werte berechnen und in m_objSumDepot speichern */
		        m_objSumDepot.setAccountBalance( addDouble(m_objSumDepot.getAccountBalance(), objCurrentDepot.getAccountBalance()) );
		        m_objSumDepot.setTotalPriceBought( addDouble(m_objSumDepot.getTotalPriceBought(), objCurrentDepot.getTotalPriceBought()) );
		        m_objSumDepot.setTotalPriceCurrent( addDouble(m_objSumDepot.getTotalPriceCurrent(), objCurrentDepot.getTotalPriceCurrent()) );
		        // maybe obsolet: m_objSumDepot.setGainCurrent( addDouble(m_objSumDepot.getGainCurrent(), objCurrentDepot.getGainCurrent()) );
		        // maybe obsolet: m_objSumDepot.setGainOverall( addDouble(m_objSumDepot.getGainOverall(), objCurrentDepot.getGainOverall()) );
		        m_objSumDepot.setTotalAmount( addInteger(m_objSumDepot.getTotalAmount(), objCurrentDepot.getTotalAmount()) );

		        // -> das knnte man aus dem alten Werten generieren...
		        m_objSumDepot.setTendency( 3 );

		}

	}

	/**
	 * die Methode macht genau das :)
	 */
	private Double addDouble (Double a, Double b) {
		return new Double(a.doubleValue() + b.doubleValue());
	}

	/**
	 * auch diese Methode macht genau das :)
	 */
	private Integer addInteger (Integer a, Integer b) {
		return new Integer(a.intValue() + b.intValue());
	}

}



