package ap.stockua;

import AMETAS.agentdev.AMETASUserAdapter;
import AMETAS.place.*;
import AMETAS.data.*;
import AMETAS.data.type.*;
import AMETAS.event.*;

import javax.swing.UIManager;
import java.awt.*;
import java.util.*;

import ap.data.*;

/**
 * Title:        Ametas StockService
 * Description:  Die Klasse stellt einen Benutzeradapater dar, mit dem die Multiagenten-
 *               und Serviceanwendung zum simulierten Aktienhandel unter Ametas vom Benutzer gesteuert wird.
 * Copyright:    Copyright (c) 2001
 * Company:      Ametas Praktikum, Uni Frankfurt, Fachbereich Biologie und Informatik, WS2000/2001
 * @author Martin Klossek, Fabian Wleklinski
 * @version 1.0
 */

public class StockUA extends AMETASNotifiableUserAdapter implements java.io.Serializable {

	/** Diese Membervariable stellt die Collection der AgentWrapper dar. */
	CStockAgentWrapperList m_vecAgentList = null;

	/** Die Membervariable auf das Hauptfenster des Benutzeradapters. */
	private CMainFrame m_frmMainFrame;

	/** Die Membervariable m_oMessage nimmt die letzte von ametas gelieferte Nachricht auf */
	AMETASMessage m_oMessage = null;



	/**
	 * Der Konstruktor unseres Benutzeradapters zur Steuerung der StockAgents
	 */
	public StockUA() {

		/* Elternkonstruktor mit Namen des Benutzeradpaters und Gruppenname aufrufen */
		super( "StockUA", "orange" );

		/* Respository fr AgentWrapperinstanzen anlegen */
		m_vecAgentList = new CStockAgentWrapperList();

	}

	/**
	 * Die allseits beliebte invoke-Methode des UA. Hier spielt ide Musik!
	 */
        public void invoke() {

		/* Benutzeradapter auf Event-Empfang vorbereiten durch Listener-Registrierung */
		gleich: registerAsListener();

		/* Fenster des Benutzeradapters initialisieren und ffnen */
		openMainFrame ();

		/* Die Kontrolle des Benutzeradapters liegt jetzt beim Fenster und
		wir knnen uns hier schlafen legen. Die Beendigung erfolgt dementsprechend,
	        wenn der UA aufgeweckt wird, daher mssen wir eine Referenz auf diese
		UA-Instanz an das Fenster bergeben (in openMainFrame!) */
		m_Driver.idle();

        }

	/**
	 * In dieser Methode wird das Fenster des Benutzeradapters geffnet. Da einige
	 * Methodenaufrufe erfolgen (Controls packen, Gre anpassen, etc.), ist der
	 * Code in diese Methode verlagert worden, um die Methode invoke() zu entlasten.
	 */
	private void openMainFrame () {

		/* die installierten Skins holen und unseren "Lieblingsskin" setzen :-) */
		try {
		        UIManager.LookAndFeelInfo[] plafs = UIManager.getInstalledLookAndFeels();
		        String sClassName = UIManager.getSystemLookAndFeelClassName();
		        for (int i = 0; i < plafs.length; i++) {
				sClassName = plafs[i].getClassName();
				// DEBUG: Ausgeben der LnF-Klassennamen: System.out.println(sClassName);
		        }

		        // DEBUG: System-LnF setzen: UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		        UIManager.setLookAndFeel(plafs[0].getClassName());
		} catch(Exception e) {
		        e.printStackTrace();
		}

		/* Erzeuge Fenster */
	        m_frmMainFrame = new CMainFrame(this);

                /* Validate frames that have preset sizes Pack frames that have
		useful preferred size info, e.g. from their layout */
		boolean packFrame = false;
	        if (packFrame) { m_frmMainFrame.pack(); }
		else { m_frmMainFrame.validate(); }

                /* Center the window */
                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                Dimension frameSize = m_frmMainFrame.getSize();
                if (frameSize.height > screenSize.height) {
                        frameSize.height = screenSize.height;
                }
                if (frameSize.width > screenSize.width) {
                        frameSize.width = screenSize.width;
                }
                m_frmMainFrame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
                m_frmMainFrame.setVisible(true);

	}

	/**
 	 * Registriert den Benutzeradapter bei der Stelle als &quot;Listener&quot;
	 * f&uuml;r die ben&ouml;tigten Ereignisse.
	 */
	private boolean registerAsListener() {

		addEventHandler(new CUAEventHandler());

		/* Den Benutzeradapter als Ereignishandler der aktuellen
		Stelle registrieren. Wichtig ist hierbei, dass wenn keine explizit
		anderslautende Option angegeben wird, die Nachricht, die ein Ereignis
		auslst, mit diesem Event mitgeschickt wird UND aus dem Postamt
		gelscht wird */
		AMETASMessageMask mask = new AMETASMessageMask( null, new AMETASPlaceUserIDMask(this.getID()), AMETASMessage.UNSPECIFIED );
		// AMETASMessageMask mask = new AMETASMessageMask();
		Object aMask[] = { mask };

		try {
		        getDriver().registerEventListener( AMETASMessageEvent.MESSAGE_RECEIVED_EVENT, 0, aMask );
		} catch ( Exception curException ) {
		        getDriver().output("m_Driver.registerEventListener( AMETASMessageEvent.MESSAGE_RECEIVED_EVENT, 0, aMask );");
			return true;
		}

		try {
			getDriver().registerEventListener( AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT, 0, aMask );
		} catch ( Exception curException ) {
			getDriver().output("m_Driver.registerEventListener( AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT, 0, aMask );");
			return true;
		}

		return false;
	}

	/* Die Eventhandler Klasse fr den eventgesteuerten Empfang von Nachrichten
	und Meldungen der Stelle */
	public class CUAEventHandler extends AMETASEventHandler {

		/**
		 * Wird von AMETASNotifiableAgent.notifyListener beim Eintreffen eines Applicationereignisses aufgerufen.
		 */
		public boolean handleApplicationMessage (java.lang.String sSpec, java.lang.Object[] aobjBody, AMETASMessage mes) {

		        /* Message in Membervariable abspeichern */
			m_oMessage = mes;

			// geht nicht mehr so: m_frmMainFrame.messageOutput ("Message erhalten " + mes.getSpecification());

		        /* Prfen, ob Message wirklich in Kategorie AMETASMessage.APPLICATION */
		        if (mes.getCategory() == AMETASMessage.APPLICATION) {

				/* Diese Nachricht kommt von einem ausgesetzten StockAgent oder einem
				StockInfoAgent, wir speichern hier seine PlaceUser-ID ab, um ihm spter
				weitere Nachrichten schicken zu knnen */
				AMETASPlaceUserID objAgentID = mes.getSenderID();

				/* Von StockInfoAgents kommen Nachrichten vom Typ GO_NOTIFY rein */
				if (mes.getSpecification().equals("NOTIFICATION")) {

				        /* Handler fr diese Nachricht aufrufen und zurck */
					handleMessageGoNotify (mes);
				        return true;

				}


				/* "CDS_FOUND" - Nachricht von Agenten erhalten, jetzt mu die Nutzlast
				ausgwertet werden (also die CD-Daten) */
				//if (mes.getSpecification().equals("CDS_FOUND")) {

				        /* Anschlieend holen wir uns die SessionId aus der Nachrichten-
				        nutzlast (aobjBody), um den richtigen CDShopAgentWrapper
				        mit der Verarbeitung der Daten zu beauftragen. */
				//        String sSessionId = aobjBody[1].toString();

				        /* Instanz von CDShopAgentWrapper im Repository suchen, falls nichts
				        gefunden, dann Fehlermeldung */
				//        CDShopAgentWrapper objAgentWrapper = getCDShopAgentWrapper (sSessionId);
				//        if (objAgentWrapper == null) {

						/* Fehlermeldung ins Log ausgeben, weil kein passender CDShopAgentWrapper gefunden wurde */
				//		getDriver().output( "Nachricht CDS_FOUND von einem dem Adapter unbekannten Agenten erhalten (SessionId = " + sSessionId + ")" );

				//        } else {

						/* Statusmeldung ins Log ausgeben */
				//		getDriver().output( "Nachricht CDS_FOUND von Agent erhalten (SessionId = " + sSessionId + ")." );

						/* PUID des Agenten an objAgentWrapper bergeben */
				//		objAgentWrapper.setAgentId (objAgentID);

				                /* Nachrichtennutzlast an objAgentWrapper bergeben */
				//		objAgentWrapper.handleCdFoundList (aobjBody);

						/* wenn im GUI-Mode, dann wird m_frmMainFrame hier mitgeteilt, dass die
						die CD-Liste eingetroffen ist. Die Poll-Technik ist nur den Webmodus
						sinnvoll, im GUI-Modus ist Pushen anwendungsfreundlicher. */
				//	        if (m_iOperationMode == 0) {
				//			m_frmMainFrame.handleCdFoundList ();
				//		}
				//        }

				//        return true;
				//}

				/* "CDS_BOUGHT" - Nachricht von Agenten erhalten, jetzt mu die Nutzlast
				ausgwertet werden (also die Erfolgsmeldungen ber den CD-Kauf) */
				//if (mes.getSpecification().equals("CDS_BOUGHT")) {

				        /* Anschlieend holen wir uns die SessionId aus der Nachrichten-
				        nutzlast (aobjBody), um den richtigen CDShopAgentWrapper
				        mit der Verarbeitung der Daten zu beauftragen. */
				//        String sSessionId = aobjBody[1].toString();

				        /* Instanz von CDShopAgentWrapper im Repository suchen, falls nichts
				        gefunden, dann Fehlermeldung */
				//        CDShopAgentWrapper objAgentWrapper = getCDShopAgentWrapper (sSessionId);
				//        if (objAgentWrapper == null) {

						/* Fehlermeldung ins Log ausgeben, weil kein passender CDShopAgentWrapper gefunden wurde */
				//		getDriver().output( "Nachricht CDS_BOUGHT von einem dem Adapter unbekannten Agenten erhalten (SessionId = " + sSessionId + ")" );

				//        } else {

						/* Statusmeldung ins Log ausgeben */
				//		getDriver().output( "Nachricht CDS_BOUGHT von Agent erhalten (SessionId = " + sSessionId + ")." );

				                /* Nachrichtennutzlast an objAgentWrapper bergeben */
				//		objAgentWrapper.handleCdBoughtList (aobjBody);

						/* wenn im GUI-Mode, dann wird m_frmMainFrame hier mitgeteilt, dass
						die CD-Liste vom Agenten-Kauflauf eingetroffen ist. Die Poll-Technik ist
						nur den Webmodus sinnvoll, im GUI-Modus ist Pushen anwendungsfreundlicher. */
				//	        if (m_iOperationMode == 0) {
				//			m_frmMainFrame.handleCdBoughtList ();
				//		}
				//        }

				//        return true;
				//}

		        }

		        /* Auch wenn wir die Nachricht nicht verwerten knnten, liefern wir true zurck,
		        quasi ein Mllschluckerfeature ;-) */
		        return true;
		}

		/**
		 * Wird von AMETASNotifiableAgent.notifyListener beim Eintreffen eines Messageereignisses aufgerufen.
		 */
		public boolean handleMessageMessage (String sSpec, Object[] aBody, AMETASMessage mes) {
		        return handleApplicationMessage (sSpec, aBody, mes);
		}

		/**
		 * Wird von AMETASNotifiableAgent.notifyListener beim Eintreffen eines Stellenereignisses aufgerufen.
		 */
		public boolean handlePlaceEvent(AMETASPlaceEvent evt) {
		        if (evt.getID() == AMETASPlaceEvent.PLACE_SHUTDOWN_EVENT) {

				/* Fabian Wleklinski: Die Stelle wird in Krze beendet,
				eventuelle Aufrumaktionen mssen jetzt durchgefhr werden */
				getDriver().output( "Stelle wird in " + evt.getGraceTime() + " Millisekunden geschlossen." );

				m_oMessage = null;
				getDriver().wakeup();

				return true;
		        }
		        return false;
		}

	        /**
		 * Handler fr die Nachricht GO_NOTIFY, die von StockInfoAgent abgesetzt wird, aufrufen. Das
		 * Nachrichtenprotokoll sieht folgendermaen aus:
		 *
		 * "NOTIFICATION", AMETASPlaceUserID puidSender, CStockAgentMsgs samMsgs
		 *
		 * mit CStockAgentMsgs definiert in der entsprechenden Klasse im package ap.data.
		 */
		void handleMessageGoNotify (AMETASMessage mes) {

		        /* Die Nutzlast der Nachricht holen */
		        Object[] aobjBody = mes.getBody();

		        /* minimale Validittsprfung der Nachricht */
		        if (aobjBody.length != 3) {
				getDriver().output("Nachrichtenarray der Nachricht NOTIFICATION mu genau 3 Elemente haben. Verwerfe Nachricht.");
				return;
		        }

		        /* Einige Variablen als Zwischenspeicher fr die Nutzdaten der Nachricht */
		        AMETASPlaceUserID puidAgent = null;
		        Object[] aobjRawStockInfoValues = null;
		        CStockAgentMsgs objStockInfoValues = null;

		        /* Daten aus der Nachricht holen (Namen, Stellennamen und vor allem
		        die Bilanzdaten in einer ametas-sendable-serialisierten CStockAgentMsgs-Instanz */
		        try {
			        puidAgent = (AMETASPlaceUserID)aobjBody[1];
		                aobjRawStockInfoValues = (Object[])aobjBody[2];
		                objStockInfoValues = new CStockAgentMsgs();
				objStockInfoValues.setData (aobjRawStockInfoValues);
		        } catch (Exception e) {
				getDriver().output("Nachricht NOTIFICATION hat ungueltiges Format. Verwerfe Nachricht.");
				return;
		        }

		        /* Agenten mit diesem Namen in Wrapperliste suchen */
		        CStockAgentWrapper objThisAgentWrapper = m_vecAgentList.getElementByPuid (puidAgent);
		        if (objThisAgentWrapper == null) {

				getDriver().output( "Nachricht NOTIFICATION fr nicht im StockUA verwalteten Agenten \"" + puidAgent.toString() + "\" erhalten. Verwerfe Nachricht.");
				return;

			} else {

			        /* und diesen Wrapper mit den Daten aus objStockInfoValues updaten */
	        	        objThisAgentWrapper.setStockInfoValues(objStockInfoValues);

				/* entsprechende Tabellenzeile sowie Summenzeile updaten lassen */
				m_vecAgentList.updateSum ();
				m_frmMainFrame.updateRow(objThisAgentWrapper);

		        }

		}

	}

	/**
	 * Liefert die Agentenwrapperliste zurck
	 */
	public CStockAgentWrapperList getStockAgentWrapperList() {

		/* Die Membervariable zurckgeben */
		return m_vecAgentList;

	}

}